/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TemplateFilter.java,v 1.1 2001/09/28 16:01:06 cryd0221 Exp $
 */



//------------------------------------------------------------------------------
// TemplateFilter

package org.enhydra.dods.builder.engine;

import java.io.*;
import java.util.Enumeration;

public class TemplateFilter
{
    public static String skipStart = "__START_SKIPPING__";
    public static String skipStop  = "__STOP_SKIPPING__";

    private TemplateReader input;
    private Writer         output;

    //--------------------------------------------------------------------------
    /**
     *  TemplateFilter
     *    constructor
     *
     *  @param reader
     *    The input template with embedded tags to be translated.
     *
     *  @param writer
     *    The output text with tags translated to values.
     */
    public TemplateFilter( TemplateReader reader, Writer writer ) throws Exception {
        input  = reader;
	input.mark( 100000 );
        output = writer;
    }

    //--------------------------------------------------------------------------
    /**
     *  process
     *    Reads from input, writing to output,
     *    replacing the tags with values from the TagSet.
     *
     *    This method is called by the implementations of the abstract methods
     *       writeFileHeader
     *       writeFileFooter
     *       writeMemberStuff
     *    which are defined in classes derived from class SourceGenerator.
     *
     *  @param tagSet
     *    The TagSet containing a values for various tags.
     */
    public void process( TagSet tagSet ) throws Exception {
	String line;
	String tag;
	int    tagLen, index, skipDepth = 0;
//System.err.println("input.markSupported()="+input.markSupported());
	while ( null != ( line = input.readLine() ) ) {
/* System.err.println("line ="+line ); */
/* System.err.println("line.length()="+line.length()); */
	    Enumeration tags = tagSet.getTags();
	    while ( tags.hasMoreElements() ) {
	        tag = (String) tags.nextElement();
/* System.err.println("tag ="+tag ); */
		while ( -1 != ( index = line.indexOf( tag ) ) ) {
/* System.err.println("index ="+index ); */
		    tagLen = tag.length();
		    line = line.substring( 0, index )
			    + tagSet.getTagValue( tag )
			    + line.substring( index + tagLen, line.length() );
		}
/* System.err.println("line ="+line ); */
	    }
	    if ( -1 != line.indexOf( skipStart ) ) {
		skipDepth++;
/* System.err.println("skipDepth="+skipDepth); */
	    } else if ( -1 != line.indexOf( skipStop ) ) {
		if ( skipDepth > 0 )
		    skipDepth--;
/* System.err.println("skipDepth="+skipDepth); */
	    } else if ( skipDepth == 0 ) {
	        output.write( line + "\n" );
	    }
	}
/*         output.close(); */
/* System.err.println("PAST CLOSE" ); */
    }

    public void reset() throws Exception {
//System.err.println("reset: input.markSupported()="+input.markSupported());
	input.reset();
    }

    public void close() throws Exception {
	// This line fixes a problem a Linux user was seeing,
	// but caused other problems on Windows.
	input.close();
    }

}   // End of class TemplateFilter
