
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: SourceGenerator.java,v 1.1 2001/09/28 16:01:06 cryd0221 Exp $
 */




package org.enhydra.dods.builder.generator.base;

import org.enhydra.dods.store.doa.*;
import org.enhydra.dods.exceptions.*;
import org.enhydra.dods.builder.engine.*;

import com.lutris.util.Config;

import java.io.*;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Vector;
import java.text.*;

abstract public class SourceGenerator {

    /*
    String		templateDir;
    TemplateReader	templateFileHeader;
    TemplateReader	templateFileFooter;
    TemplateReader	templateMemberStuff;
    */
    protected GeneratorInfo	info;
    TemplateFilter		filterFileHeader;
    TemplateFilter		filterFileFooter;
    TemplateFilter		filterMemberStuff;

    protected TagSet		tagSet;
	//note, these are all constants, and as such they should probably be declared final... -csc
    static public String TEMPLATE_BEGIN_REMARK  = "TEMPLATE_BEGIN_REMARK";
    static public String TEMPLATE_END_REMARK    = "TEMPLATE_END_REMARK";
    static public String AUTHOR			= "AUTHOR";
    static public String PROJECT_NAME		= "PROJECT_NAME";
    static public String CATEGORY		= "CATEGORY";
    static public String SUFFIX			= "SUFFIX";
    static public String PACKAGE		= "PACKAGE";
    static public String CLASS_NAME		= "CLASS_NAME";
    static public String EXTENDS		= "EXTENDS";
    static public String TABLE_NAME		= "TABLE_NAME";
    static public String UPPER_TABLE_NAME = "UPPER_TABLE_NAME";		//added csc_052201
    static public String DB_VENDOR		= "DB_VENDOR";
    static public String FILE_NAME		= "FILE_NAME";
    static public String MEMBER_NAME		= "MEMBER_NAME";
    static public String UPPER_MEMBER_NAME	= "UPPER_MEMBER_NAME";
    static public String MEMBER_JAVA_TYPE	= "MEMBER_JAVA_TYPE";
    static public String MEMBER_JAVA_TYPE_SAFE_NAME= "MEMBER_JAVA_TYPE_SAFE_NAME";
    static public String MEMBER_JAVA_DEFAULT_VALUE= "MEMBER_JAVA_DEFAULT_VALUE";
    static public String MEMBER_JDBC_TYPE	= "MEMBER_JDBC_TYPE";
    static public String MEMBER_DB_TYPE		= "MEMBER_DB_TYPE";
    static public String JDBC_TYPE_CONSTANT	= "JDBC_TYPE_CONSTANT";
    static public String MEMBER_DB_DEFAULT_VALUE= "MEMBER_DB_DEFAULT_VALUE";
    static public String MEMBER_JAVADOC		= "MEMBER_JAVADOC";
    static public String DO_CLASS_NAME        	= "DO_CLASS_NAME";
    static public String DO_INTERFACE_NAME      = "DO_INTERFACE_NAME";
    static public String QUERY_CLASS_NAME     	= "QUERY_CLASS_NAME";
    static public String BASE_NAME     		= "BASE_NAME";

    static final String BEGIN_NONCONSTANT	= "BEGIN_NONCONSTANT";
    static final String END_NONCONSTANT		= "END_NONCONSTANT";
    static final String TRUE_IF_NOTNULL		= "TRUE_IF_NOTNULL";
    static final String MARK_NEW_VALUE_EXTRAS	= "MARK_NEW_VALUE_EXTRAS";
    static final String BIG_DECIMAL_SCALE	= "BIG_DECIMAL_SCALE";

    static final String DECL_MAX_LENGTH		= "DECL_MAX_LENGTH";
    final String defaultStringLength		= "32";

    static public String START_CONCRETE       	= "START_CONCRETE";
    static public String STOP_CONCRETE        	= "STOP_CONCRETE";

    static public String START_CACHING		= "START_CACHING";
    static public String STOP_CACHING		= "STOP_CACHING";
    static public String START_FULL_CACHING	= "START_FULL_CACHING";
    static public String STOP_FULL_CACHING	= "STOP_FULL_CACHING";
    static public String TRUE_IF_PARTIAL_CACHING= "TRUE_IF_PARTIAL_CACHING";
    static public String TRUE_IF_FULL_CACHING	= "TRUE_IF_FULL_CACHING";
    static public String TRUE_IF_NO_CACHING	= "TRUE_IF_NO_CACHING";

    static public String DATABASE_SECTION       = "Database";

    static public String STRING_QUOTE_CHARACTER	= "STRING_QUOTE_CHARACTER";
    static public String STRING_MATCH		= "STRING_MATCH";
    static public String STRING_WILDCARD	= "STRING_WILDCARD";

    static public String BEGIN_COMMENT		= "BEGIN_COMMENT";
    static public String END_COMMENT		= "END_COMMENT";


    static public String VERSION_DB_COL_NAME	= "VERSION_DB_COL_NAME";
    static public String OID_DB_COL_NAME	= "OID_DB_COL_NAME";
    static public String OID_DB_TYPE		= "OID_DB_TYPE";
    static public String OID_JDBC_TYPE		= "OID_JDBC_TYPE";
    static public String INT_DB_TYPE		= "INT_DB_TYPE";

    static public String NAMED_CONSTRAINTS	= "NAMED_CONSTRAINTS";
    static public String DELETE_CASCADES	= "DELETE_CASCADES";

    static public String START_DO_IS_OID_BASED	= "START_DO_IS_OID_BASED";
    static public String END_DO_IS_OID_BASED	= "END_DO_IS_OID_BASED";
    static public String START_DO_IS_NOT_OID_BASED= "START_DO_IS_NOT_OID_BASED";
    static public String END_DO_IS_NOT_OID_BASED= "END_DO_IS_NOT_OID_BASED";

    static public String START_MEMBER_PK_NOT_OID= "START_MEMBER_PK_NOT_OID";
    static public String END_MEMBER_PK_NOT_OID  = "END_MEMBER_PK_NOT_OID";

    static final String CREATE_PARM_SIG  = "CREATE_PARM_SIG";
    static final String CREATE_PARM_LIST = "CREATE_PARM_LIST";
    String parmSig = "";
    String parmList = "";
    String comma = "";
    static final String PK_PK_SET_CALLS = "PK_PK_SET_CALLS";
    String pkpkSetCalls = "";
    static final String DO_PK_SET_CALLS = "DO_PK_SET_CALLS";
    String dopkSetCalls = "";
    static final String PK_DO_SET_CALLS = "PK_DO_SET_CALLS";
    String pkdoSetCalls = "";
    static final String QUERY_PK_SET_CALLS = "QUERY_PK_SET_CALLS";
    String querypkSetCalls = "";

    // tags set by command line options
    static final String START_JDBC1		= "START_JDBC1";
    static final String END_JDBC1		= "END_JDBC1";
    static final String START_JDBC2		= "START_JDBC2";
    static final String END_JDBC2		= "END_JDBC2";
    static final String START_ENHYDRA4		= "START_ENHYDRA4";
    static final String END_ENHYDRA4		= "END_ENHYDRA4";
    static final String START_ENHYDRA3		= "START_ENHYDRA3";
    static final String END_ENHYDRA3		= "END_ENHYDRA3";
    static final String START_NO_ENTITY_BEAN	= "START_NO_ENTITY_BEAN";
    static final String END_NO_ENTITY_BEAN	= "END_NO_ENTITY_BEAN";
    static final String START_ENTITY_BEAN	= "START_ENTITY_BEAN";
    static final String END_ENTITY_BEAN	= "END_ENTITY_BEAN";

    // tags for stored procedures
    static final String START_STORED_PROC	= "START_STORED_PROC";
    static final String END_STORED_PROC		= "END_STORED_PROC";
    static final String START_NOT_STORED_PROC	= "START_NOT_STORED_PROC";
    static final String END_NOT_STORED_PROC	= "END_NOT_STORED_PROC";
    //
    static final String START_HAS_RESULTSET	= "START_HAS_RESULTSET";
    static final String END_HAS_RESULTSET	= "END_HAS_RESULTSET";
    static final String START_NOT_HAS_RESULTSET	= "START_NOT_HAS_RESULTSET";
    static final String END_NOT_HAS_RESULTSET	= "END_NOT_HAS_RESULTSET";
    //
    static final String START_NOT_SP_PARAM	= "START_NOT_SP_PARAM";
    static final String END_NOT_SP_PARAM	= "END_NOT_SP_PARAM";
    //
    static final String START_SP_IN_PARAM	= "START_SP_IN_PARAM";
    static final String END_SP_IN_PARAM		= "END_SP_IN_PARAM";
    static final String START_NOT_SP_IN_PARAM	= "START_NOT_SP_IN_PARAM";
    static final String END_NOT_SP_IN_PARAM	= "END_NOT_SP_IN_PARAM";
    //
    static final String START_SP_OUT_PARAM	= "START_SP_OUT_PARAM";
    static final String END_SP_OUT_PARAM	= "END_SP_OUT_PARAM";
    static final String START_NOT_SP_OUT_PARAM	= "START_NOT_SP_OUT_PARAM";
    static final String END_NOT_SP_OUT_PARAM	= "END_NOT_SP_OUT_PARAM";
    //
    static final String START_RESULTSET_VALUE	= "START_RESULTSET_VALUE";
    static final String END_RESULTSET_VALUE	= "END_RESULTSET_VALUE";
    static final String START_NOT_RESULTSET_VALUE= "START_NOT_RESULTSET_VALUE";
    static final String END_NOT_RESULTSET_VALUE	= "END_NOT_RESULTSET_VALUE";

    // Tuples used to copy config section values to tag values.
    static final String[][] configSectionName_tagName_pairs = {
        {    "StringQuoteCharacter",	STRING_QUOTE_CHARACTER },
        {    "StringMatch",		STRING_MATCH },
        {    "StringWildcard",		STRING_WILDCARD },
        {    "CommentStart",		BEGIN_COMMENT },
        {    "CommentEnd",		END_COMMENT },
        {    "OidDbColumnName",		OID_DB_COL_NAME },
        {    "VersionDbColumnName",	VERSION_DB_COL_NAME },
        {    "OidDbType",		OID_DB_TYPE },
        {    "OidJdbcType",		OID_JDBC_TYPE },
        {    "NamedConstraint",		NAMED_CONSTRAINTS },
        {    "DeleteCascade",		DELETE_CASCADES },
    };


    private void setTagsFromConfig( Config config, String vendor )
    throws Exception {
//System.err.println("using vendor="+vendor );
        for ( int i = 0; i < configSectionName_tagName_pairs.length; i++ ) {
            String[] pair =  configSectionName_tagName_pairs[i];
            setTagFromConfig( config, pair[0], vendor, pair[1] );
        }
    }

    private String setTagFromConfig(    Config config,
                                        String sectionName, String vendor,
                                        String tag )
    throws Exception {
        String tagValue = null;
        try {
//System.err.println("sectionName ="+sectionName );
            Config section = (Config) config.getSection( sectionName );
//System.err.println("vendor ="+vendor );
            tagValue = section.getString( vendor );
//System.err.println("tagValue ="+tagValue );
            if ( null != tag )
                tagSet.setTagValue( tag, tagValue );
        } catch ( Exception e ) {
            Builder.print("No dods.conf value for " +
                            sectionName + "." + vendor );
            throw e;
        }
        return tagValue;
    }

    protected String val( String tagName )
    throws Exception {
        return tagSet.getTagValue( tagName );
    }

    static final String BEGIN_IS_REF		= "BEGIN_IS_REF";
    static final String END_IS_REF		= "END_IS_REF";

    protected String		db_vendor;
    protected Config 		db 		= null;
    protected Config 		jdbcConfig	= null;
    protected String 		vendor_key	= null;

    protected DataObject	currentObj;

    protected SourceWriter	output;

    protected boolean 	    	cascadeDelete =	false;
    protected boolean 	    	namedConstraint = false;

    static final String FILE_HEADER_TEMPLATE_NAME  = "FileHeader";
    static final String FILE_FOOTER_TEMPLATE_NAME  = "FileFooter";
    static final String MEMBER_STUFF_TEMPLATE_NAME = "MemberStuff";

    /* Builder.generateSQLcommands() uses oidDbType */
    public    static String oidDbType = null;

    protected static boolean columnIsText( Attribute a ) {
        return columnIsText( a.getDBType() );
    }
    protected static boolean columnIsText( String columnType ) {
        return "LONGVARCHAR".equals( columnType );
    }

    protected TemplateFilter createFilter( String templateKey )
    throws Exception {
        TemplateReader reader = info.getTemplateReader( templateKey );
        try {
            /*
            reader.mark( 100000 );
            reader.reset();
            */
        } catch ( Exception e ) {
          Builder.print("CANNOT RESET READER: " + e.getMessage() );
        }
        return new TemplateFilter( reader, output );
    }

    static private Hashtable fixedDefaults = null;

    static public void reset() {
        fixedDefaults = new Hashtable();
//csc_071101_start - do this to guarantee the vendor stuff gets reloaded on subsequent runs (see bug 11863)
		dbTypes = new Hashtable();
//csc_071101_end
    }

   /**
     * Adds tag values to the <CODE>TagSet</CODE>.
     * A subclass of <CODE>SourceGenerator</CODE> can override this method
     * to set additional tag values.
     * An overrided <CODE>objectPrep()</CODE>
     * should invoke <CODE>super.objectPrep()</CODE>
     * to get the basic tag settings performed by this method.
     * A subclass of <CODE>SourceGenerator</CODE>
     * will call <CODE>objectPrep()</CODE>
     * and then call its implementation of one of the following methods:
     *    <CODE>
     *    writeFileHeader
     *    writeFileFooter
     *    writeMemberStuff
     *    </CODE>
     * These methods should invoke
     *    <CODE>TemplateFilter.process( tagSet )</CODE>
     * which will create the output source file.
     *
     * @param o
     *   DataObject for which source code will be generated by
     *    <CODE>writeFileHeader</CODE> and
     *    <CODE>writeFileFooter</CODE>
     * @exception Exception
     *   If an unrecoverable error occurs.
     *
     * @author    Jay Gunter
     * @since     LBS1.8
     * @version   $Revision: 1.1 $
     *
     */

    public void objectPrep(	GeneratorInfo	sgi,
                                DODSProject	project,
                                DataObject	o )
    throws Exception {
        info = sgi;
        currentObj = o;
        String flavor = info.outputFileFlavor;
        if ( flavor.equals( "Remote" ) )
                flavor = "";
        String suffix = info.outputFileSuffix;
        tagSet = new TagSet();
        tagSet.setTagValue( TEMPLATE_BEGIN_REMARK, TemplateFilter.skipStart );
        tagSet.setTagValue( TEMPLATE_END_REMARK,   TemplateFilter.skipStop );
//	tagSet.setTagValue( CATEGORY,  flavor );
        tagSet.setTagValue( SUFFIX,    suffix );
        tagSet.setTagValue( BASE_NAME, o.getName() );
        tagSet.setTagValue( CLASS_NAME, getOutputClassName( o, flavor ) );
        DataObject parentClass = o.getExtensionOf();

        tagSet.setTagValue( TRUE_IF_PARTIAL_CACHING, "false" );
        tagSet.setTagValue( TRUE_IF_FULL_CACHING,    "false" );
        tagSet.setTagValue( TRUE_IF_NO_CACHING,      "true" );

        if ( o.isCaching() && o.isConcrete() && ! o.isStoredProc() ) {
            tagSet.setTagValue( TRUE_IF_NO_CACHING,      "false" );
            tagSet.setTagValue( TRUE_IF_PARTIAL_CACHING, "true" );
            tagSet.setTagValue( START_CACHING,     "" );
            tagSet.setTagValue( STOP_CACHING,      "" );
        } else {
            tagSet.setTagValue( START_CACHING,     TemplateFilter.skipStart );
            tagSet.setTagValue( STOP_CACHING,      TemplateFilter.skipStop );
        }

        if ( o.isFullCaching() && o.isConcrete() && ! o.isStoredProc() ) {
            tagSet.setTagValue( TRUE_IF_NO_CACHING,      "false" );
            tagSet.setTagValue( TRUE_IF_PARTIAL_CACHING, "false" );
            tagSet.setTagValue( TRUE_IF_FULL_CACHING,    "true" );
            tagSet.setTagValue( START_FULL_CACHING,     "" );
            tagSet.setTagValue( STOP_FULL_CACHING,      "" );
        } else {
            tagSet.setTagValue( START_FULL_CACHING,     TemplateFilter.skipStart );
            tagSet.setTagValue( STOP_FULL_CACHING,      TemplateFilter.skipStop );
        }

        if ( o.isConcrete() ) {
            tagSet.setTagValue( START_CONCRETE,     "" );
            tagSet.setTagValue( STOP_CONCRETE,      "" );
        } else {
            tagSet.setTagValue( START_CONCRETE,     TemplateFilter.skipStart );
            tagSet.setTagValue( STOP_CONCRETE,      TemplateFilter.skipStop );
        }

        // If the Data Object does not derive from another Data Object,
        // and the OutputClassParent setting is not set for this SourceGenerator
        // then the output class does not need an "extends" clause.
        tagSet.setTagValue( EXTENDS, "" );
        if ( null != info.outputClassParent ) {
            // If a Data Object does not derive from another Data Object,
            // but an OutputClassParent is specified in the config file,
            // the extends clause names that parent class.
            tagSet.setTagValue( EXTENDS,
                "extends " + info.outputClassParent );
        }
        if ( null != parentClass ) {
            // If a Data Object Y derives from another Data Object X,
            // the extends clause must reflect this.
            Builder.print("DEPRECATION WARNING: derived DOs will use " +
                          "a different schema in the future." );
            tagSet.setTagValue( EXTENDS,
                    "extends " + parentClass.getAdjustedPackageName() + "." +
                    getOutputClassName( parentClass, flavor ) );
        }

        // user-defined primary keys means DO does not use oid/version columns.
        if ( 0 < o.getPrimaryKeys().length || o.notUsingOId() || o.isStoredProc() ) {
            tagSet.setTagValue(START_DO_IS_OID_BASED,TemplateFilter.skipStart);
            tagSet.setTagValue(END_DO_IS_OID_BASED,  TemplateFilter.skipStop);
            tagSet.setTagValue(START_DO_IS_NOT_OID_BASED,"");
            tagSet.setTagValue(END_DO_IS_NOT_OID_BASED,  "");
        } else {
            tagSet.setTagValue(START_DO_IS_OID_BASED,"");
            tagSet.setTagValue(END_DO_IS_OID_BASED,  "");
            tagSet.setTagValue(START_DO_IS_NOT_OID_BASED,TemplateFilter.skipStart);
            tagSet.setTagValue(END_DO_IS_NOT_OID_BASED,  TemplateFilter.skipStop);
        }

        //db_vendor = project.getDatabase();
//        db_vendor = System.getProperty( DB_VENDOR, project.getDatabase() );
        db_vendor = System.getProperty( Builder.OPT_VENDOR, project.getDatabase() );
        tagSet.setTagValue( DB_VENDOR,    db_vendor );
        tagSet.setTagValue( TABLE_NAME,   getTableName( o ) );

		//csc_052201_start
		String tableStr = getTableName(o);
		if (tableStr!=null && tableStr.trim().length()>0) {
			tableStr = tableStr.substring(0,1).toUpperCase()+tableStr.substring(1);
		}
        tagSet.setTagValue(UPPER_TABLE_NAME, tableStr);
		//csc_052201_end

        tagSet.setTagValue( PACKAGE,      o.getAdjustedPackageName() );
        tagSet.setTagValue( AUTHOR,       System.getProperty( "user.name" ) );
        tagSet.setTagValue( PROJECT_NAME, project.getName() );

        /*
         * START/END_JDBC1/2 sections occur inside START/END_ENHYDRA4 sections
         * in the DO templates.
         * These sections determine how the generated DO and Query classes
         * obtain Connection objects.
         *
         * These sections make the DO and Query classes useful
         * to applications that do not wish to use the Enhydra 3-style
         * framework (EAF).
         *
         * See Builder.main for notes on the interaction of
         * various command line options for DODS.
         */
        boolean jdbc1 =	Builder.getBoolProp( Builder.PROP_JDBC1 );
        boolean jdbc2 =	Builder.getBoolProp( Builder.PROP_JDBC2 );
        // ensure jdbc1 or jdbc2 is true.
        if (   jdbc1 &&   jdbc2 ) jdbc1 = false;  // if both are set, choose 2
        if ( ! jdbc1 && ! jdbc2 ) jdbc1 = true;   // if neither is set, choose 1
//System.err.println("jdbc2 ="+jdbc2 );
        tagSet.setTagValue( START_JDBC1, beginNoCodeIf( ! jdbc1 ) );
        tagSet.setTagValue( END_JDBC1,   endNoCodeIf(   ! jdbc1 ) );
        tagSet.setTagValue( START_JDBC2, beginNoCodeIf( ! jdbc2 ) );
        tagSet.setTagValue( END_JDBC2,   endNoCodeIf(   ! jdbc2 ) );

        boolean enhydra3 = Builder.getBoolProp( Builder.PROP_ENHYDRA3 );
        boolean enhydra4 = Builder.getBoolProp( Builder.PROP_ENHYDRA4 );
        if (   enhydra3 &&   enhydra4 ) enhydra4 = false;  // if both, use e3
        if ( ! enhydra3 && ! enhydra4 ) enhydra4 = true;   // if neither, use e4
//System.err.println("enhydra4 ="+enhydra4 );
        tagSet.setTagValue( START_ENHYDRA3, beginNoCodeIf( ! enhydra3 ) );
        tagSet.setTagValue( END_ENHYDRA3,   endNoCodeIf(   ! enhydra3 ) );
        tagSet.setTagValue( START_ENHYDRA4, beginNoCodeIf( ! enhydra4 ) );
        tagSet.setTagValue( END_ENHYDRA4,   endNoCodeIf(   ! enhydra4 ) );

        // If the DO is supporting a bean-managed entity bean,
        // we do not generate N21_REFERRERS and N2N_REFERRERS (etc.)
        // because these methods hit the database directly
        // which is not appropriate for the EJB client to do.
        boolean ejb = Builder.getBoolProp( Builder.PROP_EJB );
        tagSet.setTagValue( START_NO_ENTITY_BEAN, beginNoCodeIf( ejb ) );
        tagSet.setTagValue( END_NO_ENTITY_BEAN,   endNoCodeIf( ejb ) );
        tagSet.setTagValue( START_ENTITY_BEAN, beginNoCodeIf(! ejb ) );
        tagSet.setTagValue( END_ENTITY_BEAN,   endNoCodeIf(! ejb ) );

        // If a DataObject represents a stored procedure,
        // 	the Query class will have
        //		set() methods for all the IN parameters,
        //		get() methods for all the OUT parameters,
        //				including the return value if any.
        //		If all of the fields are IN or OUT parameters,
        //		then the Query will not have a getNextDO() method.
        // 	the DO class will have just
        //		createExisting(ResultSet) and
        //		get() methods for any ResultSet fields.
        //		No createVirgin(), delete(), save(), etc.
        //
        boolean isSP = o.isStoredProc();
        tagSet.setTagValue( START_STORED_PROC,     beginNoCodeIf( ! isSP ) );
        tagSet.setTagValue( END_STORED_PROC,       endNoCodeIf(   ! isSP ) );
        tagSet.setTagValue( START_NOT_STORED_PROC, beginNoCodeIf(   isSP ) );
        tagSet.setTagValue( END_NOT_STORED_PROC,   endNoCodeIf(     isSP ) );
        boolean hasRS = o.hasResultSet();
        tagSet.setTagValue( START_HAS_RESULTSET,     beginNoCodeIf(! hasRS ) );
        tagSet.setTagValue( END_HAS_RESULTSET,       endNoCodeIf(  ! hasRS ) );
        tagSet.setTagValue( START_NOT_HAS_RESULTSET, beginNoCodeIf(  hasRS ) );
        tagSet.setTagValue( END_NOT_HAS_RESULTSET,   endNoCodeIf(    hasRS ) );

        // FIX the strings "DO" and "Query" should really come from
        // the dods.conf file.  These suffixes should not be hard-coded here.
        tagSet.setTagValue( DO_CLASS_NAME,	o.getName() + "DO" );
        tagSet.setTagValue( DO_INTERFACE_NAME,	o.getName() + "DOI" );
        tagSet.setTagValue( QUERY_CLASS_NAME,	o.getName() + "Query" );

        // set tag values and variables from config file values
        db = (Config) info.config.getSection( DATABASE_SECTION );
        jdbcConfig = (Config)  db.getSection( "JDBCtype" );
        vendor_key = db_vendor.replace( ' ', '_' );
        setTagsFromConfig( db, vendor_key );
        tagSet.setTagValue( INT_DB_TYPE,
                    adjustDbType( jdbcConfig, "INTEGER", vendor_key  ) );
        oidDbType = 		val( OID_DB_TYPE );
        cascadeDelete =		val( DELETE_CASCADES   ).equals( "true" );
        namedConstraint =	val( NAMED_CONSTRAINTS ).equals( "true" );

        // prepare the output file (the source generator product)
        outputFilename =	info.outputDir + File.separator +
                                o.getName() + flavor + "." + suffix;
        Builder.print("Generating "+outputFilename );
        output = new SourceWriter( outputFilename );
        tagSet.setTagValue( FILE_NAME, fixFileName( outputFilename ) );
        filterFileHeader  = createFilter( GeneratorInfo.WRITE_FILE_HEADER );
        filterFileFooter  = createFilter( GeneratorInfo.WRITE_FILE_FOOTER );
        filterMemberStuff = createFilter( GeneratorInfo.WRITE_MEMBER_STUFF );

        setOutputFilename( outputFilename );
    }

    private String outputFilename = null;
    private void setOutputFilename( String ofn ) {
        outputFilename = ofn;
    }

    public String getOutputFilename() {
        return outputFilename;
    }

    private String fixFileName( String s ) {
        /*
            On Windows platforms, it is common to get filenames like
                C:/usr/...
            except the forward slashes are backslashes.
            When javacc sees the pattern \ followed by u
            it tries to interpret it as a Unicode character
            (even if it occurs in a comment)
            and because it was not valid Unicode, you got an error.
         */
        return s.replace( '\\', '/' );
    }

    private String getOutputClassName( DataObject o, String flavor ) {
        return o.getName() + flavor;
    }

    static Hashtable tableNames = new Hashtable();
    private String getTableName( DataObject o )
    throws Exception {
        //String name  = o.getName().toLowerCase().trim();
        String name  = o.getDBTableName();
        return name;
    }

    public void memberPrep(	Attribute a ) throws Exception {
        String name = a.getName();
        String type = a.getJavaType();
        tagSet.setTagValue( MEMBER_NAME,       name );
        String firstChar = name.substring( 0, 1 ).toUpperCase();
        name = firstChar + name.substring( 1 );
        tagSet.setTagValue( UPPER_MEMBER_NAME,		name );
        tagSet.setTagValue( MEMBER_JAVA_TYPE,		type );

        // FIX if only SourceGenerator_DO needs MEMBER_JAVA_TYPE_SAFE_NAME
        // then move this line to SourceGenerator_DO.memberPrep().
/* 	tagSet.setTagValue( MEMBER_JAVA_TYPE_SAFE_NAME,	JavaType( type ) ); */
        if ( a.isObjectRef() && ! a.isForeignKey() )
            tagSet.setTagValue( MEMBER_JAVA_TYPE_SAFE_NAME, "DO" );
        else
            tagSet.setTagValue( MEMBER_JAVA_TYPE_SAFE_NAME, JavaType( type ) );

        tagSet.setTagValue( MEMBER_JAVA_DEFAULT_VALUE,  fixDefault( a ) );
        tagSet.setTagValue( MEMBER_JDBC_TYPE,		JdbcType( type ) );

        tagSet.setTagValue( BEGIN_NONCONSTANT, beginNoCodeIf( a.isConstant() ));
        tagSet.setTagValue( END_NONCONSTANT,   endNoCodeIf(   a.isConstant() ));
        tagSet.setTagValue( TRUE_IF_NOTNULL,
                a.canBeNull() ? "false" : "true" );

        /* For a BigDecimal field, the JDBC getBigDecimal method
         * takes an extra 'scale' parameter.
         * FIX the GUI should prompt for this scale value,
         * and the Attribute class should store it.
         * For now we use the Attribute size.
         */
        String strSize = a.getSize();
        if ( a.getJavaType().equals( "java.math.BigDecimal" ) ) {
            if ( null == strSize || 0 == strSize.length() )
                tagSet.setTagValue( BIG_DECIMAL_SCALE, ", 0" );
            else
                tagSet.setTagValue( BIG_DECIMAL_SCALE, ", " + strSize );
        } else {
                tagSet.setTagValue( BIG_DECIMAL_SCALE, "" );
        }
        if ( a.isObjectRef() && ! a.isForeignKey() )
                tagSet.setTagValue( BIG_DECIMAL_SCALE, ", 0" );

        /* For a String field, the GenericDO.markNewValue() method
         * takes some extra parameters that validate max length
         * and whether null strings are allowed.
         */
        String markNewValue_extraParms = "";
        String valueMaxLength = "";
        String declMaxLength = "";
        if ( a.getJavaType().equals( "String" ) ) {
            String dbtype = a.getDBType();
            if ( dbtype.equals( "LONGVARCHAR" ) )
                valueMaxLength = strSize = "0";
            else {
                if ( null == strSize || 0 == strSize.length() ) {
                    Builder.print("WARNING: String Attribute '" + a.getName()
                            + "' size defaulting to " + defaultStringLength );
                    strSize = defaultStringLength;
                }
                valueMaxLength = a.getName() + "_MaxLength";
                declMaxLength  = "static public final int " +
                    valueMaxLength + " = " + strSize + ";";
            }
            markNewValue_extraParms = ", 0, " + valueMaxLength + ", " +
                ( a.canBeNull() ? "true" : "false" );
        }
        tagSet.setTagValue( MARK_NEW_VALUE_EXTRAS, markNewValue_extraParms );
        tagSet.setTagValue( DECL_MAX_LENGTH, declMaxLength );

        tagSet.setTagValue( BEGIN_IS_REF, beginNoCodeIf( ! a.isObjectRef() ));
        tagSet.setTagValue( END_IS_REF,   endNoCodeIf(   ! a.isObjectRef() ));

        tagSet.setTagValue( JDBC_TYPE_CONSTANT, a.getDBType() );
        if ( a.isObjectRef() && ! a.isForeignKey() ) {
//System.out.println ("Setting MEMBER_DB_TYPE @pt1:");
            tagSet.setTagValue( MEMBER_DB_TYPE,	oidDbType );
        } else {
//System.out.println ("Setting MEMBER_DB_TYPE @pt2:");
            tagSet.setTagValue( MEMBER_DB_TYPE, adjustDbType(db, a.getDBType(), vendor_key ) );
		}
//System.out.println ("Setting MEMBER_DB_TYPE = "+tagSet.getTagValue( MEMBER_DB_TYPE ));


        tagSet.setTagValue( MEMBER_DB_DEFAULT_VALUE,	a.getInitialValue() );
        tagSet.setTagValue( MEMBER_JAVADOC,		a.getJavadoc() );

        // members that are foreign keys to primary keys marked by the user
        // are handled differently than oid references.
        if ( a.isPrimaryKey() ) {
            tagSet.setTagValue( START_MEMBER_PK_NOT_OID, "" );
            tagSet.setTagValue( END_MEMBER_PK_NOT_OID, "" );
        } else {
            tagSet.setTagValue( START_MEMBER_PK_NOT_OID,
                                TemplateFilter.skipStart);
            tagSet.setTagValue( END_MEMBER_PK_NOT_OID,
                                TemplateFilter.skipStop);
        }

        parmSig  += comma + a.getJavaType() + " " + a.getName();
        parmList += comma + a.getName();
        comma = ", ";
//csc_052301_start - fix bug 11292
//        pkpkSetCalls += "PK.set" + name + "( " + a.getName() + " );\n";
//        dopkSetCalls += "DO.set" + name + "( " + a.getName() + " );\n";
//        pkdoSetCalls += "DO.set" + name + "( PK.get" + name + "() );\n";
//        querypkSetCalls = "q.setQuery" + name + "( PK.get" + name + "() );\n";
        dopkSetCalls += "DO.set" + name + "( " + a.getName() + " );\n";
        if (a.isPrimaryKey()) {
			pkpkSetCalls += "PK.set" + name + "( " + a.getName() + " );\n";
//csc_11900			pkdoSetCalls += "DO.set" + name + "( PK.get" + name + "() );\n";
			pkdoSetCalls += "PK.set" + name + "( DO.get" + name + "() );\n";	//csc_11900
        	querypkSetCalls += "q.setQuery" + name + "( PK.get" + name + "() );\n";
		}
//csc_052301_end


        // tags to include/exclude stored procedure handling stuff
        boolean isSpRs = a.isResultSetValue();
        tagSet.setTagValue( START_RESULTSET_VALUE,     beginNoCodeIf(!isSpRs) );
        tagSet.setTagValue( END_RESULTSET_VALUE,       endNoCodeIf(  !isSpRs) );
        tagSet.setTagValue( START_NOT_RESULTSET_VALUE, beginNoCodeIf( isSpRs) );
        tagSet.setTagValue( END_NOT_RESULTSET_VALUE,   endNoCodeIf(   isSpRs) );
        boolean isSpIn = a.isSpInParam();
        tagSet.setTagValue( START_SP_IN_PARAM,       beginNoCodeIf(!isSpIn) );
        tagSet.setTagValue( END_SP_IN_PARAM,         endNoCodeIf(  !isSpIn) );
        tagSet.setTagValue( START_NOT_SP_IN_PARAM,   beginNoCodeIf( isSpIn) );
        tagSet.setTagValue( END_NOT_SP_IN_PARAM,     endNoCodeIf(   isSpIn) );
        boolean isSpOut = a.isSpOutParam();
        tagSet.setTagValue( START_SP_OUT_PARAM,      beginNoCodeIf(!isSpOut) );
        tagSet.setTagValue( END_SP_OUT_PARAM,        endNoCodeIf(  !isSpOut) );
        tagSet.setTagValue( START_NOT_SP_OUT_PARAM,  beginNoCodeIf( isSpOut) );
        tagSet.setTagValue( END_NOT_SP_OUT_PARAM,    endNoCodeIf(   isSpOut) );
        boolean isSpParam = isSpIn || isSpOut;
        tagSet.setTagValue( START_NOT_SP_PARAM,      beginNoCodeIf( isSpParam));
        tagSet.setTagValue( END_NOT_SP_PARAM,        endNoCodeIf(   isSpParam));
    }


//csc_071101_start
//    private static Hashtable dbTypes = new Hashtable();
    private static Hashtable dbTypes = null;
//csc_071101_end

    private String adjustDbType(Config config, String jdbcType, String vendor ) throws Exception {
//System.out.println ("Adjusting DB Type--> jdbcType:"+jdbcType+", vendor:"+vendor);
//System.err.println("jdbcType="+jdbcType);
		if ( jdbcType.equals( "VARCHAR2" ) ) {
			jdbcType = "VARCHAR";
		} else if ( jdbcType.startsWith( "NUMERIC" ) ) {
			jdbcType = "INTEGER";
		} else if ( jdbcType.startsWith( "DECIMAL" ) ) {
			jdbcType = "INTEGER";
		}
        String vendorType = jdbcType;
        String x = (String) dbTypes.get( jdbcType );

        if ( null != x ) {
            vendorType = x;
        } else if ( null != jdbcConfig ) {
            Config typeConfig = (Config) jdbcConfig.getSection( jdbcType );
            if ( null != typeConfig ) {
                vendorType = typeConfig.getString( vendor );
                dbTypes.put( jdbcType, vendorType );
            }
        }
        return vendorType;
    }



    private String fixDefault( Attribute a ) {
        String type  = a.getJavaType();
        String value = a.getInitialValue();
        // FIX handle arrays
        if ( null == value ) value = "";
        String fixedValue = value;
        String defaultValue = null;
        if ( type.endsWith( "DO" ) )
            fixedValue = "null";	// no default value allowed for DO refs
        else if ( type.equals( "String" ) ) {
            if ( fixedValue.length() == 0 )
                fixedValue = "null";
            else
                fixedValue = "\"" + value + "\"";
        } else if ( type.equals( "java.math.BigDecimal" ) ) {
            if ( value.length() == 0 )
                fixedValue = "null";
            else {
                try {
                    java.math.BigDecimal test = new java.math.BigDecimal( fixedValue );
                    fixedValue = "new " + type + "(" + fixedValue + ")";
                } catch ( Exception e ) {
                    defaultValue = "null";
                }
            }
        } else if ( type.equals( "java.sql.Date" ) ) {
            if ( value.length() == 0 )
                fixedValue = "null";
            else {
                try {
/* 		    java.util.Date d = new java.util.Date( fixedValue ); */
                    java.util.Date d = str2date( fixedValue );
                    java.sql.Date test = new java.sql.Date( d.getTime() );
                    fixedValue = "new " + type + "(" + fixedValue + ")";
                } catch ( Exception e ) {
                    defaultValue = "null";
                }
            }
        } else if ( type.equals( "java.sql.Time" ) ) {
            if ( value.length() == 0 )
                fixedValue = "null";
            else {
                try {
                    java.util.Date d = str2date( fixedValue );
                    java.sql.Time test = new java.sql.Time( d.getTime() );
                    fixedValue = "new " + type + "(" + fixedValue + ")";
                } catch ( Exception e ) {
                    defaultValue = "null";
                }
            }
        } else if ( type.equals( "java.sql.Timestamp" ) ) {
            if ( value.length() == 0 )
                fixedValue = "null";
            else {
                try {
                    java.util.Date d = str2date( fixedValue );
                    java.sql.Timestamp test = new java.sql.Timestamp( d.getTime() );
                    fixedValue = "new " + type + "(" + fixedValue + ")";
                } catch ( Exception e ) {
                    defaultValue = "null";
                }
            }
        } else if ( type.equals( "int" ) ) {
            if ( value.length() == 0 )
                fixedValue = "0";
            else {
                try {
                    int i = Integer.parseInt( value );
                } catch ( Exception e ) {
                    defaultValue = "0";
                }
            }
        } else if ( type.equals( "boolean" ) ) {
            if ( value.length() == 0 )
                fixedValue = "false";
            else {
                try {
                    if ( Boolean.valueOf( value ).booleanValue() )
                        fixedValue = "true";
                    else
                        fixedValue = "false";
                } catch ( Exception e ) {
                    defaultValue = "false";
                }
            }
        } else if ( type.equals( "float" ) ) {
            if ( value.length() == 0 )
                fixedValue = "0";
            else {
                try {
                    Float.valueOf( value );
                } catch ( Exception e ) {
                    defaultValue = "0";
                }
            }
        } else if ( type.equals( "byte" ) ) {
            if ( value.length() == 0 )
                fixedValue = "0";
            else {
                try {
                    Byte.parseByte( value );
                } catch ( Exception e ) {
                    defaultValue = "0";
                }
            }
        } else if ( type.equals( "short" ) ) {
            if ( value.length() == 0 )
                fixedValue = "0";
            else {
                try {
                    Short.parseShort( value );
                } catch ( Exception e ) {
                    defaultValue = "0";
                }
            }
        } else if ( type.equals( "long" ) ) {
            if ( value.length() == 0 )
                fixedValue = "0";
            else {
                try {
                    Long.parseLong( value );
                } catch ( Exception e ) {
                    defaultValue = "0";
                }
            }
        } else if ( type.equals( "double" ) ) {
            if ( value.length() == 0 )
                fixedValue = "0";
            else {
                try {
                    Double.valueOf( value );
                } catch ( Exception e ) {
                    defaultValue = "0";
                }
            }
        } else if ( type.equals( "byte[]" ) ) {
            try {
                // FIX
                defaultValue = "{0}";
            } catch ( Exception e ) {
            }
        } else {
                defaultValue = "null";
        }
        if ( null != defaultValue ) {
            if ( null == fixedDefaults.get( a.getName() ) ) {
                fixedDefaults.put( a.getName(), "" );
                Builder.print("WARNING: Attribute '" + a.getName() +
                                "' has a bad default value: " + value +
                                ", changed to " + defaultValue );
            }
            fixedValue = defaultValue;
        }
        return fixedValue;
    }

    private java.util.Date str2date( String s ) {
        String[] formats = {
            "yyyy-MM-dd H:mm:ss a",		"yyyy MM dd H:mm:ss a",
            "yyyy-MM-dd H:mm:ss",		"yyyy MM dd H:mm:ss",
            "yyyy-MM-dd H:mm a",		"yyyy MM dd H:mm a",
            "yyyy-MM-dd H:mm",		"yyyy MM dd H:mm",
            "yyyy-MM-dd",			"yyyy MM dd",
            "MMM dd yyyy h:mm:ss a",	"MMM dd, yyyy h:mm:ss a",
            "MMM dd yyyy h:mm:ss",		"MMM dd, yyyy h:mm:ss",
            "MMM dd yyyy h:mm a",		"MMM dd, yyyy h:mm a",
            "MMM dd yyyy h:mm",		"MMM dd, yyyy h:mm" };
        java.util.Date ret = null;
        for ( int i = 0; i < formats.length; i++ ) {
            SimpleDateFormat df = new SimpleDateFormat( formats[ i ] );
            try {
                ret = df.parse( s );
                break;
            } catch ( ParseException e ) {
            }
        }
        return ret;
    }

    private String convertArrayType( String type ) {
        String at = type;
        int i = type.indexOf( "[]" );
        if ( -1 != i )
                at = type.substring( 0, i ) + "s";
/* 		at = type.substring( 0, i ) + "Array"; */
        return at;
    }

    private String JavaType( String type ) {
/* 	return type.replace( '.', '_' ); */
        String jt = type.replace( '.', '_' );
        return convertArrayType( jt );
    }

    private String JdbcType( String javaType ) {
        String type = javaType;
        int dot = type.lastIndexOf( '.' );
        if ( -1 != dot )
                type = type.substring( dot + 1 );
        String firstChar = type.substring( 0, 1 ).toUpperCase();
/* 	return firstChar + type.substring( 1 ); */
        return convertArrayType( firstChar + type.substring( 1 ) );
    }

    protected void setTagUpperMemberName( String tag, String name )
    throws Exception {
        String firstChar = name.substring( 0, 1 ).toUpperCase();
        name = firstChar + name.substring( 1 );
        tagSet.setTagValue( tag, name );
    }

    public void adjustOutputFile() throws Exception {
        if ( null == output ) {
            // The SourceGenerator subclass decided not to generate
            // an output file.  For example, there is no
            // SQL output file needed for an abstract class.
            return;
        }
        output.close();

        String[] argv;
        int status;

        /*
        argv = new String[ 3 ];

        argv[0] = "chown";
        argv[1] = "???";
        argv[2] = output.getPath();

        Process chown = Runtime.getRuntime().exec( argv );
        status = chown.waitFor();
        if (status != 0 ) {
                     throw new Exception("Change ownership failed");
        }

        argv = new String[ 3 ];

        argv[0] = "chgrp";
        argv[1] = "???";
        argv[2] = output.getPath();

        Process chgrp  = Runtime.getRuntime().exec( argv );

        status = chgrp.waitFor();
        if (status != 0) {
                    throw new Exception("Change group failed");
        }
        */

    }

    public void defaultWriteFileHeader(  DataObject o ) throws Exception {
//System.err.println("*** defaultWriteFileHeader");
        filterFileHeader.reset();
        filterFileHeader.process( tagSet );
        filterFileHeader.close();
    }
    public void defaultWriteFileFooter(  DataObject o ) throws Exception {
//System.err.println("*** defaultWriteFileFooter");

        tagSet.setTagValue( CREATE_PARM_SIG,  parmSig );
        tagSet.setTagValue( CREATE_PARM_LIST, parmList );
        tagSet.setTagValue( PK_PK_SET_CALLS, pkpkSetCalls );
        tagSet.setTagValue( DO_PK_SET_CALLS, dopkSetCalls );
        tagSet.setTagValue( PK_DO_SET_CALLS, pkdoSetCalls );
		tagSet.setTagValue( QUERY_PK_SET_CALLS, querypkSetCalls );

        filterFileFooter.reset();
        filterFileFooter.process( tagSet );
        filterFileFooter.close();
        filterMemberStuff.close();
    }
    public void defaultWriteMemberStuff( Attribute  a ) throws Exception {
/* System.err.println("*** defaultWriteMemberStuff"); */
        filterMemberStuff.reset();
        filterMemberStuff.process( tagSet );
/* 	filterMemberStuff.close(); */
    }

    abstract public void writeFileHeader(  DataObject o ) throws Exception;
    abstract public void writeFileFooter(  DataObject o ) throws Exception;
    abstract public void writeMemberStuff( Attribute  a ) throws Exception;

    /*
    public void openTemplates( String templateDirFromConfig ) {
        templateDir = templateDirFromConfig;
        File templateFile;
        templateFile = File( templateDir + "/" + FILE_HEADER_TEMPLATE_NAME );
        templateFileHeader = new TemplateReader( templateFile.getPath() );
        templateFile = File( templateDir + "/" + FILE_FOOTER_TEMPLATE_NAME );
        templateFileFooter = new TemplateReader( templateFile.getPath() );
        templateFile = File( templateDir + "/" + MEMBER_STUFF_TEMPLATE_NAME );
        templateMemberStuff = new TemplateReader( templateFile.getPath() );
    }
    */

    /** Array impl #2 (see $qdb/todo)
     *  If DataObject XXX has an array Attribute AAA,
     *  AAA is converted to a new DataObject that refers to XXX.
     *  That new DataObject is processed normally by the Builder,
     *  generating all necessary .java and schema files.
     */
    public static DataObject handleArray( DataObject o, Attribute a ) {
        String tableName = o.getName() + "_" + a.getName();
        DataObject ao   = new DataObject( tableName );
        ao.setAbstract( false );
        ao.setCaching( false );
        try {
            ao.setDBTableName( tableName );
        } catch ( DataObjectNameException e ) {
            Builder.print(
                "ERROR!! DataObjectNameException for tableName="+tableName);
        }
        try {
            ao.setDBTableType( DataObject.CONCRETE );
        } catch ( DataObjectTypeException e ) {
            Builder.print(
                "ERROR!! DataObjectTypeException for tableName="+tableName);
        }
        ao.setFinal( true );
        ao.setFullCaching( false );
        ao.setIndex( false );
        ao.setLazyLoading( false );
        try {
            ao.setPackageName( o.getAdjustedPackageName() );
        } catch ( DataObjectNameException e ) {
            Builder.print(
                "ERROR!! DataObjectNameException for tableName="+tableName);
        }

        /*
        Attribute  ref = new Attribute(  o.getName() + "_ref" );  // ref to o
        ref.setArray( false );
        ref.setCanBeNull( true );
        ref.setConstant( false );
        if ( -1 == o.getName().indexOf( "DO" ) )
            ref.setDBType( "BigDecimal" );
        else
            ref.setDBType( a.getJavaType() );
        ref.setIndex( false );
        ref.setJavaType( o.getName() + "DO" );
        ref.setJavadoc( "/" + "* ref to " + o.getName() + " *" + "/" );
        ref.setObjectRef( true );
        ref.setRefConstraint( false );
        ref.setUsedForQuery( false );
        ao.addAttribute( ref );
        */

        Attribute index = new Attribute( "index" );
        index.setArray( false );
        index.setCanBeNull( false );
        index.setConstant( false );
        index.setDBType( "INTEGER" );
        index.setIndex( false );
        index.setJavaType( "int" );
        index.setJavadoc( "/* index for array table " + tableName + " */" );
        index.setObjectRef( false );
        index.setRefConstraint( false );
        index.setUsedForQuery( false );
        ao.addAttribute( index );

        Attribute  value = new Attribute( "value" );
        value.setArray( false );
        value.setCanBeNull( true );
        value.setConstant( false );
        if ( -1 == o.getName().indexOf( "DO" ) )
            value.setDBType( "BigDecimal" );
        else
            value.setDBType( a.getJavaType() );
        value.setIndex( false );
        value.setJavaType( a.getJavaType() );
        value.setJavadoc( a.getJavadoc() );
        value.setObjectRef( a.isObjectRef() );
        value.setRefConstraint( false );
        value.setUsedForQuery( false );
        ao.addAttribute( value );

        return ao;
    }

    /**
     * If condition is true, return tag to temporarily suspend code generation.
     * This allows conditionally processed sections within a .template file.
     */
    public String beginNoCodeIf( boolean condition ) {
        return condition ? TemplateFilter.skipStart : "";
    }

    /**
     * If condition is true, return tag to resume code generation.
     */
    public String endNoCodeIf( boolean condition ) {
        return condition ? TemplateFilter.skipStop : "";
    }

}
