/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: SourceGenerator_BDO.java,v 1.1 2001/09/28 16:01:06 cryd0221 Exp $
 */




package org.enhydra.dods.builder.generator.bdo;

import org.enhydra.dods.store.doa.DataObject;
import org.enhydra.dods.store.doa.Attribute;
import org.enhydra.dods.store.doa.DODSProject;

import org.enhydra.dods.builder.engine.*;
import org.enhydra.dods.builder.generator.base.*;

import java.io.File;
import java.io.StringWriter;
import java.util.Vector;

public class SourceGenerator_BDO extends SourceGenerator {

/*     final String DO_CLASS_NAME		= "DO_CLASS_NAME"; */
    final String QUERY_CLASS_NAME	= "QUERY_CLASS_NAME";
    final String BEGIN_NONCONSTANT	= "BEGIN_NONCONSTANT";
    final String END_NONCONSTANT	= "END_NONCONSTANT";
    final String START_NO_REFS	        = "START_NO_REFS";
    final String STOP_NO_REFS	        = "STOP_NO_REFS";
    final String STUBS_FOR_COMMIT_ASSERTIONS = "STUBS_FOR_COMMIT_ASSERTIONS";
    final String COMMIT_REFERENCED_DOS	     = "COMMIT_REFERENCED_DOS";
    final String MEMBER_BDO_CLASS_NAME	     = "MEMBER_BDO_CLASS_NAME";
    final String MEMBER_DO_CLASS_NAME	     = "MEMBER_DO_CLASS_NAME";
    final String NULL_OK	     = "NULL_OK";
    final String BEGIN_NO_CASCADE_DELETE     = "BEGIN_NO_CASCADE_DELETE";
    final String END_NO_CASCADE_DELETE       = "END_NO_CASCADE_DELETE";

    final protected String N21_REFERRERS        = "N21_REFERRERS";
    final protected String N2N_REFERRERS        = "N2N_REFERRERS";
    final protected String REFERRER             = "REFERRER";
    final protected String FULL_REFERRER             = "FULL_REFERRER";
    final protected String REFERRED             = "REFERRED";
    final protected String FULL_REFERRED             = "FULL_REFERRED";
    final protected String REFERRED_VIA             = "REFERRED_VIA";
    final protected String GET_ATTR             = "GET_ATTR";
    final protected String THIS_ATTR            = "THIS_ATTR";
    final protected String GET_ARRAY_METHOD_NAME = "GET_ARRAY_METHOD_NAME";
    final protected String GET_SINGLE_METHOD_NAME = "GET_SINGLE_METHOD_NAME";
    final protected String GET_SINGLE_DO_METHOD_NAME = "GET_SINGLE_DO_METHOD_NAME";
    final protected String GET_ARRAY_DO_METHOD_NAME = "GET_ARRAY_DO_METHOD_NAME";
    final protected String GET_SINGLE_BDO_METHOD_NAME = "GET_SINGLE_BDO_METHOD_NAME";
    final protected String GET_ARRAY_BDO_METHOD_NAME = "GET_ARRAY_BDO_METHOD_NAME";
    final protected String ADD_BDO_METHOD_NAME = "ADD_BDO_METHOD_NAME";
    final protected String REMOVE_BDO_METHOD_NAME = "REMOVE_BDO_METHOD_NAME";
    final protected String CASCADING_DELETES = "CASCADING_DELETES";

    // template reader and filter for StubForCommitAssertion.template file.
    private TemplateFilter filterStubForCommitAssertions;
    private StringWriter   StubForCommitAssertions;

    // template reader and filter for CommitDeleteOtherDO.template file.
    private TemplateFilter filterCommitDeleteOtherDOs;
    private StringWriter   CommitDeleteOtherDOs;

    // template reader and filter for manyToOneReferrers.template file.
    private TemplateFilter filterN21Referrers;
    private StringWriter   n21Referrers;

    // template reader and filter for manyToManyReferrers.template file.
    private TemplateFilter filterN2NReferrers;
    private StringWriter   n2nReferrers;

    // template reader and filter for CascadingDelete.template file.
    // Given a relationship like
    //		Person		Address
    //		oid <---------- person
    // a Person.delete() will cause an Address.delete().
    // Note that an Address.delete() will not cause a Person.delete().
    //
    // For this reason, all 1-1 relationships should be expressed as
    // back-pointers.  That is, the 1-1 relationship between Person and Address
    // should *NOT* be expressed in a DODS project as
    //		Person		Address
    //		addr ---------> oid
    // even though that is more instinctive from an OO design perspective.
    private TemplateFilter filterCascadingDeletes;
    private StringWriter   cascadingDeletes;

    public SourceGenerator_BDO() throws Exception {
    }

   /**
     * Adds tag values to the <CODE>TagSet</CODE>.
     * This method invokes 
     *    <CODE>TemplateFilter.process( tagSet )</CODE>
     * which will create the output source file.
     *
     * @param o
     *   DataObject for which source code will be generated by
     *    <CODE>writeFileHeader</CODE> and
     *    <CODE>writeFileFooter</CODE>
     * @exception Exception
     *   If an unrecoverable error occurs.
     */
    public void objectPrep(	GeneratorInfo sgi,
				DODSProject project,
				DataObject o )
    throws Exception {
	super.objectPrep( sgi, project, o );
/* 	tagSet.setTagValue( DO_CLASS_NAME,    o.getName() + "DO" ); */
	tagSet.setTagValue( QUERY_CLASS_NAME, o.getName() + "Query" );

	tagSet.setTagValue( BEGIN_NO_CASCADE_DELETE,
					beginNoCodeIf( cascadeDelete ));
	tagSet.setTagValue( END_NO_CASCADE_DELETE,
					endNoCodeIf( cascadeDelete ));

        TemplateReader reader = new TemplateReader(
             info.templateDir + File.separator 
		+ "CommitDeleteOtherDO.template" );
        CommitDeleteOtherDOs = new StringWriter();
        filterCommitDeleteOtherDOs = new TemplateFilter(
                                        reader, CommitDeleteOtherDOs );

        reader = new TemplateReader(
             info.templateDir + File.separator 
		+ "StubForCommitAssertion.template" );
        StubForCommitAssertions = new StringWriter();
        filterStubForCommitAssertions = new TemplateFilter(
                                        reader, StubForCommitAssertions );

        reader = new TemplateReader(
             info.templateDir +File.separator+ "manyToOneReferrers.template" );
        n21Referrers = new StringWriter();
        filterN21Referrers = new TemplateFilter( reader, n21Referrers );

        reader = new TemplateReader(
             info.templateDir +File.separator+ "manyToManyReferrers.template" );
        n2nReferrers = new StringWriter();
        filterN2NReferrers = new TemplateFilter( reader, n2nReferrers );

        reader = new TemplateReader(
             info.templateDir +File.separator+ "CascadingDeletes.template" );
        cascadingDeletes = new StringWriter();
        filterCascadingDeletes = new TemplateFilter( reader, cascadingDeletes );

        getAddDelReferrers( o, project );

	// FIX remove to end of method
	/* Do not generate commit() and delete() convenience methods
	 * for DO's that have references to other DO's.
	 * See template for details.
	 */
	boolean has_refs = false;
	Vector attrs = o.getAttributes();
	for ( int i = 0; i < attrs.size(); i++ ) {
	    Attribute a = (Attribute) attrs.elementAt( i );
	    if ( a.isObjectRef() ) {
		has_refs = true;
	    }
	}
	/* FIX delete
	if ( has_refs ) {
	    tagSet.setTagValue( START_NO_REFS, TemplateFilter.skipStart );
	    tagSet.setTagValue( STOP_NO_REFS,  TemplateFilter.skipStop );
	} else {
	    tagSet.setTagValue( START_NO_REFS, "" );
	    tagSet.setTagValue( STOP_NO_REFS,  "" );
	}
	*/
	tagSet.setTagValue( START_NO_REFS, beginNoCodeIf( has_refs ) );
	tagSet.setTagValue( STOP_NO_REFS,  endNoCodeIf( has_refs ) );
    }



    /**
    *
    *  This algorithm generates short or long method names.
    *  Consider the following example Data Objects used by a telephone company:
    *       Data Object         Attribute type, name
    *       -----------         --------------  -----------
    *       Phone               int             phoneNumber
    *       Person              PhoneDO,        phone
    *       Business            PhoneDO,        mainPhone
    *       Business            PhoneDO,        faxPhone
    *   Multiple Persons can share the same Phone,
    *   and (in this example) multiple Businesses can share the same Phone.
    *   We generate the following method names for the PhoneDO class:
    *       return type         method name
    *       -----------         ------------------------------------
    *       PersonDO[]          PhoneBDO.getPersonDOArray()
    *       PersonDO[]          PhoneBDO.getBusinessDOArray_mainPhone()
    *       PersonDO[]          PhoneBDO.getBusinessDOArray_faxPhone()
    *   The telephone company would find all the Persons
    *   with the same home phone number by invoking
    *       Phone.getPersonDOArray()
    *           (a short method name, since Person has
    *            just 1 Attribute referring to Phone)
    *   and find all the Businesses using a given fax line by invoking
    *       Phone.getBusinessDOArray_faxPhone()
    *           (a long method name, since Business has
    *            more than 1 Attribute referring to Phone)
    *
    *   Ok.  That's the many-to-one (n21) case.
    *
    *   Now for the many-to-many (n2n) case.
    *   Consider another example:
    *       Data Object         Attribute type, name
    *       -----------         --------------  -----------
    *       Person              PhoneDO,        phone
    *       Team                String,         name
    *       Person_Team         PersonDO,       person
    *       Person_Team         TeamDO,         team
    *   The Person_Team table has 2 attributes, a PersonDO ref & a TeamDO ref.
    *   So, a Person can be on >1 Team, and a Team can have >1 Person.
    *   To make it easy to find out what Persons are on a Team,
    *   we could use the method generated by the one-to-many stuff above
    *       return type         method name
    *       -----------         ------------------------------------
    *       PersonDO[]          Person_TeamBDO.getPersonDOArray()
    *   But the Person_Team table is really a contrivance to get the correct
    *   relationships expressed in the database (i.e. an implementation detail.)
    *   It would be better to have a method
    *       PersonDO[]  TeamBDO.getPersonDOArray()
    *   to allow the application to avoid dealing with the Person_Team table.
    *   The many-to-many handling below would generate the methods
    *       PersonDO[]          TeamBDO.getPersonDOArray()
    *       TeamDO[]            PersonBDO.getTeamDOArray()
    *   unless Person itself had a reference to Team
    *   or if  Team   itself had a reference to Person
    *   in which case the purpose of the Person_Team table becomes unclear.
    */
    private void getAddDelReferrers( DataObject obj, DODSProject project )
        throws Exception {
      
      filterN21Referrers.reset();
      filterN2NReferrers.reset();
      DataObject[] all = Builder.getAllDataObjects(project);
      Attribute attr  = null;
      Attribute attr2 = null;
      boolean DO_is_N2N_map;
      
      for ( int i = 0; i < all.length; i++ ) {
        DataObject DO = all[ i ];
        DataObject DO2 = null;
        DO_is_N2N_map = false;
        Vector attrs = DO.getAttributes();
        if ( null == attrs ) {
            continue;
        }
        Vector refsN21 = new Vector();
        Vector refsN2N = new Vector();

        // we are only interested in the attrs that are obj references.
        Vector refAttrs = new Vector();
        for ( int j = 0; j < attrs.size(); j++ ) {
          attr = (Attribute) attrs.elementAt( j );
          if ( attr.isObjectRef() && attr.isUsedForQuery() ) {
            refAttrs.addElement( attr );
          }
        }

        //for ( int j = 0; j < attrs.size(); j++ ) {
        for ( int j = 0; j < refAttrs.size(); j++ ) {
          //attr = (Attribute) attrs.elementAt( j );
          attr = (Attribute) refAttrs.elementAt( j );
          if ( attr.getObjectRefDO() != obj ) {
            continue;
          }

          refsN21.addElement( attr );

          // An N-to-N object mapping table has just 2 obj-ref attributes:
          // 2 references into different tables.
          // (It might have other attributes as well.)
          // If DO has just 2 obj-ref attributes and the other attribute
          // is a ref to a different DO, we generate N-to-N
          // access methods for those other DOs.
          if ( 2 != refAttrs.size() ) {
            continue;
          }
          int k = ( 0 == j ? 1 : 0 );  // look at other attr
          attr2 = (Attribute) refAttrs.elementAt( k );

           // does other attr refer to some other DO?
           // if obj is Person, attr is Person_Team.team
          // (see example above)
/*
                if ( ! (   attr2.isObjectRef()
//                      && attr2.getObjectRefDO() != obj
                        && attr2.isUsedForQuery() ) )
                    continue;
*/
//System.err.println("getAddDelReferrers n2n: DO.getName()="+DO.getName() + ", attr="+attr.getName());
          DO2 = (DataObject) attr2.getObjectRefDO();
          // At this point we suspect that DO expresses a many-to-many
          // relationship between obj and DO2.  (Read that again.)
          // But, if obj has a ref to DO2, or if DO2 has a ref to obj,
          // then the purpose of DO becomes unclear,
          // and we don't generate any access methods to DO2 for obj.
          if ( DOhasRefToDO( obj, DO2 ) || DOhasRefToDO( DO2, obj ) ) {
            continue;
          }

	  // If obj and DO2 are the same, then
	  // DO has two refs to obj.
	  // This is not a many-to-many case.
	  if ( obj == DO2 )
	      continue;

//System.err.println("getAddDelReferrers n2n: DO.getName()="+DO.getName() + ", attr2="+attr2.getName());
          DO_is_N2N_map = true;
        }           // end for j (looping thru attrs of DO)

        // generate convenience methods for many-to-one relationships
        String DOname = DO.getName();
        // only add these methods for lowest level classes.
        if (DO.isAbstract() || DO.getExtendedBy().size() > 0) {
          continue;
        }
        for ( int k = 0; k < refsN21.size(); k++ ) {
            attr = (Attribute) refsN21.elementAt( k );
            String get_single_DO  = "get" + DOname + "DO";
            String get_array_DO   = "get" + DOname + "DOArray";
            String get_single_BDO = "get" + DOname + "BDO";
            String get_array_BDO  = "get" + DOname + "BDOArray";
            String add_single_BDO  = "add" + DOname + "BDO";
            String remove_single_BDO  = "remove" + DOname + "BDO";
            if ( 1 < refsN21.size() )
            {
                get_single_DO  += "_" + attr.getName();
                get_array_DO   += "_" + attr.getName();
                get_single_BDO += "_" + attr.getName();
                get_array_BDO  += "_" + attr.getName();
                add_single_BDO += "_" + attr.getName();
                remove_single_BDO += "_" + attr.getName();
            }

            // manyToOneReferrers.template needs the
            // UPPER_MEMBER_NAME tag set to the referring attr name.
            super.memberPrep( attr );

            tagSet.setTagValue( GET_SINGLE_DO_METHOD_NAME,
                                                get_single_DO );
            tagSet.setTagValue( GET_ARRAY_DO_METHOD_NAME,
                                                get_array_DO );
            tagSet.setTagValue( GET_SINGLE_BDO_METHOD_NAME,
                                                get_single_BDO );
            tagSet.setTagValue( GET_ARRAY_BDO_METHOD_NAME,
                                                get_array_BDO );
            tagSet.setTagValue( ADD_BDO_METHOD_NAME,
                                                add_single_BDO );
            tagSet.setTagValue( REMOVE_BDO_METHOD_NAME,
                                                remove_single_BDO );
            tagSet.setTagValue( REFERRER, DOname );
            tagSet.setTagValue( FULL_REFERRER,
                    DO.getPackageName() + "." + DOname );
            filterN21Referrers.reset();
            filterN21Referrers.process( tagSet );
            filterCascadingDeletes.reset();
            filterCascadingDeletes.process( tagSet );
        }

        if ( DO_is_N2N_map ) {
          String DO2name = DO2.getName();
          //String get_single_DO  = "get" + DO2name + "DO";
          String get_array_DO   = "get" + DO2name + "DOArray_via_" + DOname;
          //String get_single_BDO = "get" + DO2name + "BDO";
          //String get_array_BDO  = "get" + DO2name + "BDOArray";

          // manyToOneReferrers.template needs the
          // UPPER_MEMBER_NAME tag set to the referring attr name.
          super.memberPrep( attr );

          //tagSet.setTagValue( GET_SINGLE_DO_METHOD_NAME, get_single_DO );
          tagSet.setTagValue( GET_ARRAY_DO_METHOD_NAME, get_array_DO );
          //tagSet.setTagValue( GET_SINGLE_BDO_METHOD_NAME, get_single_BDO);
          //tagSet.setTagValue( GET_ARRAY_BDO_METHOD_NAME, get_array_BDO );
          tagSet.setTagValue( REFERRER, DOname );
          tagSet.setTagValue( REFERRED, DO2name );
          tagSet.setTagValue( REFERRED_VIA, DO2name + "_via_" + DOname );
          tagSet.setTagValue( FULL_REFERRER,
                  DO.getPackageName() + "." + DOname );
          tagSet.setTagValue( FULL_REFERRED,
                  DO2.getPackageName() + "." + DO2name );
          String name = attr2.getName();
          String firstChar = name.substring( 0, 1 ).toUpperCase();
          name = firstChar + name.substring( 1 );
          tagSet.setTagValue( GET_ATTR, name );
          name = attr.getName();
          firstChar = name.substring( 0, 1 ).toUpperCase();
          name = firstChar + name.substring( 1 );
          tagSet.setTagValue( THIS_ATTR, name );

          filterN2NReferrers.reset();
          filterN2NReferrers.process( tagSet );
        }
      }
    }


    /**
    *
    *  This algorithm generates short or long method names.
    *  Consider the following example Data Objects used by a telephone company:
    *       Data Object		Attribute type, name
    *       -----------		--------------  -----------
    *       Phone       	int             phoneNumber
    *       Person      	PhoneDO,        phone
    *       Business    	PhoneDO,        mainPhone
    *       Business    	PhoneDO,        faxPhone
    *   Multiple Persons can share the same Phone,
    *   and (in this example) multiple Businesses can share the same Phone.
    *   We generate the following method names for the PhoneDO class:
    *       return type 	method name
    *       ----------- 	------------------------------------
    *       PersonDO[]  	PhoneBDO.getPersonDOArray()
    *       PersonDO[]  	PhoneBDO.getBusinessDOArray_mainPhone()
    *       PersonDO[]  	PhoneBDO.getBusinessDOArray_faxPhone()
    *   The telephone company would find all the Persons
    *   with the same home phone number by invoking
    *       Phone.getPersonDOArray()
    *           (a short method name, since Person has
    *            just 1 Attribute referring to Phone)
    *   and find all the Businesses using a given fax line by invoking
    *       Phone.getBusinessDOArray_faxPhone()
    *           (a long method name, since Business has
    *            more than 1 Attribute referring to Phone)
    *
    *	Ok.  That's the many-to-one (n21) case.
    *
    *   Now for the many-to-many (n2n) case.
    *	Consider another example:
    *	    Data Object		Attribute type, name
    *       -----------		--------------  -----------
    *       Person      	PhoneDO,        phone
    *       Team        	String,         name
    *       Person_Team 	PersonDO,       person
    *       Person_Team 	TeamDO,         team
    *	The Person_Team table has 2 attributes, a PersonDO ref & a TeamDO ref.
    *	So, a Person can be on >1 Team, and a Team can have >1 Person.
    *   To make it easy to find out what Persons are on a Team,
    *   we could use the method generated by the one-to-many stuff above
    *       return type		method name
    *       -----------		------------------------------------
    *       PersonDO[]		Person_TeamBDO.getPersonDOArray()
    *   But the Person_Team table is really a contrivance to get the correct
    *   relationships expressed in the database (i.e. an implementation detail.)
    *   It would be better to have a method
    *       PersonDO[]  TeamBDO.getPersonDOArray()
    *   to allow the application to avoid dealing with the Person_Team table.
    *   The many-to-many handling below would generate the methods
    *       PersonDO[]		TeamBDO.getPersonDOArray()
    *       TeamDO[]		PersonBDO.getTeamDOArray()
    *   unless Person itself had a reference to Team
    *   or if  Team   itself had a reference to Person
    *   in which case the purpose of the Person_Team table becomes unclear.
    */
/*    private void getAddDelReferrers( DataObject obj, DODSProject project )
    throws Exception {
//System.err.println("getAddDelReferrers: obj.getName()="+obj.getName());
        filterN21Referrers.reset();
        filterN2NReferrers.reset();
        DataObject[] all = Builder.getAllDataObjects( project );
	Attribute attr  = null;
	Attribute attr2 = null;
	boolean DO_is_N2N_map;
        for ( int i = 0; i < all.length; i++ ) {
            DataObject DO = all[ i ];
	    DataObject DO2 = null;
	    DO_is_N2N_map = false;
            Vector attrs = DO.getAttributes();
            if ( null == attrs ) 
		continue;
	    Vector refsN21 = new Vector();
	    Vector refsN2N = new Vector();

	    // we are only interested in the attrs that are obj references.
	    Vector refAttrs = new Vector();
	    for ( int j = 0; j < attrs.size(); j++ ) {
		attr = (Attribute) attrs.elementAt( j );
		if ( attr.isObjectRef() && attr.isUsedForQuery() )
		    refAttrs.addElement( attr );
	    }

	    //for ( int j = 0; j < attrs.size(); j++ ) {
	    for ( int j = 0; j < refAttrs.size(); j++ ) {
		//attr = (Attribute) attrs.elementAt( j );
		attr = (Attribute) refAttrs.elementAt( j );
		if ( attr.getObjectRefDO() != obj )
		    continue;
//System.err.println("getAddDelReferrers n21: DO.getName()="+DO.getName() + ", attr="+attr.getName());
		refsN21.addElement( attr );

		// An N-to-N object mapping table has just 2 obj-ref attributes:
		// 2 references into different tables.
		// (It might have other attributes as well.)
		// If DO has just 2 obj-ref attributes and the other attribute
		// is a ref to a different DO, we generate N-to-N
		// access methods for those other DOs.
		if ( 2 != refAttrs.size() ) 
		    continue;
		int k = ( 0 == j ? 1 : 0 );  // look at other attr
		attr2 = (Attribute) refAttrs.elementAt( k );

		// does other attr refer to some other DO?
		// if obj is Person, attr is Person_Team.team
		// (see example above)
//
//		if ( ! (   attr2.isObjectRef()
//			&& attr2.getObjectRefDO() != obj
//		        && attr2.isUsedForQuery() ) )
//		    continue;
//
//System.err.println("getAddDelReferrers n2n: DO.getName()="+DO.getName() + ", attr="+attr.getName());
		DO2 = attr2.getObjectRefDO();
		// At this point we suspect that DO expresses a many-to-many
		// relationship between obj and DO2.  (Read that again.)
		// But, if obj has a ref to DO2, or if DO2 has a ref to obj,
		// then the purpose of DO becomes unclear,
		// and we don't generate any access methods to DO2 for obj.
		if ( DOhasRefToDO( obj, DO2 ) || DOhasRefToDO( DO2, obj ) )
		    continue;

//System.err.println("getAddDelReferrers n2n: DO.getName()="+DO.getName() + ", attr2="+attr2.getName());
		DO_is_N2N_map = true;
	    }		// end for j (looping thru attrs of DO)

	    // generate convenience methods for many-to-one relationships
	    String DOname = DO.getName();
	    for ( int k = 0; k < refsN21.size(); k++ ) {
		attr = (Attribute) refsN21.elementAt( k );
		String get_single_DO  = "get" + DOname + "DO";
		String get_array_DO   = "get" + DOname + "DOArray";
		String get_single_BDO = "get" + DOname + "BDO";
		String get_array_BDO  = "get" + DOname + "BDOArray";
		if ( 1 < refsN21.size() ) {
		    get_single_DO  += "_" + attr.getName();
		    get_array_DO   += "_" + attr.getName();
		    get_single_BDO += "_" + attr.getName();
		    get_array_BDO  += "_" + attr.getName();
		}

		// manyToOneReferrers.template needs the
		// UPPER_MEMBER_NAME tag set to the referring attr name.
		super.memberPrep( attr );

		tagSet.setTagValue( GET_SINGLE_DO_METHOD_NAME,
						    get_single_DO );
		tagSet.setTagValue( GET_ARRAY_DO_METHOD_NAME,
						    get_array_DO );
		tagSet.setTagValue( GET_SINGLE_BDO_METHOD_NAME,
						    get_single_BDO );
		tagSet.setTagValue( GET_ARRAY_BDO_METHOD_NAME,
						    get_array_BDO );
		tagSet.setTagValue( REFERRER, DOname );
		tagSet.setTagValue( FULL_REFERRER,
			DO.getPackageName() + "." + DOname );
		filterN21Referrers.reset();
		filterN21Referrers.process( tagSet );
		filterCascadingDeletes.reset();
		filterCascadingDeletes.process( tagSet );
            }

	    if ( DO_is_N2N_map ) {
		String DO2name = DO2.getName();
		//String get_single_DO  = "get" + DO2name + "DO";
		String get_array_DO   = "get" + DO2name + "DOArray_via_" + DOname;
		//String get_single_BDO = "get" + DO2name + "BDO";
		//String get_array_BDO  = "get" + DO2name + "BDOArray";
		
		// manyToOneReferrers.template needs the
		// UPPER_MEMBER_NAME tag set to the referring attr name.
		super.memberPrep( attr );

		//tagSet.setTagValue( GET_SINGLE_DO_METHOD_NAME, get_single_DO );
		tagSet.setTagValue( GET_ARRAY_DO_METHOD_NAME, get_array_DO );
		//tagSet.setTagValue( GET_SINGLE_BDO_METHOD_NAME, get_single_BDO);
		//tagSet.setTagValue( GET_ARRAY_BDO_METHOD_NAME, get_array_BDO );
		tagSet.setTagValue( REFERRER, DOname );
		tagSet.setTagValue( REFERRED, DO2name );
		tagSet.setTagValue( REFERRED_VIA, DO2name + "_via_" + DOname );
		tagSet.setTagValue( FULL_REFERRER,
			DO2.getPackageName() + "." + DOname );
                tagSet.setTagValue( FULL_REFERRED,
                    DO2.getPackageName() + "." + DO2name );			
		String name = attr2.getName();
		String firstChar = name.substring( 0, 1 ).toUpperCase();
		name = firstChar + name.substring( 1 );
		tagSet.setTagValue( GET_ATTR, name );
		name = attr.getName();
		firstChar = name.substring( 0, 1 ).toUpperCase();
		name = firstChar + name.substring( 1 );
		tagSet.setTagValue( THIS_ATTR, name );

		filterN2NReferrers.reset();
		filterN2NReferrers.process( tagSet );
	    }
        }
    }
*/
    private boolean DOhasRefToDO( DataObject o1, DataObject o2 ) {
	Vector attrs = o1.getAttributes();
	for ( int i = 0; i < attrs.size(); i++ ) {
	    Attribute attr = (Attribute) attrs.elementAt( i );
	    if ( attr.isObjectRef()
	      && attr.getObjectRefDO() == o2
	      && attr.isUsedForQuery() )
	    {
		return true;
	    }
	}
	return false;
    }

    public void memberPrep(	Attribute a ) throws Exception {
	super.memberPrep( a );
	//tagSet.setTagValue( BEGIN_NONCONSTANT, a.isConstant() ? "/*" : "" );
	//tagSet.setTagValue( END_NONCONSTANT,   a.isConstant() ? "*/" : "" );
	tagSet.setTagValue( BEGIN_NONCONSTANT, beginNoCodeIf( a.isConstant() ));
	tagSet.setTagValue( END_NONCONSTANT,   endNoCodeIf(   a.isConstant() ));

	if ( a.isObjectRef() ) {
	    DataObject ao = a.getObjectRefDO();

	    tagSet.setTagValue( MEMBER_DO_CLASS_NAME, 
		ao.getPackageName() + "." + ao.getName() + "DO" );
	    tagSet.setTagValue( MEMBER_BDO_CLASS_NAME, 
		ao.getPackageName() + "." + ao.getName() + "BDO" );

	    if ( a.canBeNull() )
		tagSet.setTagValue( NULL_OK,  "true" );
	    else
		tagSet.setTagValue( NULL_OK,  "false" );
	    filterStubForCommitAssertions.reset();
	    filterStubForCommitAssertions.process( tagSet );

	    filterCommitDeleteOtherDOs.reset();
	    filterCommitDeleteOtherDOs.process(    tagSet );
	}
    }

    public void writeFileHeader(  DataObject o ) throws Exception {
	defaultWriteFileHeader( o );
 
        Vector ancestors = o.getAncestors();
        if ( null != ancestors ) {
            for ( int i = 0; i < ancestors.size(); i++ ) {
                DataObject p = (DataObject) ancestors.elementAt( i );
                if ( null != p ) {
                    Vector members = p.getAttributes();
                    for ( int j = 0; j < members.size(); j++ ) {
                        Attribute attr = (Attribute) members.elementAt( j );
                        memberPrep( attr );
                        writeMemberStuff(   attr );
                    }
                }
            }
        }
    }

    public void writeMemberStuff( Attribute  a ) throws Exception {
	defaultWriteMemberStuff( a );
    }

    public void writeFileFooter(  DataObject o ) throws Exception {
	tagSet.setTagValue( STUBS_FOR_COMMIT_ASSERTIONS, 
		StubForCommitAssertions.toString() );
	tagSet.setTagValue( COMMIT_REFERENCED_DOS, 
		CommitDeleteOtherDOs.toString() );
        if ( null != n21Referrers )
            tagSet.setTagValue( N21_REFERRERS, n21Referrers.toString() );
        if ( null != n2nReferrers )
            tagSet.setTagValue( N2N_REFERRERS, n2nReferrers.toString() );
        if ( null != cascadingDeletes )
            tagSet.setTagValue( CASCADING_DELETES, cascadingDeletes.toString() );
	defaultWriteFileFooter( o );
        filterN21Referrers.close();
        filterN2NReferrers.close();
	filterCascadingDeletes.close();
	filterStubForCommitAssertions.close();
	filterCommitDeleteOtherDOs.close();
    }
}
