/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: SourceGenerator_DataStruct.java,v 1.1 2001/09/28 16:01:06 cryd0221 Exp $
 */




package org.enhydra.dods.builder.generator.datastruct;

import org.enhydra.dods.store.doa.DataObject;
import org.enhydra.dods.store.doa.Attribute;
import org.enhydra.dods.store.doa.DODSProject;

import org.enhydra.dods.builder.engine.*;
import org.enhydra.dods.builder.generator.base.*;

import java.io.File;
import java.io.StringWriter;

public class SourceGenerator_DataStruct extends SourceGenerator {

    final String CLONING_ASSIGNMENTS = "CLONING_ASSIGNMENTS";
    final String CLONE_VALUE         = "CLONE_VALUE";
    final String BEGIN_CONSTANT      = "BEGIN_CONSTANT";
    final String END_CONSTANT        = "END_CONSTANT";

    private TemplateFilter filterCloningAssignments;
    private StringWriter   CloningAssignments;

    public SourceGenerator_DataStruct() throws Exception {
    }

   /**
     * Adds tag values to the <CODE>TagSet</CODE>.
     * This method invokes 
     *    <CODE>TemplateFilter.process( tagSet )</CODE>
     * which will create the output source file.
     *
     * @param o
     *   DataObject for which source code will be generated by
     *    <CODE>writeFileHeader</CODE> and
     *    <CODE>writeFileFooter</CODE>
     * @exception Exception
     *   If an unrecoverable error occurs.
     *
     */
    public void objectPrep(	GeneratorInfo sgi,
				DODSProject project,
				DataObject o )
    throws Exception {
	super.objectPrep( sgi, project, o );
	TemplateReader reader = new TemplateReader(info.templateDir + File.separator + "cloning.template" );
    
	CloningAssignments = new StringWriter();
	filterCloningAssignments = new TemplateFilter(
					reader, CloningAssignments );

    }

    public void memberPrep(	Attribute a ) throws Exception {
	super.memberPrep( a );
	String name = a.getName();

	// set the CLONE_VALUE tag
		// boolean, byte, short, int, long, float, double
		// String
		// java.sql.Date, java.sql.Time, java.sql.Timestamp
		// byte[]
		// java.math.BigDecimal
		// 
	String tagValue = null;
	String primitiveTypes = 
		":boolean:char:byte:short:int:long:float:double:";
	String timeTypes = 
		":java.sql.Date:java.sql.Time:java.sql.Timestamp:";
	String type  = a.getJavaType();
	if ( -1 != primitiveTypes.indexOf( ":" + type + ":" ) ) {
	    tagValue = name;
	} else if ( -1 != timeTypes.indexOf( ":" + type + ":" ) ) {
//	    tagValue = "new " + type + "(" + name + ".getTime() )";
	    String shortType = type.substring( type.lastIndexOf( '.' ) + 1 );
	    tagValue = "GenericDO.copy" + shortType + "(" + name + ")";
	} else if ( type.equals( "String" ) ) {
//	    tagValue = name + " + \"\"";
	    tagValue = "GenericDO.copyString(" + name + ")";
	} else if ( type.equals( "byte[]" ) ) {
//	    tagValue =  "copyByteArray(" + name + ")";
	    tagValue =  "GenericDO.copyByteArray(" + name + ")";
	} else if ( type.equals( "java.math.BigDecimal" ) ) {
//	    tagValue =  "new java.math.BigDecimal(" + name + ".toString())";
	    tagValue =  "GenericDO.copyBigDecimal(" + name + ")";
	} else {
	    if ( a.isObjectRef() )
		tagValue = type + ".createCopy( " + name + " )";
	    else
		// assume data type is of a cloneable class
		tagValue = "(" + type + ") " + name + ".clone()";
	}
	tagSet.setTagValue( CLONE_VALUE, tagValue );
	tagSet.setTagValue( BEGIN_CONSTANT, 
		a.isConstant() ? TemplateFilter.skipStart : "" );
	tagSet.setTagValue( END_CONSTANT,
		a.isConstant() ? TemplateFilter.skipStop : "" );
    }

    public void writeFileHeader(  DataObject o ) throws Exception {
	defaultWriteFileHeader( o );
    }

    public void writeMemberStuff( Attribute  a ) throws Exception {
	defaultWriteMemberStuff( a );
	filterCloningAssignments.reset();
	filterCloningAssignments.process( tagSet );
    }

    public void writeFileFooter(  DataObject o ) throws Exception {
	if ( null != CloningAssignments )
	    tagSet.setTagValue( CLONING_ASSIGNMENTS,
		CloningAssignments.toString() );
	defaultWriteFileFooter( o );
	filterCloningAssignments.close();
    }
}
