/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: SourceGenerator_Query.java,v 1.1 2001/09/28 16:01:06 cryd0221 Exp $
 */




package org.enhydra.dods.builder.generator.query;

import org.enhydra.dods.store.doa.DataObject;
import org.enhydra.dods.store.doa.Attribute;
import org.enhydra.dods.store.doa.DODSProject;

import org.enhydra.dods.builder.engine.*;
import org.enhydra.dods.builder.generator.base.*;

import java.io.File;
import java.io.StringWriter;
import java.util.*;

public class SourceGenerator_Query extends SourceGenerator {

//    final String NULL_FLAG		= "NULL_FLAG";
    final String CAN_QUERY		= "CAN_QUERY";
    final String QUERY_COLUMN_NAMES	= "QUERY_COLUMN_NAMES";
    final String QUERY_OID_AND_VERSION	= "QUERY_OID_AND_VERSION";

    final String UPPER_FK_MEMBER_NAME	= "UPPER_FK_MEMBER_NAME";
    final String UPPER_PK_MEMBER_NAME	= "UPPER_PK_MEMBER_NAME";
    final String FK_SETQUERY_CALLS	= "FK_SETQUERY_CALLS";
    final String FK_SETQUERY_METHODS	= "FK_SETQUERY_METHODS";
    final String FK_REF_DO		= "FK_REF_DO";
    final String FK_REF_QUERY_METHOD	= "FK_REF_QUERY_METHOD";

    final String CALL_SP		= "CALL_SP";
    final String SP_PARAM_INDEX		= "SP_PARAM_INDEX";

    final String REGISTER_OUT_PARAMS	= "REGISTER_OUT_PARAMS";
    private TemplateFilter filter_registerOutParameter;
    private StringWriter   registerOutParameter;

    public SourceGenerator_Query() throws Exception {
    }

   /**
     * Adds tag values to the <CODE>TagSet</CODE>.
     * This method invokes
     *    <CODE>TemplateFilter.process( tagSet )</CODE>
     * which will create the output source file.
     *
     * @param o
     *   DataObject for which source code will be generated by
     *    <CODE>writeFileHeader</CODE> and
     *    <CODE>writeFileFooter</CODE>
     * @exception Exception
     *   If an unrecoverable error occurs.
     *
     */
    public void objectPrep(	GeneratorInfo sgi,
				DODSProject project,
				DataObject o )
    throws Exception {
	super.objectPrep( sgi, project, o );

	TemplateReader reader = new TemplateReader(
		info.templateDir + File.separator +
		"registerOutParameter.template" );
	registerOutParameter = new StringWriter();
	filter_registerOutParameter = new TemplateFilter(
				reader, registerOutParameter );
    }

    public void memberPrep(	Attribute a ) throws Exception {
	if ( ! a.isUsedForQuery() ) return;
	super.memberPrep( a );

	if ( null != a.getSpParamIndex() )
	    tagSet.setTagValue( SP_PARAM_INDEX, a.getSpParamIndex() );

	/*
	tagSet.setTagValue( NULL_FLAG,
		a.canBeNull() ? "NULL_OK" : "NOT_NULL" );
		*/
	tagSet.setTagValue( CAN_QUERY,
		a.isUsedForQuery() ? "public" : "private" );

    }

    private String sql_query_column_names = "";
    private String table_prefix = "";

    public void writeFileHeader(  DataObject o ) throws Exception {
	defaultWriteFileHeader( o );

	table_prefix = tagSet.getTagValue( TABLE_NAME ) + ".";

        Vector ancestors = o.getAncestors();
	if ( null != ancestors ) {
	    for ( int i = 0; i < ancestors.size(); i++ ) {
		DataObject p = (DataObject) ancestors.elementAt( i );
		if ( null != p ) {
		    Vector members = p.getAttributes();
		    for ( int j = 0; j < members.size(); j++ ) {
			Attribute attr = (Attribute) members.elementAt( j );
			memberPrep( attr );
			writeMemberStuff(   attr );
		    }
		}
	    }
	}
    }

    private void generateSetQueryMethodsForFKs( DataObject o )
    throws Exception {
	tagSet.setTagValue( FK_SETQUERY_METHODS, "" );

	// template reader & filter for FK_setQuery_method.template file.
	TemplateReader reader = new TemplateReader(
	    info.templateDir + File.separator + "FK_setQuery_methods.template");
	StringWriter FKsetQueryMethods = new StringWriter();
	TemplateFilter filterFKsetQueryMethods = new TemplateFilter(
						reader, FKsetQueryMethods);

	// template reader & filter for FK_setQuery_calls.template file.
	StringWriter FKsetQueryCalls = null;
	TemplateFilter filterFKsetQueryCalls = null;

	Hashtable fkByDO = o.getForeignKeysGroupedByReferencedDO();
//System.err.println("fkByDO ="+fkByDO );
	for ( Enumeration e = fkByDO.keys(); e.hasMoreElements(); ) {
	    DataObject referencedDO = (DataObject) e.nextElement();
//System.err.println("referencedDO.getName()="+referencedDO.getName());
	    // o has FK(s) that reference user-defined PK(s) in referencedDO.
	    // Generate calls to setQuery<FK> methods which will
	    // comprise the body of the setQueryReferencedDO() method.
	    Hashtable fkGroups = (Hashtable) fkByDO.get( referencedDO );
            for ( Enumeration e2 = fkGroups.keys(); e2.hasMoreElements(); ) {
                String fkGroup = (String) e2.nextElement();
//System.err.println("fkGroup ="+fkGroup );
                Vector v = (Vector) fkGroups.get( fkGroup );
		reader = new TemplateReader(
		    info.templateDir + File.separator +
		    "FK_setQuery_calls.template" );
		FKsetQueryCalls = new StringWriter();
		filterFKsetQueryCalls = new TemplateFilter(
						reader, FKsetQueryCalls );
		for ( int i = 0; i < v.size(); i++ ) {
		    String[] pk_fk_pair = (String[]) v.elementAt(i);
		    setTagUpperMemberName( UPPER_PK_MEMBER_NAME, pk_fk_pair[0]);
//System.err.println("pk_fk_pair[0]="+pk_fk_pair[0]);
		    setTagUpperMemberName( UPPER_FK_MEMBER_NAME, pk_fk_pair[1]);
//System.err.println("pk_fk_pair[1]="+pk_fk_pair[1]);
		    filterFKsetQueryCalls.reset();
		    filterFKsetQueryCalls.process( tagSet );
		}
		filterFKsetQueryCalls.close();

		// Generate the setQueryReferencedDO() method.
		//String fkRefQueryMethod = "setQuery_" + fkGroup;
		String fkRefQueryMethod = "setQuery_" + fkGroup;
		tagSet.setTagValue( FK_REF_QUERY_METHOD, fkRefQueryMethod );
		tagSet.setTagValue( FK_REF_DO, referencedDO.getName() + "DO" );
		tagSet.setTagValue( FK_SETQUERY_CALLS,
				    FKsetQueryCalls.toString() );
//System.err.println("FKsetQueryCalls="+FKsetQueryCalls);
		filterFKsetQueryMethods.reset();
		filterFKsetQueryMethods.process( tagSet );
	    }
	}
	filterFKsetQueryMethods.close();
	tagSet.setTagValue( FK_SETQUERY_METHODS, FKsetQueryMethods.toString() );
//System.err.println("FKsetQueryMethods="+FKsetQueryMethods);
    }

    private String query_comma		= "";
    private String spReturnQmark	= "";
    private String spInQmarks		= "";
    private String commaSP		= "";

    public void writeMemberStuff( Attribute  a ) throws Exception {
	// Spell out the names of all columns, so that the generated code
	// will not break if a new table definition changes the column order.
	sql_query_column_names += query_comma + table_prefix + a.getName();
	query_comma = ", ";

	if ( a.isSpInParam() || ( a.isSpOutParam() && ! a.isSpReturnValue()))
	    addQmarkForSpParam( a );
	if ( a.isSpOutParam() ) {
	    filter_registerOutParameter.reset();
	    filter_registerOutParameter.process( tagSet );
	}
	if ( a.isSpReturnValue() )
	    spReturnQmark = "? = ";

	if ( ! a.isUsedForQuery() ) return;
	defaultWriteMemberStuff( a );
    }

    private void addQmarkForSpParam( Attribute  a ) throws Exception {
	if ( a.isConstant() )
	    return;
	spInQmarks += commaSP + "?";
	commaSP = ", ";
    }

    public void writeFileFooter(  DataObject o ) throws Exception {
	generateSetQueryMethodsForFKs( o );

	tagSet.setTagValue( QUERY_COLUMN_NAMES,	sql_query_column_names );
	// When spelling out the names of the columns,
	// we do not automatically include "oid" and "version".
	// This is because some DOs are used as VIEWs
	// whose definitions are created by hand.
	tagSet.setTagValue( QUERY_OID_AND_VERSION,
	    ", " + table_prefix + val( OID_DB_COL_NAME ) +
	    ", " + table_prefix + val( VERSION_DB_COL_NAME ) );

	if ( o.isStoredProc() ) {
	    if ( 0 != spInQmarks.length() )
	        spInQmarks = "( " + spInQmarks + " )";
	    tagSet.setTagValue( CALL_SP,
		"{" + spReturnQmark + " call " +
		o.getName() + spInQmarks + "}" );
	    tagSet.setTagValue( REGISTER_OUT_PARAMS,
		registerOutParameter.toString() );
	}
	defaultWriteFileFooter( o );
	filter_registerOutParameter.close();
    }
}
