/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: DatabaseAdapter.java,v 1.1 2001/09/28 16:01:06 cryd0221 Exp $
 */
   package org.enhydra.dods.gui.importer;

   import javax.swing.tree.*;
   import javax.swing.*;
   import java.net.*;
   import java.util.*;
   import java.sql.*;

/**
 * This is a temporary database adapter, until I have time to figure
 * out the Enhydra DatabaseManager classes... Yeah, kinda messy right now  :)
 * @author  Larry Wolcott
 * @version $Revision: 1.1 $
 */

   public class DatabaseAdapter {
      private static boolean DEBUG = true;
      private static String systemName = "";
      private static String driverName = "";
      private static String userID = "";
      private static String password = "";
      private static Connection c = null;
      private static String lastError = null;
      private static String dbmsName = "";
      private static String tname1, tname2, tname3;
      private static JTree tree;
      private static boolean shouldBeConnected = false;
   
      public static void main(String[] args){
      }
   
    /**
     * this method tries to connect the static Connection (c)
     */
      public static synchronized void makeConnection () throws SQLException, ClassNotFoundException {
      // freshen up the connection, then try to connect
         sysPrint("Creating Connection");
         try { 
            if (c != null) { 
               c.close(); 
               c = null; 
            }
         } 
            catch (SQLException e) { 
               throw e;
            }
         try { 
            Class.forName(driverName); 
            sysPrint("Registering driver");// Register JDBC driver
            c = DriverManager.getConnection(systemName, userID, password);
            sysPrint("Driver connecting");
            setIsConnected(true);
         }
            catch (SQLException e) {
               sysPrint("DRIVER CONNECT ERROR: " + e.getErrorCode() + "    STATE: " + e.getSQLState());
               setIsConnected(false); 
               throw e;
            }
            catch (ClassNotFoundException e) {
               sysPrint("DRIVER ERROR, REGISTERING- Class not found"); 
               setIsConnected(false); 
               throw e;
            }
      }
    /**
     * This method takes a (SQL)string, and querys the database with it
     * @param queryString is the string to execute
     */
      public static synchronized ResultSet executeQuery(String queryString) throws SQLException{
         Statement s = null; 
         ResultSet rs = null;
      //Create the statement, then execute query text
         try { 
            s = c.createStatement(); 
            sysPrint("ExecuteQuery: Creating SQL statement");
            sysPrint("QUERY = " + queryString); 
            rs = s.executeQuery(queryString);
            sysPrint("Executing query: " + queryString );
         }
            catch (SQLException e) { 
               sysPrint("ExecuteQuery: SQL EXECUTE ERROR: " + e.getErrorCode() + "     STATE: " + e.getSQLState()); 
               throw e;
            }
            finally { 
               return rs; 
            }
      }
   
    /**
     * This method takes a (SQL)string, and updates the database with it
     * @param queryString is the string to update
     */
      public static synchronized int executeUpdate(String queryString) throws SQLException{
         int  retInt = 0; 
         Statement s = null;	
         try { 
            s = c.createStatement();
            sysPrint("ExecuteUpdate: Creating SQL statement"); 
            retInt = s.executeUpdate(queryString); 
            sysPrint("Executing query: " + queryString ); 
         }
            catch (SQLException e) { 
               sysPrint("ExecuteUpdate: SQL CREATE OR UPDATE ERROR: " + e.getErrorCode() + "     STATE: " + e.getSQLState()); 
               throw e;
            }
            finally { 
               return retInt; 
            }
      }
   
    /*
     * This method tries to close the static Connection (c)
     */
      public static synchronized void closeConnection() throws SQLException{
         try {
            c.close();
            sysPrint("CloseConnection: Close connection..");
            setIsConnected(false);
         }
            catch (SQLException e) { 
               sysPrint("CloseConnection: CONNECTION CLOSE ERROR: " + e.getErrorCode() + "     STATE: " + e.getSQLState()); 
            }
      
      }
   
    /**
     * This method returns a Vector full of catalogs available
     * at the connection
     */
      public static synchronized Vector getCatalogs() throws SQLException{
         Vector thisVect = new Vector(0);
         try {
            DatabaseMetaData dbmd = c.getMetaData();
            dbmsName = dbmd.getDatabaseProductName();
            ResultSet rs = dbmd.getTableTypes();
            ResultSetMetaData rsmdatax = rs.getMetaData();
         
            ResultSet rs2 = dbmd.getCatalogs();
            String catName = "";
            ResultSetMetaData rsmdata = rs2.getMetaData();
         
            sysPrint("The following Schemas are ");
            sysPrint("available in " + dbmsName + ":  ");
            ResultSet rs3 = dbmd.getSchemas();
            String schemaName = "";
            ResultSetMetaData rsmdata2 = rs3.getMetaData();
            sysPrint("col name = " + rsmdata2.getColumnName(1) ); 
            tname3=rsmdata2.getColumnName(1);
            thisVect = new Vector(0);
            while (rs3.next()) {
               schemaName = rs3.getString(tname3);
               thisVect.addElement(schemaName);
               sysPrint("Schema Name = " + schemaName);
            }
            rs.close();
            rs2.close();
            rs3.close();
            // rsx.close();
            //c.close();
         
         } 
            catch(SQLException e) {
               sysPrint("SQLException: ");
               sysPrint(e.getMessage());
               throw e;
            }
            finally { 
               return thisVect; 
            }
      }
   
    /**
     * This method returns a Vector full of tables available
     * in the collection, at the specified index.
     * @param getCollections is the Vector available collections
     * @param specificNumber is the index of the desired collection to import
     */
      public static synchronized Vector getTables(Vector getCollections, int specificNumber) throws SQLException{
         Vector thisVect = new Vector(0);
         try {
            DatabaseMetaData dbmd = c.getMetaData();
            String whichCollection = null;
            if (getCollections != null) { 
               whichCollection = (String)getCollections.elementAt(specificNumber);
            } 
            else {
               whichCollection = null; 
            }
            sysPrint("The following Tables are ");
            sysPrint("available in " + dbmsName + ", Collection: " + whichCollection);
            String[] tableTypes = { "TABLE" , "VIEW" };
         	//whichCollection.trim()
            ResultSet rsx = dbmd.getTables(null , whichCollection , null , tableTypes ); //null means to get everything!
            ResultSetMetaData rsmdata = rsx.getMetaData();
            String tname=rsmdata.getColumnName(1);
            String tname2=rsmdata.getColumnName(2);
            String tname3=rsmdata.getColumnName(3);
            sysPrint(tname + "  " + tname2 + "  " + tname3);
            int count = 0;
            thisVect = new Vector(0);
            sysPrint("Getting tables! for **" + whichCollection + "**");
            //rsx.first();
            //boolean test;
            //test = rsx.isFirst();
            while (rsx.next()) {
               count++;
               String tabName = rsx.getString("TABLE_NAME");
               thisVect.addElement(tabName);
               sysPrint("Table Name = " + tabName);
            }
         
            rsx.close();
            //c.close();
         } 
            catch(SQLException e) {
               sysPrint("SQLException: ");
               sysPrint(e.getMessage());
               throw e;
            }
            finally { 
               return thisVect; 
            }
      }
   
    /*
     * This method is used to output debug info
     */    
      public static void sysPrint(String string){
         if (DEBUG) 
            System.out.println(string); 
      }
   
      public static synchronized void setUserID(String string){ 
         userID = string;
      }
      public static synchronized void setPassword(String string){ 
         password = string; 
      }
      public static synchronized void setSystemName(String string){ 
         systemName = string; 
      }
      public static synchronized void setDriverName(String string){ 
         driverName = string;
      }
      public static synchronized void setLastError(String string){ 
         lastError = string; 
      }
      public static synchronized void setIsConnected(boolean string){ 
         shouldBeConnected = string;
      }
   
      public static synchronized boolean isConnected(){ 
         return shouldBeConnected; 
      }
      public static synchronized String getSystemName(){ 
         return systemName;
      }
      public static synchronized String getDataSourceName(){ 
         return dbmsName; 
      }
      public static synchronized String getDriverName(){ 
         return driverName;
      }
      public static synchronized String getLastError(){ 
         return lastError;
      }
      public static synchronized String getUserID(){ 
         return userID;
      }
      public static synchronized boolean getIsConnected(){ 
         boolean stat = true;
         if (c != null) {	
            sysPrint("**********************   " + stat);
            try {
               stat = c.isClosed();
            } 
               catch ( SQLException e ) { 
                  sysPrint("GetIsConnected ERROR");
               }	
            sysPrint("**********************   " + stat);
         } 
         else 
            stat = true;
         if (stat) 
            return false; 
         else 
            return true;
      }
   
      public void finalize() { 
         c = null; System.exit(0); 
      }
   
   }
