/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: PackageTreeNode.java,v 1.1 2001/09/28 16:01:07 cryd0221 Exp $
 */
package org.enhydra.dods.gui.tree;

import org.enhydra.dods.store.doa.DOPackage;
import org.enhydra.dods.store.doa.DataObject;
import java.util.Vector;
import javax.swing.tree.TreePath;
import javax.swing.tree.DefaultMutableTreeNode;

/**
 * This class provides a wrapper for displaying DOPackages in a treeview.
 * 
 * @author  Paul Mahar
 * @version $Revision: 1.1 $
 */
public class PackageTreeNode extends DefaultMutableTreeNode {
    private DOPackage exclude = null;

    /**
     * Invoking this construtor will cause this node to be created as
     * well as all childs that belong to it.
     *
     *
     * @param thisPackage
     * The package display in a tree.
     */
    public PackageTreeNode(DOPackage thisPackage) {
	this(thisPackage, null);
    }

    /**
     * Invoking this construtor will cause this node to be created as
     * well as all child nodes that are not equal to a given excluded package.    
     *
     *
     * @param thisPackage
     * The package to display in a tree.
     * @param ex
     * The package to exclude when populating child nodes.
     *  
     */
    public PackageTreeNode(DOPackage thisPackage, DOPackage ex) {
	super(thisPackage, true);

	exclude = ex;

	populate();
    }

    /**
     * Create the child nodes.
     */
    private void populate() {
	DOPackage thisPackage = (DOPackage) this.getUserObject();
	Vector    dataObjects = thisPackage.getDataObjects();
	int       count = dataObjects.size();

	for (int i = 0; i < count; i++) {
	    if (dataObjects.elementAt(i) instanceof DOPackage) {
		DOPackage newPackage = (DOPackage) dataObjects.elementAt(i);

		if ((exclude == null) || (!exclude.equals(newPackage))) {
		    PackageTreeNode newNode = null;

		    newNode = new PackageTreeNode(newPackage, exclude);

		    add(newNode);
		} 
	    } 
	} 
    } 

    /**
     * Find a node containing a given user object. The found node may be this
     * node or and descendents of this node.
     *
     * @param userObject
     * The object to look for.
     *
     * @return
     * A node containing the userObject.
     */
    public PackageTreeNode getNodeContaining(Object userObject) {
	PackageTreeNode node = null;

	if (getUserObject().equals(userObject)) {
	    node = this;
	} else {
	    int count = getChildCount();

	    for (int i = 0; i < count; i++) {
		node = (PackageTreeNode) getChildAt(i);

		if (node.getUserObject().equals(userObject)) {
		    break;
		} else {
		    node = node.getNodeContaining(node);

		    if (node != null) {
			break;
		    } 
		} 
	    } 
	} 

	return node;
    } 

    /**
     * Get a tree path for this node.
     * 
     * @return
     * A tree path you can use to make selections in a JTree.
     */
    public TreePath getTreePath() {
	return new TreePath(getPath());
    } 

    /**
     * Get the string to show in a tree.
     *
     * @return
     * This text is displayed by the default cell renderer.
     */
    public String toString() {
	DOPackage thisPackage = (DOPackage) this.getUserObject();

	return thisPackage.getName();
    } 

}


