/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: AbstractValidationPanel.java,v 1.1 2001/09/28 16:01:07 cryd0221 Exp $
 */

package org.enhydra.dods.merlot;

// Merlot imports
import org.merlotxml.merlot.*;
import org.merlotxml.util.xml.*;
import org.w3c.dom.*;

// Standard imports
import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.Container;
import java.awt.Component;
import java.awt.Color;
import java.awt.event.*;
import java.util.*;
import java.beans.*;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 * @version $Revision: 1.1 $
 */
abstract public class AbstractValidationPanel extends GenericDOMEditPanel {
    private LocalFocusListener  focusListener = null;
    private LocalChangeListener changeListener = null;
    private LocalCancelListener cancelListener = null;
    private JTabbedPane         tabbedPane = null;
    private JButton cancelButton = null;
    private boolean             inWarning = false;
    private boolean             cancelled = false;
    private int                 index = 0;

    /**
     * Constructor declaration
     *
     *
     * @param node
     *
     * @see
     */
    public AbstractValidationPanel(MerlotDOMNode node) {
        super(node);
        pmInit();
    }

    /**
     * Method declaration
     *
     *
     * @exception ValidationException
     *
     * @see
     */
    public void validateValues() throws ValidationException {
      if (! cancelled) {
        validateComponents(this);
      }
    }

    private void validateComponents(Container cont) throws ValidationException {
      int count = cont.getComponentCount();
      for (int i = 0 ; i < count ; i++ ) {
         Component cursor = null;
         cursor = cont.getComponent(i);
         if (cursor instanceof Validator) {
           Validator validator = (Validator) cursor;
           validator.validateValue();
         }
         if (cursor instanceof Container) {
           validateComponents((Container)cursor);
         }
      }
    }


    /**
     * Method declaration
     *
     *
     * @see
     */
    public void grabFocus() {
        if (tabbedPane.getTabCount() > index) {
          tabbedPane.setSelectedIndex(index);
          super.grabFocus();
        }
    }

    /**
     * Method declaration
     *
     *
     * @exception PropertyVetoException
     *
     * @see
     */
    public void save() throws PropertyVetoException {
        try {
           validateValues();
           super.save();
        } catch (ValidationException e) {
          PropertyVetoException pv = null;
          pv = new PropertyVetoException(
             e.getMessage(), e.getPropertyChangeEvent());
          throw pv;
        }
    }

    public void firePanelSavedEvent (String changedNodeType) {
	Vector listeners = new Vector();

	JTabbedPane tabbedPane = (JTabbedPane) getParent().getParent();
	if (tabbedPane == null) {
		return;
	}

	Component[] tabs = tabbedPane.getComponents();
	for (int i = 0; i < tabs.length; i++) {
		Component[] tabComponents = ((JPanel) tabs[i]).getComponents();
		for (int j = 0; j < tabComponents.length; j++) {
			if (tabComponents[j] instanceof GenericDOMEditPanel) {
				Component[] widgets = ((JPanel) tabComponents[j]).getComponents();
				for (int k = 0; k < widgets.length; k++) {
					if (widgets[k] instanceof SaveListener) {
						listeners.add(widgets[k]);
					}
				}
			}
		}
	}

	for (int i = 0; i < listeners.size(); i++) {
		((SaveListener) listeners.elementAt(i)).panelSaved(changedNodeType);
	}
    }

    public void saveValue(HashMap map) throws PropertyVetoException {
      super.save(map);
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void pmInit() {
        initFocusListener();
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    private boolean isValueSetValid() {
        boolean valid = false;

        if (!inWarning) {
            try {
                validateValues();
                valid = true;
            } catch (ValidationException ex) {
                inWarning = true;
                grabFocus();    // Always get focus before
                warn(ex);       // opening alert dialog
                inWarning = false;
            }
        }
        return valid;
    }


    /**
     * Method declaration
     *
     *
     * @see
     */
    private void initTabListener() {
        findTabbedPane();
        if (tabbedPane != null) {
            index = tabbedPane.getSelectedIndex();
            changeListener = new LocalChangeListener();
            tabbedPane.addChangeListener(changeListener);
            findCancelButton(tabbedPane);
        }
        if (cancelButton != null) {
          cancelListener = new LocalCancelListener();
          cancelButton.addActionListener(cancelListener);
        }
    }

    private void findTabbedPane() {
        Container parent = null;
        JPanel    editPanel = null;

        parent = getParent();
        while (parent != null) {
            if (parent.getParent() instanceof JTabbedPane
                    && parent instanceof JPanel) {
                editPanel = (JPanel) parent;
                tabbedPane = (JTabbedPane) parent.getParent();
                parent = null;
            } else {
                parent = parent.getParent();
            }
        }
    }

    private void findCancelButton(Container cont) {
      int count = cont.getComponentCount();
      for (int i = 0 ; i < count ; i++ ) {
        Component comp = null;
        comp = cont.getComponent(i);
        if (comp instanceof JButton) {
          JButton button = (JButton) comp;
          String cancelText = MerlotResource.getString(UI,"cancel.button.text").trim();
          if (button.getText().trim().equalsIgnoreCase(cancelText)) {
            cancelButton = button;
            break;
          }
        } else if (comp instanceof Container) {
          findCancelButton((Container)comp);
        }
      }
    }


    /**
     * Method declaration
     *
     *
     * @see
     */
    private void initFocusListener() {
        focusListener = new LocalFocusListener();
        addFocusListenerToChildren(this);
    }

    private void addFocusListenerToChildren(Container cont) {
      int count = cont.getComponentCount();
      for (int i = 0 ; i < count ; i++ ) {
         Component cursor = null;
         cursor = cont.getComponent(i);
         if (cursor instanceof JTextArea) {
           cursor.addFocusListener(focusListener);
         } else if (cursor instanceof JTextField) {
           cursor.addFocusListener(focusListener);
         } else if (cursor instanceof Container) {
           addFocusListenerToChildren((Container)cursor);
         }
      }
    }


    /**
     * Method declaration
     *
     *
     * @param e
     *
     * @see
     */
    private void warn(ValidationException e) {
        if (tabbedPane.getTabCount() > index) {
          JOptionPane.showMessageDialog(this, e.getMessage(),
                                        "DODS Validation Exception",
                                        JOptionPane.WARNING_MESSAGE);
        }
    }

    /**
     * Class declaration
     *
     *
     * @author Paul Mahar
     * @version $Revision: 1.1 $
     */
    class LocalChangeListener implements ChangeListener {

        /**
         * Method declaration
         *
         *
         * @param e
         *
         * @see
         */
        public void stateChanged(ChangeEvent e) {
            if (tabbedPane.getSelectedIndex() != index) {
              if (tabbedPane.getTabCount() > index) {
                isValueSetValid();
              }
            }
        }

    }

    class LocalCancelListener implements ActionListener {
      public void actionPerformed(ActionEvent event) {
        cancelled = true;
      }
    }

    /**
     * Class declaration
     *
     *
     * @author Paul Mahar
     * @version $Revision: 1.1 $
     */
    class LocalFocusListener implements FocusListener {

        /**
         * Method declaration
         *
         *
         * @param e
         *
         * @see
         */
        public void focusGained(FocusEvent e) {
            cancelled = false;
            if (tabbedPane == null) {
                initTabListener();
            }
        }

        /**
         * Method declaration
         *
         *
         * @param e
         *
         * @see
         */
        public void focusLost(FocusEvent e) {}

    }
}

