/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: GeneratorDialog.java,v 1.1 2001/09/28 16:01:07 cryd0221 Exp $
 */
package org.enhydra.dods.merlot;

// DODS imports
import org.enhydra.dods.builder.engine.Builder;
import org.enhydra.dods.store.doml.DOMLConstants;
import org.enhydra.dods.builder.generator.base.SourceGenerator;

// Swing imports
import javax.swing.*;
import javax.swing.border.*;

// AWT imports
import java.awt.*;
import java.awt.event.*;

// Merlot imports
import org.merlotxml.merlot.XMLEditorDoc;
import org.merlotxml.merlot.XMLEditorFrame;

// Standard imports
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Properties;
import java.util.Enumeration;
import java.util.Vector;

public class GeneratorDialog extends JDialog implements ActionListener, ItemListener, DOMLConstants {

	// GUI constants
	private final static String CHECK_BOX_TEXT = "Overwrite";
	private final static String DIRECTORY_LABEL_TEXT = "Directory";
        private final static String DB_TYPE_LABEL_TEXT = "Database Type:";
	private final static String BROWSE_BUTTON_TEXT = "Browse ...";
        private final static String OPTION_PANEL_TEXT = "Database Manager";
	private final static String OK_BUTTON_TEXT = "Ok";
        private final static String CANCEL_BUTTON_TEXT = "Cancel";
        private final static String ENHYDRA4_RADIO_TEXT = "EAS 4.x / JDBC";
	private final static String ENHYDRA3_RADIO_TEXT = "Enhydra 3.x";
        private final static String DATA_SOURCE_CHECK_TEXT = "Use Data Sources";
        private final static String EJB_CHECK_TEXT = "Generate EJB";

	// Generation constants
	private final static String TITLE = "Generate Java Source";
  	private final static String DODS_HOME_KEY = "DODS_HOME";
  	private final static String ENHYDRA_DIR_KEY = "ENHYDRA_DIR";
  	private final static String FILE_SEPARATOR = "file.separator";
	private final static String USER_HOME = "user.home";
	private final static String PROPS_HOME = ".enhydra";
	private final static String DODSUI_PROPS_FILENAME = "dodsui.properties";
	private final static String PROP_FILE_HEADER = "DODS UI GeneratorDialog Properties";
	private final static String PATH_KEY = ".generatorpath";
        private final static String OVERWRITE_KEY = ".overwrite";
	private final static String DB_VENDOR_KEY = ".dbvendor";
	private final static String ENHYDRA4_KEY = ".enhydra4";
        private final static String ENHYDRA3_KEY = ".enhydra3";
        private final static String DATA_SOURCES_KEY = ".datasources";
        private final static String GEN_EJB_KEY = ".generateejb";
        private final static String TRUE_TEXT = "true";
        private final static String FALSE_TEXT = "false";
  	private final static String WINDOWS_FILE_SEPARATOR = "\\";
	private final static String _NX_FILE_SEPARATOR = "/";
  	private final static String PARENT_DIRECTORY = "..";
  	private final static String DODS_DIRECTORY = "dods";
	private final static String REGEN_OPTION = "regen";
	private final static String ENHYDRA3_OPTION = "-enhydra3";
	private final static String ENHYDRA4_OPTION = "-enhydra4";
	private final static String JDBC1_OPTION = "-jdbc1";
	private final static String JDBC2_OPTION = "-jdbc2";
	private final static String GENERATE_EJB_OPTION = "-ejb";
	private final static String VENDOR_OPTION_PREFIX = "-vendor=";
	public final static String ERROR_DIALOG_TITLE = "DODS Generator";
	private final static String NO_PATH_ERROR_MESSAGE = "Please specify a path.";
	public final static String NO_DOCUMENT_ERROR_MESSAGE = "Please open a file to generate";
	private final static String NONEXISTANT_PATH_ERROR_MESSAGE
		= "The specified path does not exist.\nPlease try again.";
	private final static String OVERWRITE_WARNING_MESSAGE
		= "You are about to delete the entire \ncontents of the target directory.\nIs this OK?";
	private final static String CANNOT_OVERWRITE_MESSAGE
		= "Cannot overwrite the specified directory.\nPlease clear the directory or choose another.";
	private final static String FILE_SELECTED_MESSAGE
		= "The specified path is an existing file.\nPlease specify a different path and try again.";
	private final static String PROPS_FILE_READ_ERROR
		= "***** Exception encountered while loading properties file in GeneratorDialog: ";
	private final static String PROPS_FILE_WRITE_ERROR
		= "***** Exception encountered while writing properties file in GeneratorDialog: ";
	private final static String INPUT_STREAM_CREATION_ERROR
		= "***** Unable to create FileInputStream in GeneratorDialog: ";
	private final static String OUTPUT_STREAM_CREATION_ERROR
		= "***** Unable to create FileOutputStream in GeneratorDialog: ";
	private final static String INPUT_STREAM_CLOSE_ERROR
		= "***** Exception encountered while closing FileInputStream in GeneratorDialog: ";
	private final static String OUTPUT_STREAM_CLOSE_ERROR
		= "***** Exception encountered while closing FileOutputStream in GeneratorDialog: ";
	private final static String GENERATION_COMPLETED = "\n\n***** DODS GENERATION COMPLETED ... \n";

	private JTextField pathField;
	private JPanel upperPanel, overwritePanel, pathPanel,
	               buttonPanel, dbTypePanel, optionPanel, optionContentPanel,
	               useDataSourcesPanel, generateEjbPanel, enhydra4Panel, enhydra3Panel;
	private JLabel directoryLabel, dbTypeLabel;
	private JCheckBox overwriteCheckBox, useDataSourcesCheckBox, generateEjbCheckBox;
        private JComboBox dbTypeCombo;
	private JButton okButton, cancelButton, browseButton;
        private Border optionBorder;
        private TitledBorder titledOptionBorder;
        private JRadioButton enhydra4Radio, enhydra3Radio;
        private ButtonGroup versionButtonGroup;

	public GeneratorDialog(Frame parent) {
		super(parent, TITLE, true);

		Container c = getContentPane();
		//c.setLayout(new GridLayout(5, 1, 5, 5));
		c.setLayout(new BoxLayout(c, BoxLayout.Y_AXIS));
		c.add(Box.createVerticalStrut(15));

		upperPanel = new JPanel();
		upperPanel.setLayout(new GridLayout(1, 1, 5, 5));
		c.add(upperPanel);

		c.add(Box.createVerticalStrut(5));

		overwritePanel = new JPanel();
		overwritePanel.setLayout(new BoxLayout(overwritePanel, BoxLayout.X_AXIS));
		//overwritePanel.setLayout(new GridLayout(1, 1, 0, 0));
		c.add(overwritePanel);

		c.add(Box.createVerticalStrut(5));

		dbTypePanel = new JPanel();
		dbTypePanel.setLayout(new BoxLayout(dbTypePanel, BoxLayout.X_AXIS));
		c.add(dbTypePanel);

		c.add(Box.createVerticalStrut(5));

		optionPanel = new JPanel();
		optionPanel.setLayout(new BoxLayout(optionPanel, BoxLayout.X_AXIS));
		c.add(optionPanel);

		c.add(Box.createVerticalStrut(5));

		buttonPanel = new JPanel();
		c.add(buttonPanel);

		c.add(Box.createVerticalStrut(15));

		pathPanel = new JPanel();
		pathPanel.setLayout(new BoxLayout(pathPanel, BoxLayout.X_AXIS));
		pathField = new JTextField(20);
		//pathField.setText(populatePathField());
		browseButton = new JButton(BROWSE_BUTTON_TEXT);
		browseButton.addActionListener(this);
		directoryLabel = new JLabel(DIRECTORY_LABEL_TEXT);
		pathPanel.add(Box.createHorizontalStrut(10));
		pathPanel.add(directoryLabel);
		pathPanel.add(Box.createHorizontalStrut(10));
		pathPanel.add(pathField);
		pathPanel.add(Box.createHorizontalStrut(10));
		pathPanel.add(browseButton);
		pathPanel.add(Box.createHorizontalStrut(10));
		upperPanel.add(pathPanel);

		overwriteCheckBox = new JCheckBox(CHECK_BOX_TEXT, true);
		overwritePanel.add(Box.createHorizontalStrut(10));
		overwritePanel.add(overwriteCheckBox);

		dbTypeLabel = new JLabel(DB_TYPE_LABEL_TEXT);
		dbTypeCombo = new JComboBox();
		populateDbTypeCombo();
		dbTypePanel.add(Box.createHorizontalStrut(10));
		dbTypePanel.add(dbTypeLabel);
		dbTypePanel.add(Box.createHorizontalStrut(10));
		dbTypePanel.add(dbTypeCombo);
		dbTypePanel.add(Box.createHorizontalStrut(90));

		optionBorder = BorderFactory.createEtchedBorder();
		titledOptionBorder = new TitledBorder(optionBorder, OPTION_PANEL_TEXT);
		enhydra4Radio = new JRadioButton(ENHYDRA4_RADIO_TEXT);
		enhydra4Radio.addItemListener(this);
		enhydra3Radio = new JRadioButton(ENHYDRA3_RADIO_TEXT);
		enhydra3Radio.addItemListener(this);
		versionButtonGroup = new ButtonGroup();
		versionButtonGroup.add(enhydra4Radio);
		versionButtonGroup.add(enhydra3Radio);
		enhydra4Panel = new JPanel();
		enhydra4Panel.setLayout(new BoxLayout(enhydra4Panel, BoxLayout.X_AXIS));
		enhydra4Panel.add(Box.createHorizontalStrut(10));
		enhydra4Panel.add(enhydra4Radio);
		enhydra3Panel = new JPanel();
		enhydra3Panel.setLayout(new BoxLayout(enhydra3Panel, BoxLayout.X_AXIS));
		enhydra3Panel.add(Box.createHorizontalStrut(10));
		enhydra3Panel.add(enhydra3Radio);
		useDataSourcesCheckBox = new JCheckBox(DATA_SOURCE_CHECK_TEXT);
		useDataSourcesPanel = new JPanel();
		useDataSourcesPanel.setLayout(new BoxLayout(useDataSourcesPanel, BoxLayout.X_AXIS));
		useDataSourcesPanel.add(Box.createHorizontalStrut(30));
		useDataSourcesPanel.add(useDataSourcesCheckBox);
		generateEjbCheckBox = new JCheckBox(EJB_CHECK_TEXT);
		generateEjbPanel = new JPanel();
		generateEjbPanel.setLayout(new BoxLayout(generateEjbPanel, BoxLayout.X_AXIS));
		generateEjbPanel.add(Box.createHorizontalStrut(30));
		generateEjbPanel.add(generateEjbCheckBox);
		optionContentPanel = new JPanel();
		optionContentPanel.setLayout(new BoxLayout(optionContentPanel, BoxLayout.Y_AXIS));
		optionContentPanel.setLayout(new GridLayout(4, 1, 0, 0));
		optionContentPanel.add(enhydra4Panel);
		optionContentPanel.add(useDataSourcesPanel);
		optionContentPanel.add(generateEjbPanel);
		optionContentPanel.add(enhydra3Panel);
		optionContentPanel.setBorder(titledOptionBorder);
		optionPanel.add(optionContentPanel);
		enhydra3Radio.setSelected(true);

		okButton = new JButton(OK_BUTTON_TEXT);
		okButton.addActionListener(this);
		cancelButton = new JButton(CANCEL_BUTTON_TEXT);
		cancelButton.addActionListener(this);
		buttonPanel.add(okButton);
		buttonPanel.add(Box.createHorizontalStrut(10));
		buttonPanel.add(cancelButton);

		prepopulateFromProps();

		//setSize(400, 150);
		setSize(500, 310);
		setResizable(false);
		show();
	}

        private void populateDbTypeCombo() {
	    for(int i = 0; i < VENDOR_ATTRIBUTE_LIST.length; i++) {
		//if (!VENDOR_ATTRIBUTE_LIST[i].equals(DEFAULT)) {
		    dbTypeCombo.addItem(VENDOR_ATTRIBUTE_LIST[i]);
		//}
	    }
        }

	private void prepopulateFromProps() {
		String pathText = new String("");
		String overwrite = new String("");
		String dbVendorName = new String("");
		String enhydra4 = new String("");
		String enhydra3 = new String("");
		String dataSources = new String("");
		String generateEJB = new String("");

		String pathToPropsFile
			= System.getProperty(USER_HOME) + File.separator
			+ PROPS_HOME + File.separator + DODSUI_PROPS_FILENAME;
		File f = new File(pathToPropsFile);
		if (f.exists()) {
			//read
			FileInputStream in = null;

			try {
				in = new FileInputStream(f);
			} catch (FileNotFoundException fnfe) {
				System.err.println(INPUT_STREAM_CREATION_ERROR);
				fnfe.printStackTrace();
				try {
					if (in != null) {
						in.close();
					}
				} catch (IOException ioe) {
					System.err.println(INPUT_STREAM_CLOSE_ERROR + ioe);
					ioe.printStackTrace();
				}
			}

			Properties props = new Properties();

			try {
				props.load(in);
			} catch (IOException ioe) {
				System.err.println(PROPS_FILE_READ_ERROR + ioe);
				ioe.printStackTrace();
			} finally {
				try {
					if (in != null) {
						in.close();
					}
				} catch (IOException ioe) {
					System.err.println(INPUT_STREAM_CLOSE_ERROR + ioe);
					ioe.printStackTrace();
				}
			}

			XMLEditorDoc doc = XMLEditorFrame.getSharedInstance().getCurrentDocument();
			String filename;
			if (doc == null) {
				return;
			} else {
				filename = doc.getFile().getPath();
			}

			pathText = props.getProperty(filename + PATH_KEY);
			if (pathText == null) {
				pathText = "";
			}
			pathField.setText(pathText);

			overwrite = props.getProperty(filename + OVERWRITE_KEY);
		        if (overwrite == null) {
			    overwriteCheckBox.setSelected(true);
			} else {
			    if (overwrite.equals(TRUE_TEXT)) {
				overwriteCheckBox.setSelected(true);
			    } else if (overwrite.equals(FALSE_TEXT)) {
				overwriteCheckBox.setSelected(false);
			    }
			}

			dbVendorName = props.getProperty(filename + DB_VENDOR_KEY);
			if (dbVendorName != null) {
			    dbTypeCombo.setSelectedItem(dbVendorName);
			}

			enhydra4 = props.getProperty(filename + ENHYDRA4_KEY);
			if (enhydra4 != null) {
			    if (enhydra4.equals(TRUE_TEXT)) {
				enhydra4Radio.setSelected(true);
			    } else if (enhydra4.equals(FALSE_TEXT)) {
				enhydra4Radio.setSelected(false);
			    }
			}

			enhydra3 = props.getProperty(filename + ENHYDRA3_KEY);
			if (enhydra3 != null) {
			    if (enhydra3.equals(TRUE_TEXT)) {
				enhydra3Radio.setSelected(true);
			    } else if (enhydra3.equals(FALSE_TEXT)) {
				enhydra3Radio.setSelected(false);
			    }
			}

			dataSources = props.getProperty(filename + DATA_SOURCES_KEY);
			if (dataSources != null) {
			    if (dataSources.equals(TRUE_TEXT)) {
				useDataSourcesCheckBox.setSelected(true);
			    } else if (dataSources.equals(FALSE_TEXT)) {
				useDataSourcesCheckBox.setSelected(false);
			    }
			}

			generateEJB = props.getProperty(filename + GEN_EJB_KEY);
			if (generateEJB != null) {
			    if (generateEJB.equals(TRUE_TEXT)) {
				generateEjbCheckBox.setSelected(true);
			    } else if (generateEJB.equals(FALSE_TEXT)) {
				generateEjbCheckBox.setSelected(false);
			    }
			}
		}
	}

        public void itemStateChanged(ItemEvent e) {
	    if (e.getSource() == enhydra4Radio) {
		if (e.getStateChange() == ItemEvent.SELECTED) {
		    useDataSourcesCheckBox.setEnabled(true);
		    generateEjbCheckBox.setEnabled(true);
		}
	    } else if (e.getSource() == enhydra3Radio) {
		if (e.getStateChange() == ItemEvent.SELECTED) {
		    useDataSourcesCheckBox.setEnabled(false);
		    generateEjbCheckBox.setEnabled(false);
		}
	    }
        }

	public void actionPerformed(ActionEvent e) {
	        if (e.getSource() == cancelButton) {
		    dispose();
	        } else if (e.getSource() == browseButton) {
			JFileChooser chooser = new JFileChooser();
			chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
			int returnVal = chooser.showOpenDialog(this);
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				pathField.setText(chooser.getSelectedFile().getPath());
			}
		} else if (e.getSource() == okButton) {
		        String pathText = null;
			XMLEditorDoc doc = XMLEditorFrame.getSharedInstance().getCurrentDocument();
			if (doc != null) {
				pathText = pathField.getText();
				if ((pathText != null) && (pathText.length() > 0) && (!pathText.equals(""))) {
					pathText = trimPathName(pathText);  // trim
					pathField.setText(pathText);		//trim
					if (!fileSelected()) {
						if (writeOk()) {
							try {
								buildSource(pathText, doc.getFile().getPath());
								System.out.println(GENERATION_COMPLETED);
								dispose();
							} catch (FileNotFoundException fnfe) {
								showGeneratorErrorDialog(NONEXISTANT_PATH_ERROR_MESSAGE);
								pathField.setText("");
							}
						} else {
							showGeneratorErrorDialog(CANNOT_OVERWRITE_MESSAGE);
						}
					} else {
							showGeneratorErrorDialog(FILE_SELECTED_MESSAGE);
					}
				} else {
					showGeneratorErrorDialog(NO_PATH_ERROR_MESSAGE);
				}
			} else {
				showGeneratorErrorDialog(NO_DOCUMENT_ERROR_MESSAGE);
				dispose();
			}
		}
	}

	private boolean fileSelected() {
		boolean fs = true;
		File f = new File(pathField.getText());
		if (f.exists()) {
			if (f.list() != null) {
				// f.list()==null means that f is a [existing] file
				fs = false;
			}
		} else {
			fs = false;
		}
		return fs;
	}

	private boolean writeOk() {
		boolean ok = false;
		if (!overwriteCheckBox.isSelected()) {
			File f = new File(pathField.getText());
			if (f.exists()) {
				String[] directoryContents = f.list();
				if (directoryContents == null) {
					return false;
				} else if (directoryContents.length == 0) {
					ok = true;
				}
			} else {
				ok = true;
			}
		} else {
			int option = JOptionPane.showConfirmDialog(this, OVERWRITE_WARNING_MESSAGE, TITLE,
				JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE);
			if (option == JOptionPane.YES_OPTION) {
				ok = true;
			}
		}
		return ok;
	}

	// This method will trim all characters after and including the last slash
	private String trimLastDirectory(String pathName) {
		String trimmedPathName = new String();
		int index = -1;

		if ((File.separator).equals(WINDOWS_FILE_SEPARATOR)) {
			index = pathName.lastIndexOf(WINDOWS_FILE_SEPARATOR );
		} else {
			index = pathName.lastIndexOf(_NX_FILE_SEPARATOR);
		}

		if (index > -1) {
			char[] frag = new char[index];
			pathName.getChars(0, index, frag, 0);
			trimmedPathName = new String(frag);
		} else {
			trimmedPathName = pathName;
		}
		return trimmedPathName;
	}

	private String trimPathName(String pathName) {
		String trimmedName = new String();
		if (pathName != null) {
			trimmedName = pathName.trim();
			if (trimmedName.length() > 1) { // this allows for a path of / on *NX systems
				if (trimmedName.lastIndexOf(File.separator) == trimmedName.length() - 1) {
						trimmedName = trimLastDirectory(trimmedName);
				}
			}
		}
		return trimmedName;
	}


	private void buildSource(String path, String file) throws FileNotFoundException {
		String dodsHomeValue = new String();	// ends up as ../../dods
		String enhydraDirValue = new String();	// ends up as ../../

		saveToPropsFile();

		// Call builder to generate Java code
		try {
			/****************** OLD WAY TO SET dodsHomeValue AND enhydraDirValue ****************
			String fileSeparator = System.getProperty(FILE_SEPARATOR);
			if (fileSeparator.equals(WINDOWS_FILE_SEPARATOR)) {
				dodsHomeValue = PARENT_DIRECTORY + fileSeparator + fileSeparator
						  + PARENT_DIRECTORY + fileSeparator + fileSeparator + DODS_DIRECTORY;
				enhydraDirValue = PARENT_DIRECTORY + fileSeparator + fileSeparator
						    + PARENT_DIRECTORY + fileSeparator + fileSeparator;
			} else {
				dodsHomeValue = PARENT_DIRECTORY + fileSeparator
						  + PARENT_DIRECTORY + fileSeparator + DODS_DIRECTORY;
				enhydraDirValue = PARENT_DIRECTORY + fileSeparator
						    + PARENT_DIRECTORY + fileSeparator;
			}
			**********************************/

			// Start new way
			if ((File.separator).equals(WINDOWS_FILE_SEPARATOR)) {
				dodsHomeValue = PARENT_DIRECTORY + File.separator + File.separator
					+ PARENT_DIRECTORY + File.separator + File.separator + DODS_DIRECTORY;
				enhydraDirValue = PARENT_DIRECTORY + File.separator + File.separator
   					+ PARENT_DIRECTORY + File.separator + File.separator;
			} else {
				dodsHomeValue = PARENT_DIRECTORY + File.separator
					+ PARENT_DIRECTORY + File.separator + DODS_DIRECTORY;
				enhydraDirValue = PARENT_DIRECTORY + File.separator
				    	+ PARENT_DIRECTORY + File.separator;
			}
			// End new way


			System.setProperty(DODS_HOME_KEY, dodsHomeValue);
			System.setProperty(ENHYDRA_DIR_KEY, enhydraDirValue);

			BuilderMessageFrame msgFrame = new BuilderMessageFrame();
			msgFrame.addWindowListener(
				new WindowAdapter() {
					public void WindowClosing(WindowEvent e) {
						System.exit(0);
					}
				}
			);

			clearTargetDirectory(path);
			Builder builder = new Builder();
			builder.setMessageListener(msgFrame);

			// build cmdArray
			Vector args = new Vector();
			args.add(file);
			args.add(path);
			args.add(REGEN_OPTION);
			if (!((String) dbTypeCombo.getSelectedItem()).equals(DEFAULT)) {
				args.add(VENDOR_OPTION_PREFIX + (String) dbTypeCombo.getSelectedItem());
			}

			if (enhydra3Radio.isSelected()) {
				args.add(ENHYDRA3_OPTION);
			} else if (enhydra4Radio.isSelected()) {
				args.add(ENHYDRA4_OPTION);
				if (useDataSourcesCheckBox.isSelected()) {
					args.add(JDBC2_OPTION);
				} else {
					args.add(JDBC1_OPTION);
				}

				if (generateEjbCheckBox.isSelected()) {
					args.add(GENERATE_EJB_OPTION);
				}
			}

System.out.println("Command line args:");
			String[] cmdArray = new String[args.size()];
			for (int i = 0; i < cmdArray.length; i++) {
				cmdArray[i] = (String) args.elementAt(i);
System.out.println("\t" + cmdArray[i]);
			}
			builder.main(cmdArray);

			// Attempt at fixing a Builder state bug:
			//System.getProperties().setProperty( SourceGenerator.DB_VENDOR, null );

		} catch (FileNotFoundException fnfe) {
			throw new FileNotFoundException();
		} catch (Exception ex) {
			System.out.println("***** Error while executing Builder: " + ex);
			ex.printStackTrace();
		}
	}

	private void saveToPropsFile() {
		String pathToProps = System.getProperty(USER_HOME) + File.separator + PROPS_HOME;
		File f = new File(pathToProps);
		if (!f.exists()) {
			f.mkdirs();
		}
		String pathToPropsFile = pathToProps + File.separator + DODSUI_PROPS_FILENAME;
		f = new File(pathToPropsFile);

		Properties props = new Properties();
		if (f.exists()) {
			FileInputStream in = null;

			try {
				in = new FileInputStream(f);
			} catch (FileNotFoundException fnfe) {
				System.err.println(INPUT_STREAM_CREATION_ERROR);
				fnfe.printStackTrace();
				try {
					if (in != null) {
						in.close();
					}
				} catch (IOException ioe) {
					System.err.println(INPUT_STREAM_CLOSE_ERROR + ioe);
					ioe.printStackTrace();
				}

			}

			Properties defaults = new Properties();

			try {
				defaults.load(in);
			} catch (IOException ioe) {
				System.err.println(PROPS_FILE_READ_ERROR + ioe);
				ioe.printStackTrace();
			} finally {
				try {
					if (in != null) {
						in.close();
					}
				} catch (IOException ioe) {
					System.err.println(INPUT_STREAM_CLOSE_ERROR + ioe);
					ioe.printStackTrace();
				}
			}

			Enumeration defaultList = defaults.propertyNames();
			while (defaultList.hasMoreElements()) {
				String key = (String) defaultList.nextElement();
				props.setProperty(key, defaults.getProperty(key));
			}
		}

		FileOutputStream out = null;

		try {
			out = new FileOutputStream(f);
		} catch (FileNotFoundException fnfe) {
			System.err.println(OUTPUT_STREAM_CREATION_ERROR);
			fnfe.printStackTrace();
			try {
				if (out != null) {
					out.close();
				}
			} catch (IOException ioe) {
				System.err.println(OUTPUT_STREAM_CLOSE_ERROR + ioe);
				ioe.printStackTrace();
			}
		}

		XMLEditorDoc doc = XMLEditorFrame.getSharedInstance().getCurrentDocument();
		String filename;
		if (doc == null) {
			return;
		} else {
			filename = doc.getFile().getPath();
		}

		// set all of the components' values into props object
		props.setProperty(filename + PATH_KEY, pathField.getText());

		if (overwriteCheckBox.isSelected()) {
		    props.setProperty(filename + OVERWRITE_KEY, TRUE_TEXT);
		} else {
		    props.setProperty(filename + OVERWRITE_KEY, FALSE_TEXT);
		}

		props.setProperty(filename + DB_VENDOR_KEY, (String) dbTypeCombo.getSelectedItem());

		if (enhydra4Radio.isSelected()) {
		    props.setProperty(filename + ENHYDRA4_KEY, TRUE_TEXT);
		} else {
		    props.setProperty(filename + ENHYDRA4_KEY, FALSE_TEXT);
		}

		if (enhydra3Radio.isSelected()) {
		    props.setProperty(filename + ENHYDRA3_KEY, TRUE_TEXT);
		} else {
		    props.setProperty(filename + ENHYDRA3_KEY, FALSE_TEXT);
		}

		if (useDataSourcesCheckBox.isSelected()) {
		    props.setProperty(filename + DATA_SOURCES_KEY, TRUE_TEXT);
		} else {
		    props.setProperty(filename + DATA_SOURCES_KEY, FALSE_TEXT);
		}

		if (generateEjbCheckBox.isSelected()) {
		    props.setProperty(filename + GEN_EJB_KEY, TRUE_TEXT);
		} else {
		    props.setProperty(filename + GEN_EJB_KEY, FALSE_TEXT);
		}

		// store the properties to disk
		try {
			props.store(out, PROP_FILE_HEADER);
		} catch (IOException ioe) {
			System.err.println(PROPS_FILE_WRITE_ERROR + ioe);
			ioe.printStackTrace();
		} finally {
			try {
				if (out != null) {
					out.close();
				}
			} catch (IOException ioe) {
				System.err.println(OUTPUT_STREAM_CLOSE_ERROR + ioe);
				ioe.printStackTrace();
			}
		}
	}

	private void clearTargetDirectory(String targetDirectoryName) {
		File targetDirectory = new File(targetDirectoryName);
		if (targetDirectory.exists()) {
			File[] files = targetDirectory.listFiles();
			if (files != null) {
				for (int i = 0; i < files.length; i++) {
					delete(files[i]);
				}
			}
		}
	}

	private void delete(File root) {
		File[] children = root.listFiles();
		if (children != null) { 		// root.listFiles()==null means that root is a file
			if (children.length != 0) {	// root.listFiles().length==0 means that it is an empty directory
				for (int i = 0; i < children.length; i++) {
					if (children[i].isDirectory()) {
						delete(children[i]);
					} else if (children[i].isFile()) {
						children[i].delete();
					}
				}
			}
		}
		root.delete();
	}

	private void showGeneratorErrorDialog(String msg) {
		JOptionPane.showMessageDialog(this, msg, ERROR_DIALOG_TITLE, JOptionPane.ERROR_MESSAGE);

	}
}
