/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: NodeUtil.java,v 1.1 2001/09/28 16:01:07 cryd0221 Exp $
 */

package org.enhydra.dods.merlot;

import org.merlotxml.merlot.MerlotDOMNode;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import java.util.Vector;
import org.enhydra.dods.store.doml.DOMLConstants;

public class NodeUtil {
   private Vector foundVector = new Vector();

   public MerlotDOMNode[] findNodesByName(MerlotDOMNode startNode, String findName) {
	MerlotDOMNode   root = null;
	MerlotDOMNode[] foundNodes = new MerlotDOMNode[0];

	foundVector.removeAllElements();

	root = startNode.getParentNode();
	if (root != null) {					// OJ
		while (root.getParentNode() != null) {
	    		if (root == null) {			// OJ
				break;				// OJ
	    		}						// OJ
	    		root = root.getParentNode();
		}
	}								// OJ

	addFoundNodesToVector(root, findName);

	foundNodes = (MerlotDOMNode[]) foundVector.toArray(foundNodes);

	foundVector.removeAllElements();

	return foundNodes;
    }

    /**
     * Method declaration
     *
     *
     * @param parent
     *
     * @see
     */
    private void addFoundNodesToVector(MerlotDOMNode parent, String findName) {
	MerlotDOMNode[] children = new MerlotDOMNode[0];

	if (parent != null) {
	    children = parent.getChildNodes();

	    for (int i = 0; i < children.length; i++) {
                String nodeName = new String();
                nodeName = children[i].getNodeName().trim();
                if (nodeName.equalsIgnoreCase(findName)) {
		    foundVector.addElement(children[i]);
		}
		addFoundNodesToVector(children[i], findName);
	    }
	}
    }

    public void storeAttributeValue(MerlotDOMNode node,
                  String atName, String value) {
        NamedNodeMap  attributes = null;
        Node item = null;
        attributes = node.getAttributes();
        item = attributes.getNamedItem(atName);
        if (item != null) {
          item.setNodeValue(value);
        }
    }

    public String lookupAttributeValue(MerlotDOMNode node, String atName) {
        NamedNodeMap  attributes = null;
        Node item = null;
        String value = null;
        attributes = node.getAttributes();
        item = attributes.getNamedItem(atName);
        if (item != null) {
          value = item.getNodeValue();
        }
        return value;
    }


    /**
     * Method declaration
     *
     *
     * @param value
     *
     * @see
     */
	public MerlotDOMNode findParentVendorNode(MerlotDOMNode startNode) {

		MerlotDOMNode foundNode = null;
	     	MerlotDOMNode parentNode = startNode.getParentNode();

		if (parentNode != null) {
			if (parentNode.getNodeName().equals(DOMLConstants.VENDOR)) {
				foundNode = parentNode;
			} else if (parentNode.getNodeName().equals(DOMLConstants.DOML)) {
				// skip - let null be returned
			} else {
				foundNode = findParentVendorNode(parentNode);
			}
		} else {
			// skip - let null be returned
		}
		return foundNode;
	}



  public String expandPackageName(MerlotDOMNode node) {
     StringBuffer buf = new StringBuffer();
     String fullName = new String();
     MerlotDOMNode parent = null;
     parent = node.getParentNode();
     while (parent != null && parent.getNodeName().equalsIgnoreCase("package")) {
        String packName = new String();
        packName = lookupAttributeValue(parent,"name");
        if (packName != null) {
		buf.insert(0, ".");
		buf.insert(0, packName);
          //buf.append(packName);
          //buf.append('.');
        }
        parent = parent.getParentNode();
     }

     fullName = lookupAttributeValue(node, "name");
     if (fullName != null) {
        buf.append(fullName);
     }
     return buf.toString();
  }



	public String collapsePackageName(String expandedName) {
		String collapsedName = null;
		int index = -1;
		if (expandedName != null) {
			index = expandedName.lastIndexOf(DOMLConstants.PACKAGE_SEPARATOR);
		} else {
			return "";
		}
	
		if (index > -1) {
			char[] strFrag = new char[expandedName.length()-index-1];
			expandedName.getChars(index+1, expandedName.length(), strFrag, 0);
			collapsedName = new String(strFrag);		
		} else {
			collapsedName = expandedName;
		}
		return collapsedName;
	}




  public String createUniqueSibling(MerlotDOMNode node, String attribute, String root) {
        int             count = 1;
        String          base =  root;
        String          unique = base + count;
        MerlotDOMNode   parent = null;
        MerlotDOMNode[] siblings = new MerlotDOMNode[0];

        parent = node.getParentNode();
        siblings = parent.getChildNodes();
        for (int i = 0; i < siblings.length; i++) {
            if (siblings[i].equals(node)) {
                continue;
            } else {
                Node         item = null;
                NamedNodeMap map = null;

                map = siblings[i].getAttributes();
                if (map != null) {
                    item = map.getNamedItem(attribute);
                    if ((item != null)
                            && item.getNodeValue().equals(unique)) {
                        count++;
                        unique = base + count;
                        i = 0;
                        continue;
                    }
                }
            }
        }
        return unique;

  }
/**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isUniqueSibling(MerlotDOMNode node, String attribute, String value) {
        boolean         dup = false;
        MerlotDOMNode   parent = null;
        MerlotDOMNode[] siblings = new MerlotDOMNode[0];

        parent = node.getParentNode();
        if (parent != null) {
          siblings = parent.getChildNodes();
        }
        for (int i = 0; i < siblings.length; i++) {
            if (siblings[i].equals(node)) {
                continue;
            } else {
                String itemName = new String();
                Node         item = null;
                NamedNodeMap map = null;

                map = siblings[i].getAttributes();
                if (map != null) {
                    item = map.getNamedItem(attribute);
                    if (item != null) {
                        itemName = item.getNodeValue().trim();
                    }
                    if (itemName.equalsIgnoreCase(value)) {
                       dup = true;
                       break;
                    }
                }
            }
        }
        return (! dup);
    }

}
