/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: ExtendsCombo.java,v 1.1 2001/09/28 16:01:07 cryd0221 Exp $
 */

package org.enhydra.dods.merlot.cls;

// DOML plug-in imports
import org.enhydra.dods.merlot.AbstractAttributeCombo;
import org.enhydra.dods.merlot.NodeUtil;
import org.enhydra.dods.merlot.SaveListener;
import org.enhydra.dods.merlot.DOMLEditor;

// Merlot imports
import org.merlotxml.merlot.*;
import org.merlotxml.util.xml.*;
import org.w3c.dom.*;

// DODS imports
import org.enhydra.dods.store.doml.DOMLConstants;

// Standard imports
import javax.swing.*;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.*;
import java.util.*;
import java.beans.*;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 * @version $Revision: 1.1 $
 */
public class ExtendsCombo extends AbstractAttributeCombo implements SaveListener {
    public static final String ATTRIBUTE_NAME = "extends";
    public static final String DEFAULT_VALUE = "";

    private Vector oldClassNames, newClassNames;
    private String savedPanelType, oldSelectedItem;
    private boolean matched = false;

    /**
     * Constructor declaration
     *
     *
     * @param node
     *
     * @see
     */
    public ExtendsCombo() {
	super();
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getAttributeName() {
	return ExtendsCombo.ATTRIBUTE_NAME;
    }

    /**
     * Overloads the generic call to get a color chooser button if the attr name is "color"
     */
    public void init() {
	populate();

	if (isEnabled()) {
	    setSelectedItem(getNodeValue());
	}
    }

    /**
     * Method declaration
     *
     *
     * @param value
     *
     * @see
     */
    private void populate() {
	NodeUtil      util = null;
	MerlotDOMNode[] classNodes = new MerlotDOMNode[0];
	newClassNames = new Vector();

	removeAllItems();
	addItem(new String());

	util = new NodeUtil();
	classNodes = util.findNodesByName(getNode(), "class");

	for (int i = 0; i < classNodes.length; i++) {
	    String modifier = null;
	    String exClassName = new String();

	    if (classNodes[i].equals(getNode())) {

		// skip;
	    } else {
		modifier = util.lookupAttributeValue(classNodes[i], "modifier");

		if (modifier == null) {

		    // do not include.
		} else if (modifier.equalsIgnoreCase("abstract")) {
		    exClassName = util.expandPackageName(classNodes[i]);
		    addItem(exClassName);
		    newClassNames.add(exClassName);
		}
	    }
	}

	if (getItemCount() < 2) {
	    Dimension d = getPreferredSize();
            d.width = (int) (d.getWidth() * 5);
	    setPreferredSize(d);
	    setSelectedIndex(0);
	    setEnabled(false);
	}
    }

	public void panelSaved(String changedNodeType) {		
		savedPanelType = changedNodeType;
		oldSelectedItem = (String) getSelectedItem();
		oldClassNames = newClassNames;
		populate();
		if (DOMLEditor.isLink(oldSelectedItem)) {
			for (int i = 0; i < newClassNames.size(); i++) {
				if (!matched) {
					if (isSelectedItem((String) newClassNames.elementAt(i))) {
						setSelectedItem(newClassNames.elementAt(i));
					}
				}
			}
		} else {
			setSelectedItem(oldSelectedItem);
		}
		matched = false;
	}


	private boolean isSelectedItem(String expandedClassName) {
		boolean selected = false;
		if (savedPanelType.equals(DOMLConstants.PACKAGE)) {
			NodeUtil util = new NodeUtil();		
			if (util.collapsePackageName(expandedClassName)
				.equals(util.collapsePackageName(oldSelectedItem))) {
				selected = true; //return true;
			}
		} else {
			if (expandedClassName.equals(oldSelectedItem)) {
				selected = true;
				matched = true;
			} else {
				boolean found = false;
				for(int i = 0; i < oldClassNames.size(); i++) {
					if (expandedClassName.equals((String) oldClassNames.elementAt(i))) {
						found = true;
					}
				}
				selected = !found;
			}
		}


		return selected;
	}



}



/*--- formatting done in "Sun Java Convention" style on 06-21-2000 ---*/

