/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TableTextField.java,v 1.1 2001/09/28 16:01:07 cryd0221 Exp $
 */

package org.enhydra.dods.merlot.cls;

// DODS Plugin imports
import org.enhydra.dods.merlot.AbstractJavaIdentField;
import org.enhydra.dods.merlot.ValidationException;
import org.enhydra.dods.merlot.NodeUtil;

// Merlot imports
import org.merlotxml.merlot.*;
import org.merlotxml.util.xml.*;
import org.w3c.dom.*;

// Standard imports
import javax.swing.*;
import java.awt.Container;
import java.awt.Color;
import java.awt.event.*;
import java.util.*;
import java.beans.*;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 * @version $Revision: 1.1 $
 */
public class TableTextField extends AbstractJavaIdentField {
    public static final String ATTRIBUTE_NAME = "table";
    public static final String DEFAULT_VALUE = "table1";

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getAttributeName() {
	return TableTextField.ATTRIBUTE_NAME;
    }

    /**
     * Overloads the generic call to get a color chooser button if the attr name is "color"
     */
    public void init() {
	setText(getNodeValue());

	if (getText().trim().length() == 0) {
	    setText(getDefaultValue());
	}
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    private String getDefaultValue() {
	int		count = 1;
	String		base = "table";
	String		defaultV = base + count;
	MerlotDOMNode   parent = null;
	MerlotDOMNode[] classNodes = new MerlotDOMNode[0];
        NodeUtil util = new NodeUtil();

	classNodes = util.findNodesByName(getNode(), "class");

	for (int i = 0; i < classNodes.length; i++) {
	    if (classNodes[i].equals(getNode())) {
		continue;
	    } else {
		Node	     item = null;
		NamedNodeMap map = null;

		map = classNodes[i].getAttributes();

		if (map != null) {
		    item = map.getNamedItem(getAttributeName());

		    if ((item != null)
			    && item.getNodeValue().equals(defaultV)) {
			count++;
			defaultV = base + count;
			i = 0;

			continue;
		    }
		}
	    }
	}

	return defaultV;
    }


    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    private boolean isDuplicate() {
	boolean		dup = false;
	MerlotDOMNode   parent = null;
	MerlotDOMNode[] classNodes = new MerlotDOMNode[0];
	NodeUtil util = new NodeUtil();

	classNodes = util.findNodesByName(getNode(), "class");

	for (int i = 0; i < classNodes.length; i++) {
	    if (classNodes[i].equals(getNode())) {
		continue;
	    } else {
		String       itemName = new String();
		Node	     item = null;
		NamedNodeMap map = null;

		map = classNodes[i].getAttributes();

		if (map != null) {
		    item = map.getNamedItem(getAttributeName());

		    if (item != null) {
			itemName = item.getNodeValue().trim();
		    }

		    if (itemName.equalsIgnoreCase(getText().trim())) {
			dup = true;

			break;
		    }
		}
	    }
	}

	return dup;
    }

    /**
     * Method declaration
     *
     *
     * @exception ValidationException
     *
     * @see
     */
    public void validateValue() throws ValidationException {
	ValidationException ve = null;

	if (getText().trim().length() == 0) {
	    ve = new ValidationException("Table name cannot be blank");
	} else if (isDuplicate()) {
	    ve = new ValidationException("Table name must be unique");
	}

	if (ve != null) {
	    PropertyChangeEvent changeEvent = null;

	    changeEvent = new PropertyChangeEvent(this, getAttributeName(),
						  getNodeValue(), getText());

	    ve.setPropertyChangeEvent(changeEvent);

	    throw ve;
	}
    }

}



/*--- formatting done in "Sun Java Convention" style on 06-19-2000 ---*/

