/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: ConstraintCombo.java,v 1.1 2001/09/28 16:01:07 cryd0221 Exp $
 */

package org.enhydra.dods.merlot.field;

// DODS imports
import org.enhydra.dods.store.doml.DOMLConstants;

// DOML plug-in imports
import org.enhydra.dods.merlot.DOMLEditor;
import org.enhydra.dods.merlot.AbstractAttributeCombo;
import org.enhydra.dods.merlot.NodeUtil;

// Merlot imports
import org.merlotxml.merlot.*;
import org.merlotxml.util.xml.*;
import org.w3c.dom.*;

// Standard imports
import javax.swing.*;
import java.awt.Container;
import java.awt.Color;
import java.awt.event.*;
import java.util.*;
import java.beans.*;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 * @version $Revision: 1.1 $
 */
public class ConstraintCombo extends AbstractAttributeCombo implements DOMLConstants {
    public static final String ATTRIBUTE_NAME = CONSTRAINT;

    /**
     * Constructor declaration
     *
     *
     * @param node
     *
     * @see
     */
    public ConstraintCombo() {
        super();
        setEnabled(false);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getAttributeName() {
        return ConstraintCombo.ATTRIBUTE_NAME;
    }

    /**
     * Overloads the generic call to get a color chooser button if the attr name is "color"
     */
    public void init() {
        removeAllItems();
        addItem("true");
        addItem("false");
        String item = getNodeValue();
        if (item != null) {
          if (item.equals("true") ||
              item.equals("false")) {
            setSelectedItem(item);
          }
        }
        refresh();
    }


// This finds sibling components
   protected void refresh() {
      JavaTypeCombo typeCombo = null;
      String type = new String();
      boolean enable = false;

	Container parent = getParent();
	if (parent != null) {
		int count = parent.getComponentCount();
		for (int i = 0 ; i < count ; i++) {
			if (parent.getComponent(i) instanceof JavaTypeCombo) {
				typeCombo = (JavaTypeCombo) parent.getComponent(i);
			}
		}
	}

      if (typeCombo == null) {
        NodeUtil util = new NodeUtil();
        type = util.lookupAttributeValue(getNode(),
           JavaTypeCombo.ATTRIBUTE_NAME);
      } else {
        type = typeCombo.getSelectedItem().toString();
        typeCombo = null;
      }
      setEnabled(DOMLEditor.isLink(type));
	if (DOMLEditor.isLink(type)) {
		setSelectedItem("true");
	}
      updateUI();
    }





/* This is the old version of this method -
 * it does not find sibling components
 */
/*
   protected void refresh() {
      JavaTypeCombo typeCombo = null;
      String type = new String();
      boolean enable = false;

      typeCombo = new JavaTypeCombo();
      typeCombo = (JavaTypeCombo) DOMLEditor.findComponentByClass(
         getTopLevelAncestor(), typeCombo.getClass());

      if (typeCombo == null) {
        NodeUtil util = new NodeUtil();
        type = util.lookupAttributeValue(getNode(),
           JavaTypeCombo.ATTRIBUTE_NAME);
      } else {
        type = typeCombo.getSelectedItem().toString();
        typeCombo = null;
      }
      setEnabled(DOMLEditor.isLink(type));
      updateUI();
    }
*/


}
