/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: DatabaseTypeCombo.java,v 1.1 2001/09/28 16:01:07 cryd0221 Exp $
 */

package org.enhydra.dods.merlot.field;

// DOML plug-in imports
import org.enhydra.dods.merlot.AbstractAttributeCombo;

// Merlot imports
import org.merlotxml.merlot.*;
import org.merlotxml.util.xml.*;
import org.w3c.dom.*;

// Standard imports
import javax.swing.*;
import java.awt.Color;
import java.awt.event.*;
import java.util.*;
import java.beans.*;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 * @version $Revision: 1.1 $
 */
public class DatabaseTypeCombo extends AbstractAttributeCombo {
    public static final String ATTRIBUTE_NAME = "database-type";
    private final String       DEFAULT_VALUE = "VARCHAR";

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getAttributeName() {
        return DatabaseTypeCombo.ATTRIBUTE_NAME;
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    public void init() {
        String javaType = null;

        javaType = getNodeValue(JavaTypeCombo.ATTRIBUTE_NAME);
        populate(javaType);
        setSelectedItem(getNodeValue());
    }

    /**
     * Method declaration
     *
     *
     * @param javaType
     *
     * @see
     */
    protected void populate(String javaType) {
        MerlotDOMNode type = null;
        NamedNodeMap  attributes = null;

        removeAllItems();
        if (javaType.equals("String")) {
		addItem("CHAR");
            addItem("VARCHAR");
            addItem("LONGVARCHAR");
            setEnabled(true);
        } else if (javaType.equals("java.math.BigDecimal")) {
            //addItem("NUMERIC");
            addItem("DECIMAL");
            setEnabled(true);
        } else if (javaType.equals("boolean")) {
            addItem("BIT");
            //addItem("CHAR");
            setEnabled(true);
        } else if (javaType.equals("byte[]")) {
		addItem("CHAR");
            addItem("BINARY");
            addItem("VARBINARY");
            addItem("LONGVARBINARY");
            setEnabled(true);
        } else if (javaType.equals("byte")) {
		addItem("CHAR");
            addItem("BINARY");
            addItem("VARBINARY");
            addItem("LONGVARBINARY");
            //addItem("TINYINT");
            setEnabled(true);
        } else if (javaType.equals("short")) {
            addItem("SMALLINT");
		addItem("TINYINT");
            setEnabled(true);
        } else if (javaType.equals("int")) {
            addItem("INTEGER");
            setEnabled(true);
        } else if (javaType.equals("long")) {
            addItem("BIGINT");
            setEnabled(true);
        } else if (javaType.equals("float")) {
		addItem("FLOAT");
            //addItem("REAL");
            setEnabled(true);
        } else if (javaType.equals("double")) {
		addItem("REAL");
            //addItem("DOUBLE");
            setEnabled(true);
        } else if (javaType.equals("java.sql.Date")) {
            addItem("DATE");
            setEnabled(true);
        } else if (javaType.equals("java.sql.Time")) {
            addItem("TIME");
            setEnabled(true);
        } else if (javaType.equals("java.sql.Timestamp")) {
            addItem("TIMESTAMP");
            setEnabled(true);
        } else {
            addItem(new String());
            setEnabled(false);
        }
    }

}
