/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: JavaTypeCombo.java,v 1.1 2001/09/28 16:01:07 cryd0221 Exp $
 */

package org.enhydra.dods.merlot.field;

// w2c imports
import org.w3c.dom.NamedNodeMap;

// DODS imports
import org.enhydra.dods.store.doml.DOMLConstants;

// DOML plug-in imports
import org.enhydra.dods.merlot.DOMLEditor;
import org.enhydra.dods.merlot.AbstractAttributeCombo;
import org.enhydra.dods.merlot.NodeUtil;
import org.enhydra.dods.merlot.SaveListener;

// Merlot imports
import org.merlotxml.merlot.*;
import org.merlotxml.util.xml.*;
import org.w3c.dom.*;

// Standard imports
import javax.swing.*;
import java.awt.Component;
import java.awt.Container;
import java.awt.Color;
import java.awt.event.*;
import java.util.*;
import java.beans.*;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 * @version $Revision: 1.1 $
 */
public class JavaTypeCombo extends AbstractAttributeCombo implements DOMLConstants, SaveListener {
    public static final String ATTRIBUTE_NAME = JAVATYPE;

    private String savedPanelType, newSelectedItem, oldSelectedItem;	// new logic
    private boolean initializing, matched = false;				// new logic
    private Vector oldClassNames, newClassNames;				// new logic

    /**
     * Constructor declaration
     *
     *
     * @param node
     *
     * @see
     */
    public JavaTypeCombo() {
        super();
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getAttributeName() {
        return JavaTypeCombo.ATTRIBUTE_NAME;
    }

    /**
     * Overloads the generic call to get a color chooser button if the attr name is "color"
     */
    public void init() {
	  initializing = true;
        populate();
        setSelectedItem(getNodeValue());
        LocalItemListener itemListener = new LocalItemListener();
        addItemListener(itemListener);
    }

    /**
     * Method declaration
     *
     *
     * @param value
     *
     * @see
     */
    private void populate() {
      NodeUtil util = null;
      MerlotDOMNode classNodes[] = new MerlotDOMNode[0];
	  
      removeAllItems();
      TreeSet set = new TreeSet();
      for (int i = 0; i < JAVA_TYPE_LIST.length; i++) {
        set.add(JAVA_TYPE_LIST[i]);
      }

      util = new NodeUtil();
	classNodes = util.findNodesByName(getNode(), "class");
	newClassNames = new Vector();

	for (int i = 0; i < classNodes.length; i++) {
	    String modifier = null;
	    String exClassName = new String();

	    if (classNodes[i].equals(getNode().getParentNode())) {	
		// skip;
	    } else {
		modifier = util.lookupAttributeValue(classNodes[i], "modifier");

		if (modifier == null) {

		    // do not include.
		} else if (! modifier.equalsIgnoreCase("abstract")) {
		    	exClassName = util.expandPackageName(classNodes[i]);
		    	set.add(exClassName);
		    	newClassNames.add(exClassName);				// new logic

			if (!initializing && !matched) {				// new logic
				boolean select = isSelectedItem(exClassName);	// new logic
				if (select) {						// new logic
					newSelectedItem = exClassName;		// new logic
				}								// new logic
												// new logic	
			}									// new logic
		}
	    }
	}

      Iterator list = set.iterator();
      while (list.hasNext()) {
        addItem((String) list.next());
      }
      set.clear();
      set = null;

	initializing = false;
	matched = false;
    }

	public void panelSaved(String changedNodeType) {
		savedPanelType = changedNodeType;
		oldSelectedItem = (String) getSelectedItem();
		oldClassNames = newClassNames;
		populate();
		if (DOMLEditor.isLink(oldSelectedItem)) {
			setSelectedItem(newSelectedItem);
		} else {
			setSelectedItem(oldSelectedItem);
		}
	}

	private boolean isSelectedItem(String expandedClassName) {
		boolean selected = false;
		if (savedPanelType.equals(DOMLConstants.PACKAGE)) {
			NodeUtil util = new NodeUtil();
			if (util.collapsePackageName(expandedClassName)
				.equals(util.collapsePackageName(oldSelectedItem))) {
				selected = true; //return true;
			}
		} else {
			if (expandedClassName.equals(oldSelectedItem)) {
				selected = true;
				matched = true;
			} else {
				boolean found = false;
				for(int i = 0; i < oldClassNames.size(); i++) {
					if (expandedClassName.equals((String) oldClassNames.elementAt(i))) {
						found = true;
					}
				}
				selected = !found;
			}
		}


		return selected;
	}

	private boolean isCircularReference(String classToCheck) {	
		boolean circular = false;
		MerlotDOMNode referencedClassNode = null;	
		NodeUtil util = new NodeUtil();
		classToCheck = util.collapsePackageName(classToCheck);
		MerlotDOMNode[] classNodes = util.findNodesByName(getNode(), CLASS);

		if (classNodes != null) {	
			for (int i = 0; i < classNodes.length; i++) {				
				String thisClassName 
					= util.lookupAttributeValue(classNodes[i], CLASS_NAME_ATTRIBUTE);
				if ((thisClassName).equals(classToCheck)) {
					referencedClassNode = classNodes[i];
				}
			} // for
		} else { 
			return false;
		}

		if (referencedClassNode == null) {
			return false;
		}

		MerlotDOMNode[] fields = referencedClassNode.getChildNodes();
		if (fields != null) {
			for (int i = 0; i < fields.length; i++) {
				if (fields[i] instanceof MerlotDOMElement) {
					String javaType = util.lookupAttributeValue(fields[i], ATTRIBUTE_NAME);			
					String originatingClassName 
						= util.lookupAttributeValue(getNode().getParentNode(), CLASS_NAME_ATTRIBUTE);

					if (util.collapsePackageName(javaType).equals(originatingClassName)) {
						circular = true;
					} else if (!DOMLEditor.isLink(javaType)) {
						// Do nothing. Go on and examine next field
					} else {
						circular = isCircularReference(javaType);
					}
				}
			}
		} else {
			return false;
		}
		return circular;
	}


    private void updateDbTypes() {
      Container parent =  getParent();
      DatabaseTypeCombo dbCombo = null;
      int count = parent.getComponentCount();
      for (int i = 0 ; i < count ; i++) {
         if (parent.getComponent(i) instanceof DatabaseTypeCombo) {
           dbCombo = (DatabaseTypeCombo) parent.getComponent(i);
           dbCombo.populate(getSelectedItem().toString());
           break;
         }
      }
    }




/* This was the original version of method - 
 * does not guarantee to find a sibling component
 */
/*
   private void updateSize() {
      SizeTextField sizeText = new SizeTextField();
      sizeText = (SizeTextField) DOMLEditor.findComponentByClass(
         getTopLevelAncestor(), sizeText.getClass());
      if (sizeText != null) {
         sizeText.refresh();
      }
    }
*/

// This one is guaranteed to return a sibling component
   private void updateSize() {
      Container parent =  getParent();
      SizeTextField sizeText = null;
      int count = parent.getComponentCount();
      for (int i = 0 ; i < count ; i++) {
		if (parent.getComponent(i) instanceof SizeTextField) {
			sizeText = (SizeTextField) parent.getComponent(i);
			sizeText.refresh();
			break;
		}
	}
    }




/* This was the original version of this method -
 * it does not guarantee to find a sibling component
 */
/*
   private void updateConstraint() {
      ConstraintCombo conCombo = new ConstraintCombo();
      conCombo = (ConstraintCombo) DOMLEditor.findComponentByClass(
         getTopLevelAncestor(), conCombo.getClass());
      if (conCombo != null) {
         conCombo.refresh();
      }
    }
*/

// This one is guaranteed to return a sibling component
   private void updateConstraint() {
	Container parent =  getParent();
	ConstraintCombo conCombo = null;
	int count = parent.getComponentCount();
	for (int i = 0 ; i < count ; i++) {
		if (parent.getComponent(i) instanceof ConstraintCombo) {
			conCombo = (ConstraintCombo) parent.getComponent(i);
			conCombo.refresh();
			break;
		}
	}
   }

    class LocalItemListener implements ItemListener {
      public void itemStateChanged(ItemEvent event) {

/***********************************************************************************************
// Start Diagnostics
//System.out.println("!!!!! JavaTypeCombo.getParent()==" + getParent());
//System.out.println("!!!!!!! JavaTypeCombo.getParent().getParent()==" + getParent().getParent());
//System.out.println("!!!!!!!!! JavaTypeCombo.getParent().getParent().getParent()==" + getParent().getParent().getParent());
JTabbedPane tabbedPane = (JTabbedPane) getParent().getParent().getParent();
System.out.println("\n\n##### Tab Count==" + tabbedPane.getTabCount());
JPanel firstTab = (JPanel) tabbedPane.getComponentAt(0);
Component[] firstTabComponents = firstTab.getComponents();

//System.out.println("##### First Tab==" + tabbedPane.getComponentAt(0));
System.out.println("##### firstTabComponents.length==" + firstTabComponents.length);
System.out.println("##### First Tab Component Count==" + firstTab.getComponentCount());
for (int i = 0; i < firstTabComponents.length; i++) {
	if (firstTabComponents[i] instanceof GenericDOMEditPanel) {
		System.out.println("\t" + i + ": " + firstTabComponents[i]);
		System.out.println("\tChild Count: " + ((JPanel) firstTabComponents[i]).getComponentCount());
		Component[] editPanelComponents = ((JPanel) firstTabComponents[i]).getComponents();
		for (int j = 0; j < editPanelComponents.length; j++) {
			System.out.println("\t\t" + j + ": " + editPanelComponents[j]);
		}
	}
}

// End Diagnostics
*************************************************************************************************/


         if (event.getStateChange() == ItemEvent.SELECTED) {
		boolean circular = false;
		String selection = getSelectedItem().toString();
		if (DOMLEditor.isLink(selection)) {
			circular = isCircularReference(selection); 
		}
		
		if (!circular) {
           		updateDbTypes();
           		updateSize();
           		updateConstraint();
		} else {
			JOptionPane.showMessageDialog(getParent(), 
				"You have attempted to create a circular reference.",
				"Circular Reference Checker",
				JOptionPane.ERROR_MESSAGE);

			NodeUtil util = new NodeUtil();
			String savedJavaType = util.lookupAttributeValue(getNode(), ATTRIBUTE_NAME);
			setSelectedItem(savedJavaType);
		}


         }
      }
    }

}
