/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: NameTextField.java,v 1.1 2001/09/28 16:01:07 cryd0221 Exp $
 */

package org.enhydra.dods.merlot.field;

// DODS Plugin imports
import org.enhydra.dods.merlot.AbstractJavaIdentField;
import org.enhydra.dods.merlot.ValidationException;
import org.enhydra.dods.merlot.NodeUtil;

// Merlot imports
import org.merlotxml.merlot.*;
import org.merlotxml.util.xml.*;
import org.w3c.dom.*;

// Standard imports
import javax.swing.*;
import java.awt.Container;
import java.awt.Color;
import java.awt.event.*;
import java.util.*;
import java.beans.*;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 * @version $Revision: 1.1 $
 */
public class NameTextField extends AbstractJavaIdentField {
    public static final String ATTRIBUTE_NAME = "name";
    public static final String DEFAULT_VALUE = "field1";

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getAttributeName() {
        return NameTextField.ATTRIBUTE_NAME;
    }

    /**
     * Overloads the generic call to get a color chooser button if the attr name is "color"
     */
    public void init() {
        setText(getNodeValue());
        if (getText().trim().length() == 0) {
            setText(getDefaultValue());
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    private String getDefaultValue() {
       NodeUtil util = null;
       String unique = null;
       util = new NodeUtil();
       unique = util.createUniqueSibling(getNode(), getAttributeName(), "field");
       return unique;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    private boolean isDuplicate() {
       NodeUtil util = null;
       boolean unique = true;
       util = new NodeUtil();
       unique = util.isUniqueSibling(getNode(), getAttributeName(), getText().trim());
       return (! unique);
    }


    /**
     * Method declaration
     *
     *
     * @exception ValidationException
     *
     * @see
     */
    public void validateValue() throws ValidationException {
        ValidationException ve = null;

        if (getText().trim().length() == 0) {
            ve = new ValidationException("Field name cannot be blank");
        } else if (isDuplicate()) {
            ve = new ValidationException("Field name must be unique");
        }
        if (ve != null) {
            PropertyChangeEvent changeEvent = null;

            changeEvent = new PropertyChangeEvent(this, getAttributeName(),
                                                  getNodeValue(), getText());
            ve.setPropertyChangeEvent(changeEvent);
            throw ve;
        }
    }

}
