/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: SizeTextField.java,v 1.1 2001/09/28 16:01:07 cryd0221 Exp $
 */

package org.enhydra.dods.merlot.field;

// DODS Plugin imports
import org.enhydra.dods.merlot.AbstractAttributeTextField;
import org.enhydra.dods.merlot.ValidationException;
import org.enhydra.dods.merlot.Validator;
import org.enhydra.dods.merlot.NodeUtil;
import org.enhydra.dods.merlot.DOMLEditor;

// DODS imports
import org.enhydra.dods.store.doml.DOMLConstants;

// Merlot imports
import org.merlotxml.merlot.*;
import org.merlotxml.util.xml.*;

// Standard imports
import javax.swing.*;
import javax.swing.text.Document;
import javax.swing.text.PlainDocument;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import java.awt.Container;
import java.awt.Color;
import java.awt.SystemColor;
import java.awt.event.*;
import java.util.*;
import java.beans.*;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 * @version $Revision: 1.1 $
 */
public class SizeTextField extends AbstractAttributeTextField
     implements Validator {
    private Color defaultBackground = SystemColor.white;
    private static final String DEFAULT_STRING_SIZE = "32";
    private static final String DEFAULT_BYTE_SIZE = "8";
    private static final String DEFAULT_DECIMAL_SIZE = "18,0";

    public static final String ATTRIBUTE_NAME = "size";

    private String javaType = null; 
    private boolean initializing = false;

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getAttributeName() {
        return SizeTextField.ATTRIBUTE_NAME;
    }



    private boolean isDecimal() {
      boolean decimal = false;
     	DatabaseTypeCombo combo = new DatabaseTypeCombo();
      String dbType = new String();
	 Container parent = getParent();

	 if (parent != null) {
	  	 int count = parent.getComponentCount();
	  	 for (int i = 0 ; i < count ; i++) {
			 if (parent.getComponent(i) instanceof DatabaseTypeCombo) {
			 	 combo = (DatabaseTypeCombo) parent.getComponent(i);
			 }
	  	 }
	 }

	if (combo != null) {
		// This logic can be optimized. 
		//	The call to lookupAttributeValue() can be made once.
		if (combo.getSelectedItem() != null) {
			dbType = combo.getSelectedItem().toString();
		} else {
        		NodeUtil util = new NodeUtil();
        		dbType = util.lookupAttributeValue(getNode(),
           			DatabaseTypeCombo.ATTRIBUTE_NAME);
		}
	} else {

        	NodeUtil util = new NodeUtil();
        	dbType = util.lookupAttributeValue(getNode(),
           		DatabaseTypeCombo.ATTRIBUTE_NAME);
	}      


      if (dbType != null) {
        decimal = (dbType.equalsIgnoreCase("DECIMAL") ||
                   dbType.equalsIgnoreCase("NUMERIC"));
      }
	//System.out.println("!!!!! End of isDecimal()");
      return decimal;
    }

/****************************************************************

    private boolean isString() {
      boolean string = false;
      //DatabaseTypeCombo combo = new DatabaseTypeCombo();
	//System.out.println("%%%%%% combo==" + combo);
      String dbType = new String();

        NodeUtil util = new NodeUtil();
        dbType = util.lookupAttributeValue(getNode(),
           DatabaseTypeCombo.ATTRIBUTE_NAME);
      
      if (dbType != null) {
        string = dbType.trim().toUpperCase().endsWith("CHAR") ||
		     dbType.trim().toUpperCase().endsWith("CHAR2");
      }
	//System.out.println("!!!!! End of isString()...");
      return string;
    }
********************************************************************/

/******
    private boolean isDecimal() {
      boolean decimal = false;
      DatabaseTypeCombo combo = new DatabaseTypeCombo();
      combo = (DatabaseTypeCombo) DOMLEditor.findComponentByClass(
         getTopLevelAncestor(),
         combo.getClass());
      String dbType = new String();
      if (combo != null) {
        dbType = combo.getSelectedItem().toString();
      } else {
        NodeUtil util = new NodeUtil();
        dbType = util.lookupAttributeValue(getNode(),
           DatabaseTypeCombo.ATTRIBUTE_NAME);
      }
      if (dbType != null) {
        decimal = (dbType.equalsIgnoreCase("DECIMAL") ||
                   dbType.equalsIgnoreCase("NUMERIC"));
      }
      return decimal;
    }
******/

    private boolean isString() {
      boolean string = false;
      DatabaseTypeCombo combo = new DatabaseTypeCombo();
      combo = (DatabaseTypeCombo) DOMLEditor.findComponentByClass(
         getTopLevelAncestor(),
         combo.getClass());
      String dbType = new String();
      if (combo != null) {
        dbType = combo.getSelectedItem().toString();
      } else {
        NodeUtil util = new NodeUtil();
        dbType = util.lookupAttributeValue(getNode(),
           DatabaseTypeCombo.ATTRIBUTE_NAME);
      }
      if (dbType != null) {
        string = dbType.trim().toUpperCase().endsWith("CHAR") ||
		     dbType.trim().toUpperCase().endsWith("CHAR2");
      }
      return string;
    }





    /**
     * Overloads the generic call to get a color chooser button if the attr name is "color"
     */
    public void init() {
        defaultBackground = getBackground();
        setText(getNodeValue());
	  initializing = true;

        refresh();
    }

    public void refresh() {

       if (isString()) {
         if ((getText().indexOf(',')    >  0) ||
             (getText().trim().length() == 0)) {
           setText(SizeTextField.DEFAULT_STRING_SIZE);
         }
         setEnabled(true);
       } else if (isDecimal()) {
         if (getText().trim().length() == 0) {
           setText(SizeTextField.DEFAULT_DECIMAL_SIZE);
         }
         setEnabled(true);
       } else {
         setText(new String());
         setEnabled(false);
       }

	 JavaTypeCombo combo = null;
	  
	 Container parent = getParent();
	 if (parent != null) {
	  	 int count = parent.getComponentCount();
	  	 for (int i = 0 ; i < count ; i++) {
			 if (parent.getComponent(i) instanceof JavaTypeCombo) {
			 	 combo = (JavaTypeCombo) parent.getComponent(i);
			 }
	  	 }
	 }

	  if (combo != null) {			
	     	javaType = combo.getSelectedItem().toString();  
	  } else {

		NodeUtil util = new NodeUtil();
		javaType = util.lookupAttributeValue(getNode(),
           		JavaTypeCombo.ATTRIBUTE_NAME);
	  }
	 

	 if (javaType.equals(DOMLConstants.STRING_JAVA_TYPE) || 
	     javaType.equals(DOMLConstants.BYTE_JAVA_TYPE) || 		// byte
	     javaType.equals(DOMLConstants.BYTE_ARRAY_JAVA_TYPE) || 	// byte
	     javaType.equals(DOMLConstants.BIG_DECIMAL_JAVA_TYPE)) {  

	   	setEnabled(true);

		if (!initializing) {   
			NodeUtil util = new NodeUtil();  // NEW
			String savedJavaType = util.lookupAttributeValue(getNode(),JavaTypeCombo.ATTRIBUTE_NAME);  // NEW
			String savedSize = util.lookupAttributeValue(getNode(),SizeTextField.ATTRIBUTE_NAME);  // NEW
//System.out.println("!!!!! savedJavaType==" + savedJavaType);
//System.out.println("!!!!! savedSize==" + savedSize + "\n\n");
			// This logic can be optimized. Can test javaType.equals once-
			// either set to savedSize or go to an else where you set to default string or decimal size
	   		if (javaType.equals(DOMLConstants.STRING_JAVA_TYPE)) {  
				if (javaType.equals(savedJavaType)) {  	// NEW
	     				setText(savedSize); 			// NEW
				} else {  						// NEW
					setText(SizeTextField.DEFAULT_STRING_SIZE);
				}							// NEW
			} else if (javaType.equals(DOMLConstants.BYTE_JAVA_TYPE) || 		// byte
				     javaType.equals(DOMLConstants.BYTE_ARRAY_JAVA_TYPE) ) {	// byte	
				if (javaType.equals(savedJavaType)) {					// byte
					setText(savedSize);							// byte
				} else {										// byte
					setText(SizeTextField.DEFAULT_BYTE_SIZE);				// byte
				}											// byte		
	   		} else {
				if (javaType.equals(savedJavaType)) {  	// NEW
					setText(savedSize); 			// NEW
				} else {  						// NEW
					setText(SizeTextField.DEFAULT_DECIMAL_SIZE);
				}							// NEW

						
         		}  	   
		}

	 } else {	
	   setText("");							
	   setEnabled(false);		      						      			
	 }									

	initializing= false;

       if (isEnabled()) {
         setBackground(defaultBackground);
       } else {
         setBackground(SystemColor.control);
       }
    }


    /**
     * Method declaration
     *
     *
     * @exception ValidationException
     *
     * @see
     */
    public void validateValue() throws ValidationException {
        ValidationException ve = null;
        int iVal = 0;
        if (isEnabled()) {
          if (isDecimal()) {
            char[] chars = getText().toCharArray();
           for (int i = 0 ; i < chars.length ; i++ ) {
             if (Character.isDigit(chars[i])) {
               // ok
             } else if (i == 0 || i == (chars.length - 1)) {
                ve = new ValidationException("Invalid decimal size");
                break;
             } else if (chars[i] == ',') {
               // ok
             } else {
                ve = new ValidationException("Invalid decimal size");
               break;
             }
           }
          } else if (isString()) {
            try {
              iVal = Integer.parseInt(getText());
            } catch (NumberFormatException e) {
              ve = new ValidationException(e.getMessage());
            }
            if (iVal < 1) {
                ve = new ValidationException("Size must be a positive integer");
            }
          } else {
            setEnabled(false);
            setText(new String());
          }
          if (ve != null) {
              PropertyChangeEvent changeEvent = null;

              changeEvent = new PropertyChangeEvent(this, getAttributeName(),
                                                    getNodeValue(), getText());
              ve.setPropertyChangeEvent(changeEvent);
              throw ve;
          }
        } else {
          setText(new String());
        }
    }


  protected Document createDefaultModel() {
     return new LocalDocument();
   }

  class LocalDocument extends PlainDocument {

        public void insertString(int offs, String str, AttributeSet a)
          throws BadLocationException {

           if (str == null) {
              return;
           }

           StringBuffer buf = new StringBuffer();
           char[] chars = str.toCharArray();
           for (int i = 0; i < chars.length; i++) {
             if (Character.isDigit(chars[i])) {
               buf.append(chars[i]);
             } else if (chars[i] == ',' && isDecimal()) {
               buf.append(chars[i]);
             }
           }
           super.insertString(offs, buf.toString(), a);
        }
  }


}
