/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: PackageEditPanel.java,v 1.1 2001/09/28 16:01:07 cryd0221 Exp $
 */

package org.enhydra.dods.merlot.pack;

// DOML plug-in imports
import org.enhydra.dods.merlot.AbstractAttributeCombo;
import org.enhydra.dods.merlot.AbstractValidationPanel;
import org.enhydra.dods.merlot.DOMLEditor;
import org.enhydra.dods.merlot.NodeUtil;
import org.enhydra.dods.merlot.ValidationException;
import org.enhydra.dods.merlot.field.JavaTypeCombo;

// Merlot imports
import org.merlotxml.merlot.*;
import org.merlotxml.util.xml.*;
import org.w3c.dom.*;

// Standard imports
import javax.swing.*;
import java.awt.Color;
import java.awt.event.*;
import java.util.*;
import java.beans.*;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 * @version $Revision: 1.1 $
 */
public class PackageEditPanel extends AbstractValidationPanel {
    private String atName = null;


    /**
     * Constructor declaration
     *
     *
     * @param node
     *
     * @see
     */
    public PackageEditPanel(MerlotDOMNode node) {
        super(node);
    }

    public void save() throws PropertyVetoException {
      NameTextField nameText = null;
      NodeUtil      util     = null;
      MerlotDOMNode node     = null;
      MerlotDOMNode[] fieldNodes = new MerlotDOMNode[0];
      String preSavePack  = new String();
      String postSavePack = new String();

      util     = new NodeUtil();
      nameText = new NameTextField();
      nameText = (NameTextField) DOMLEditor.findComponentByClass(
         getTopLevelAncestor(), nameText.getClass());
      if (nameText != null) {
        node = nameText.getNode();
        nameText = null;
        preSavePack = util.expandPackageName(node);
      }

      super.save();

      if (node != null) {
        postSavePack = util.expandPackageName(node);
      }
      if ((preSavePack != null) && (postSavePack != null)) {
        if (! preSavePack.equals(postSavePack)) {
          fieldNodes = util.findNodesByName(node, "field");
          util = null;
          updateTypes(fieldNodes, preSavePack, postSavePack);
        }
      }
    }

    private void updateTypes(
        MerlotDOMNode[] fieldNodes, String preSavePack,
           String postSavePack) {

      NodeUtil util   = new NodeUtil();
      String   type   = new String();
      String   suffix = new String();
      int index = -1;

      for (int i = 0 ; i < fieldNodes.length ; i++ ) {
        type = util.lookupAttributeValue(fieldNodes[i],
             JavaTypeCombo.ATTRIBUTE_NAME);
        if (DOMLEditor.isLink(type)) {
          index = type.indexOf(".");
          if (index > 0 && type.startsWith(preSavePack)) {
            suffix = type.substring(preSavePack.length());
            type   = postSavePack + suffix;
            util.storeAttributeValue(fieldNodes[i],
                  JavaTypeCombo.ATTRIBUTE_NAME,
                  type);
          }
        }
      }
    }


    /**
     * Overloads the generic call to get a color chooser button if the attr name is "color"
     */
    protected JComponent getEditComponent(DTDAttribute attr) {
    	JComponent comp = null;
    	atName = attr.getName().trim();
        if (atName.equalsIgnoreCase(NameTextField.ATTRIBUTE_NAME)) {
          NameTextField nameText = new NameTextField();
          nameText.setNode(this._node);
          nameText.init();
          comp = nameText;
        } else {
          comp = super.getEditComponent(attr);
        }
        return comp;
    }

}
