/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: Store.java,v 1.1 2001/09/28 16:01:07 cryd0221 Exp $
 */
package org.enhydra.dods.store;

import org.enhydra.dods.exceptions.DataObjectNameException;
import org.enhydra.dods.store.doml.DOMLFactory;
import org.enhydra.dods.store.doa.DODSProject;
import java.io.File;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import java.util.Vector;

/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */
public class Store {
    private File		 outputFile = null;
    private File		 inputFile = null;
    private File		 currentDirectory = null;
    private DODSProject          project = null;

    /**
     * Constructor declaration
     *
     *
     * @see
     */
    public Store() {}

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isSaved() {
	boolean				      saved = false;
	org.enhydra.dods.store.doa.DODSProject diskImage = null;

	if (getOutputFile() != null) {
	    diskImage = DOMLFactory.readDOML(getOutputFile());
	}

	if (project != null && diskImage != null) {
	    if (project.equals(diskImage)) {
		saved = true;
	    }
	}

	return saved;
    }

    /**
     * Method declaration
     *
     *
     * @param f
     *
     * @see
     */
    public void setInputFile(File f) {
	inputFile = f;
	outputFile = null;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public File getInputFile() {
	return inputFile;
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    public void create() throws DataObjectNameException {
	outputFile = null;
	inputFile = null;
	project = new DODSProject("root");
	project.setDatabase("Standard");
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public DODSProject getProject() {
        if (project == null) {
          try {
            create();
          } catch (DataObjectNameException e) {
            project = null;
            e.printStackTrace();
          }
        }
	return project;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isInputDOA() {
	boolean inputDOA = false;
	String  extension =
	    org.enhydra.dods.util.Util.getExtension(getInputFile());

	if (extension != null && extension.equalsIgnoreCase("doa")) {
	    inputDOA = true;
	}

	return inputDOA;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isInputDOML() {
	boolean inputDOML = false;
	String  extension =
	    org.enhydra.dods.util.Util.getExtension(getInputFile());

	if (extension != null && extension.equalsIgnoreCase("doml")) {
	    inputDOML = true;
	}

	return inputDOML;
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    public void read() {
	try {
	    if (isInputDOML()) {
		readDOML();
	    }

	    project.setFile(getOutputFile());
	} catch (NullPointerException e) {
	    project = null;
	}
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void readDOML() {
	project = DOMLFactory.readDOML(inputFile);
    }


    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public File write() {
	DOMLFactory.writeDOML(project, outputFile, true);

	return outputFile;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public File getOutputFile() {
	String outName = null;

	if (outputFile == null && inputFile != null) {
	    outName = inputFile.getParent() + File.separator
		      + org.enhydra.dods.util.Util.getBase(inputFile)
		      + ".doml";
	    outputFile = new File(outName);

	    if (project != null) {
		project.setFile(outputFile);
	    }
	}

	return outputFile;
    }

    /**
     * Method declaration
     *
     *
     * @param o
     *
     * @see
     */
    public void setOutputFile(File o) {
	String extension = org.enhydra.dods.util.Util.getExtension(o);

	if (extension != null && extension.equalsIgnoreCase("doml")) {
	    outputFile = o;
	} else {
	    outputFile = new File(o.getAbsolutePath() + ".doml");
	}

	if (project != null) {
	    project.setFile(outputFile);
	}
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public File getCurrentDirectory() {
	String eDir = System.getProperty("ENHYDRA_DIR");
        if (eDir == null) {
          eDir = System.getProperty("user.home");
        }

	if (getOutputFile() != null) {
	    currentDirectory = new File(getOutputFile().getParent());
	} else if (currentDirectory == null && (new File(eDir)).exists()) {
	    currentDirectory = new File(eDir);
	    currentDirectory = new File(currentDirectory.getParent());
	}

	return currentDirectory;
    }

}



/*--- formatting done in "Sun Java Convention" style on 01-19-2000 ---*/

