/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: DOPackage.java,v 1.1 2001/09/28 16:01:07 cryd0221 Exp $
 */
package org.enhydra.dods.store.doa;

import org.enhydra.dods.exceptions.*;
import java.util.Vector;

/**
 * A holder for all the information describing a DOPackage.
 *
 * @author  Philip Martin
 * @version $Revision: 1.1 $
 */
public class DOPackage extends DataObject {
    /**
     * This is the Vector which holds all of the DataObjects and Packages that
     * are in this package.
     */
    protected Vector dataObjects;
    /**
     * This constructor creates a DOPackage with name "NewPackage"
     */
    public DOPackage() {
        this("NewPackage");
    }
    /**
     * This constructor creates a DOPackage with the specified name.
     */
    public DOPackage(String name) {
        super(name);
        dataObjects = new Vector();
    }
    /**
     * @return The DataObjects in this Package.
     */
    public Vector getDataObjects() {
        return dataObjects;
    }
    /**
     * This method adds a DataObject to this Package.
     */
    public void addDataObject(DataObject DO) {
        dataObjects.addElement(DO);
        try {
            if(this.getPackageName().equals("")){
                DO.setPackageName(this.getName());
            }
            else {
                DO.setPackageName(this.getPackageName()+ "." + this.getName());
            }
        }
        catch(DataObjectNameException e){}
    }
    /**
     * This method adds a Package into this Package.
     */
    public void addDOPackage(DOPackage DOP) {
        dataObjects.addElement(DOP);
        try {
            if(this.getPackageName().equals("")){
                DOP.setPackageName(this.getName());
            }
            else {
                DOP.setPackageName(this.getPackageName()+ "." + this.getName());
            }
        }
        catch(DataObjectNameException e){
        }
    }
    /**
     * This method is used when a package name is changed, so all of the Objects
     * in that Package need to have thier packageNames updated.
     */
    public void updatePackageName() {
        for(int i = 0; i < dataObjects.size(); i++) {
            try {
                ((DataObject)dataObjects.elementAt(i)).setPackageName(this.getPackageName() + "." + this.getName());
                if(this.getPackageName().equals("")) {
                    ((DataObject)dataObjects.elementAt(i)).setPackageName(this.getName());
                }
                if((DataObject)dataObjects.elementAt(i) instanceof DOPackage) {
                    ((DOPackage)dataObjects.elementAt(i)).updatePackageName();
                }
            }
            catch(DataObjectNameException e) {}
        }
    }
    /**
     * Returns a String representation of the form:
     * "<I>packageName</I>.<I>name</I>"
     */
    public String toString() {
        if(packageName.equals("")){
            return new String(name + "");
        }
        return new String(packageName + "." + name);
    }
    /**
     * This will return true if the two names and all of the dataObjects are equal.
     */
    public boolean equals(DOPackage dop) {
        try {
            if(dataObjects.size()!=dop.getDataObjects().size()) {
                return false;
            }
            for(int i = 0; i < dataObjects.size(); i++) {
                DataObject dataObject = (DataObject)dataObjects.elementAt(i);
                if(dataObject instanceof DOPackage) {
                    DOPackage doPackage = (DOPackage)dataObject;
                    if(!((DOPackage)dop.getDataObjects().elementAt(i)).equals(doPackage)) {
                        return false;
                    }
                }
                else {
                    if(!((DataObject)dop.getDataObjects().elementAt(i)).equals(dataObject)) {
                        return false;
                    }
                }
            }
            if(!name.equals(dop.getName()))
                return false;
            if(!packageName.equals(dop.getPackageName()))
                return false;
        }
        catch(Exception e) {
            return false;
        }
        return true;
    }
    /**
     * This method adds all of the DataObjects in this package and any subPackages
     * into the given vector.
     * @param The vector to add the DataObjects to.
     */
    public void getAllDataObjects(Vector allDOS) {
        for(int i = 0; i < dataObjects.size(); i++) {
            if(dataObjects.elementAt(i) instanceof DOPackage) {
                ((DOPackage)dataObjects.elementAt(i)).getAllDataObjects(allDOS);
            }
            else {
                allDOS.addElement(dataObjects.elementAt(i));
            }
        }
    }
}
