/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: DOMLConstants.java,v 1.1 2001/09/28 16:01:07 cryd0221 Exp $
 */
package org.enhydra.dods.store.doml;

public interface DOMLConstants {
    public final String[]      JAVA_TYPE_LIST = {
        "String", "java.math.BigDecimal", "boolean", "byte", "short", "int",
        "long", "float", "double", "byte[]", "java.sql.Date",
        "java.sql.Time", "java.sql.Timestamp"
    };
    public final String[]      VENDOR_ATTRIBUTE_LIST = {
        "default", "Standard", "InstantDB", "Oracle", "Informix", "Msql",
        "Sybase", "PostgreSQL",
    };
    public static final String PACKAGE_SEPARATOR = ".";
    public final String        REF_SUFFIX = "DO";

    //
    public final String        PUBLIC_ID =
        "-//enhydra.org//Enhydra DOML for DODS 4.0//EN";
    public final String        SYSTEM_ID =
        "http://dods.enhydra.org/dtd/doml.dtd";
    public final String        DOCTYPE = "<!DOCTYPE doml PUBLIC \""
                                         + PUBLIC_ID + "\" \"" + SYSTEM_ID
                                         + "\">";

    //
    public final String        JAVATYPE = "java-type";
    public final String        CONSTRAINT = "constraint";
    public final String        VENDOR_NAME_ATTRIBUTE = "name";
    public final String        CLASS_NAME_ATTRIBUTE = "name";
    public final String        CLASS_VENDOR_ATTRIBUTE = "vendor";

    // current keys
    public static final String DOML = "doml";
    public final String        VENDOR = "vendor";
    public final String        VENDOR_NAME = "vendor.name";
    public final String        PACKAGE = "package";
    public final String        PACKAGE_NAME = "package.name";
    public final String        CLASS = "class";
    public final String        CLASS_TABLE = "class.table";
    public final String        CLASS_MODIFIER = "class.modifier";
    public final String        CLASS_CACHING = "class.caching";
    public final String        CLASS_LAZYLOADING = "class.lazy-loading";
    public final String        CLASS_EXTENDS = "class.extends";
    public final String        CLASS_VENDOR = "class.vendor";
    public final String        CLASS_OID = "class.use-oid";
    public final String        CLASS_IS_STORED_PROC = "class.is-stored-proc";
    public final String        FIELD = "field";
    public final String        FIELD_NAME = "field.name";
    public final String        FIELD_INDEX = "field.index";
    public final String        FIELD_PRIMARY = "field.primary-key";
    public final String        FIELD_CANQUERY = "field.can-query";
    public final String        FIELD_CONSTANT = "field.constant";
    public final String        FIELD_JAVADOC = "field.javadoc";
    public final String        FIELD_NULLALLOWED = "field.null-allowed";
    public final String        FIELD_SIZE = "field.size";
    public final String        FIELD_DATABASETYPE = "field.database-type";
    public final String        FIELD_SP_PARAM_TYPE = "field.sp-param-type";
    public final String        FIELD_SP_PARAM_INDEX = "field.sp-param-index";
    public final String        FIELD_JAVATYPE = FIELD + "." + JAVATYPE;
    public final String        FIELD_CONSTRAINT = FIELD + "." + CONSTRAINT;
    public final String        FIELD_FK_COLUMN = "field.foreignKeyColumn";
    public final String        FIELD_FK_GROUP = "field.foreignKeyGroup";
    public final String        DEFAULT = "default";

    // current values
    public final String        CACHE_NONE = "none";
    public final String        CACHE_PARTIAL = "partial";
    public final String        CACHE_FULL = "full";
    public final String        MICROSOFT = "Microsoft";
    public final String        STANDARD = "Standard";
    public final String        ABSTRACT = "abstract";
    public final String        FINAL = "final";
    public final String        DEFAULT_DATABASE_TYPE = "VARCHAR";
    public final String        DEFAULT_JAVA_TYPE = "String";
    public final static String BIG_DECIMAL_JAVA_TYPE = "java.math.BigDecimal";
    public final static String STRING_JAVA_TYPE = "String";
    public final static String BYTE_JAVA_TYPE = "byte";
    public final static String BYTE_ARRAY_JAVA_TYPE = "byte[]";
    public final String        DEFAULT_CLASS_VENDOR = "default";

    // values for sp-param-type
    public final static String SP_RETURN_VALUE	= "return";
    public final static String SP_IN_PARAM	= "in";
    public final static String SP_OUT_PARAM	= "out";
    public final static String SP_INOUT_PARAM	= "inout";
    public final static String SP_RESULTSET	= "resultset";

    // old keys
    public final String        V3_DATABASE = "database";
    public final String        V3_TABLE = "dbTableName";
    public final String        V3_ID = "id";
    public final String        V3_FINAL = "isFinal";
    public final String        V3_ABSTRACT = "isAbstract";
    public final String        V3_LAZY_LOADING = "isLazyLoading";
    public final String        V3_OID = "notUsingOId";
    public final String        V3_EXTENDS = "extensionOf";
    public final String        V3_INDEX = "isIndex";
    public final String        V3_PRIMARY = "isPrimaryKey";
    public final String        V3_CAN_QUERY = "usedForQuery";
    public final String        V3_CONSTANT = "isConstant";
    public final String        V3_DEFAULT = "initialValue";
    public final String        V3_TYPE = "type";
    public final String        V3_NULL_ALLOWED = "canBeNull";
    public final String        V3_DATABASE_TYPE = "dbType";
    public final String        V3_JAVA_TYPE = "javaType";
    public final String        V3_REFERENCE_OBJ = "referenceObject";
    public final String        V3_REFERENCE = "reference";

    public final String        V3_FK_COLUMN = "foreignKeyColumn";
    public final String        V3_FK_GROUP  = "foreignKeyGroup";

    // old values
    public final String        V3_MICROSOFT = "MSQL";
    public final String        V3_STANDARD = "Standard_JDBC";
}
