/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: DOMLUpdate.java,v 1.1 2001/09/28 16:01:07 cryd0221 Exp $
 */
package org.enhydra.dods.store.doml;

// DODS imports
import org.enhydra.dods.store.doa.DODSProject;
import org.enhydra.dods.common.FileUtil;

// Standard imports
import javax.swing.JOptionPane;
import java.awt.Component;
import java.io.File;
import java.io.FileReader;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.FileWriter;

/**
 * Class declaration
 *
 *
 * @author Lutris Technologies
 */
public class DOMLUpdate implements DOMLConstants {
    private File source;

    /**
     * Constructor declaration
     *
     *
     * @param in
     *
     * @see
     */
    public DOMLUpdate(File in) {
        source = in;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isCurrent() {
        boolean        current = false;
        int            index   = 0;
        FileReader     fileReader = null;
        BufferedReader bufReader = null;
        String         cursor = null;

        try {
            fileReader = new FileReader(source);
            bufReader = new BufferedReader(fileReader);
            cursor = bufReader.readLine();
            while (cursor != null) {
                cursor = bufReader.readLine();

                if (cursor == null) {
                    break;
                } else {
                   index = cursor.trim().indexOf(DOCTYPE.trim());
                   if (index > -1) {
                     current = true;
                     break;
                   }
                }
            }
            bufReader.close();
            fileReader.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return current;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @exception IOException
     *
     * @see
     */
    public File removeDocType() throws IOException {
        File           outFile = null;
        BufferedReader in = null;
        PrintWriter    out = null;
        String line = null;
        int index = -1;

        outFile = File.createTempFile("tmp", ".tmp");
        in = new BufferedReader(new FileReader(source));
        out = new PrintWriter(new FileWriter(outFile));
        line = in.readLine();

        while (line != null) {
            index = line.indexOf(DOCTYPE);
            if (index > -1) {
              line = line.substring(0, index) +
                     line.substring(index + DOCTYPE.length());
              line.trim();
              if (line.length() > 0) {
                out.println(line);
              }
            } else {
              out.println(line);
            }
            line = in.readLine();
        }
        in.close();
        out.close();
        outFile.deleteOnExit();
        return outFile;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public File convert(Component owner) throws IOException {
        File newFile = source;

        if (!isCurrent()) {
            File         dods3 = null;
            StringBuffer buf = new StringBuffer();
            int          index = -1;

            buf.append(source.getName());
            index = buf.toString().lastIndexOf(".");
            if (index > -1) {
                buf.replace(index, index + 5, ".dods3");
            } else {
                buf.append(".dods3");
            }
            dods3 = new File(source.getParentFile(), buf.toString());
            FileUtil.copyFile(source, dods3);
            DODSProject        project = null;
            DOMLProjectFactory projFactory = null;

            projFactory = new DOMLProjectFactory();
            try {
                project = projFactory.createProject(source);
                DOMLFactory.writeDOML(project, source, false);
            } catch (Exception e) {
                e.printStackTrace();
                JOptionPane.showMessageDialog(owner,
                                              "Unable to update file: \n"
                                              + source.getAbsolutePath(),
                                              "DOML Plugin for Merlot",
                                              JOptionPane.ERROR_MESSAGE);
            }
        }
        return source;
    }

}
