/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 * Omead Jensen
 */
package org.merlotxml.merlot;

// ToolBox
//import org.enhydra.tool.common.Browser;
//import org.enhydra.tool.common.PathHandle;
//import org.enhydra.tool.ToolBoxInfo;

// dods
import org.enhydra.dods.store.doml.DOMLUpdate;

// dods-tool;
import org.enhydra.dods.merlot.GeneratorDialog;
import org.merlotxml.merlot.XMLEditorFrame;

// JDK
import javax.swing.filechooser.FileFilter;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JMenu;
import javax.swing.event.MenuEvent;
import javax.swing.event.MenuListener;
import javax.swing.JOptionPane;
import javax.swing.Action;
import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.io.IOException;
import java.util.Vector;

//
public class DOMLPatch {
    private static String AFTER = MerlotResource.getString(MerlotConstants.UI,
            "node.popup.add.after");
    private static String BEFORE =
        MerlotResource.getString(MerlotConstants.UI, "node.popup.add.before");
    private static String SPECIAL =
        MerlotResource.getString(MerlotConstants.UI,
                                 "node.popup.add.special");

    private DOMLPatch() {}

    public static void patchMenu(JMenuBar menuBar) {
        int     count = menuBar.getMenuCount();
        JMenu[] menus = new JMenu[count];
        String  fileText = MerlotResource.getString(MerlotConstants.UI,
                                                    "file");
        String  plugText = MerlotResource.getString(MerlotConstants.UI,
                                                    "plugin");
        String  helpText = MerlotResource.getString(MerlotConstants.UI,
                                                    "help");
        String  currentText = new String();

        for (int i = 0; i < count; i++) {
            menus[i] = menuBar.getMenu(i);
        }
        for (int i = 0; i < count; i++) {
            currentText = menus[i].getText();
            if (currentText.equalsIgnoreCase(fileText)) {
                DOMLPatch.patchFileMenu(menus[i]);
            } else if (currentText.equalsIgnoreCase(plugText)) {
                menuBar.remove(menus[i]);
            } else if (currentText.equalsIgnoreCase(helpText)) {
                DOMLPatch.patchHelpMenu(menus[i]);
            }
        }
        DOMLPatch.patchInsertMenu(menuBar);
    }

    private static void patchInsertMenu(JMenuBar menuBar) {
        JMenu     insertMenu = new JMenu();
        JMenuItem itemVendor = new JMenuItem();
        JMenuItem itemPack = new JMenuItem();
        JMenuItem itemClass = new JMenuItem();
        JMenuItem itemField = new JMenuItem();
        JMenuItem itemJavaDoc = new JMenuItem();
        JMenuItem itemDefault = new JMenuItem();

        insertMenu.setText("Insert");
        insertMenu.addMenuListener(new MenuListener() {
            public void menuDeselected(MenuEvent event) {}

            public void menuCanceled(MenuEvent event) {}

            public void menuSelected(MenuEvent event) {
                JMenu          menu = (JMenu) event.getSource();
                JMenu          addMenu = null;
                JMenuItem[]    menuItems = new JMenuItem[0];
                JMenuItem[]    popItems = new JMenuItem[0];
                MerlotDOMNode  node = null;
                XMLEditorFrame frame = null;
                String         menuText = new String();
                Vector         popVector = new Vector();

                // create vector of desired items from nodes add popup
                if (menu.getTopLevelAncestor() instanceof XMLEditorFrame) {
                    frame = (XMLEditorFrame) menu.getTopLevelAncestor();
                    if (frame.getCurrentDocument() != null) {
                        node =
                            frame.getCurrentDocument()._ui.getSelectedNode();
                        if (node != null) {
                            addMenu =
                                frame.getCurrentDocument()._ui.getNodeAddMenu(node,
                                    MerlotConstants.INTO);
                            for (int i = 0; i < addMenu.getItemCount(); i++) {
                                if (addMenu.getItem(i) != null) {
                                    menuText = addMenu.getItem(i).getText();
                                    if (menuText.equalsIgnoreCase(AFTER)
                                            || menuText.equalsIgnoreCase(BEFORE)
                                            || menuText.equalsIgnoreCase(SPECIAL)) {

                                        // ignore
                                    } else {
                                        popVector.addElement(addMenu.getItem(i));
                                    }
                                }
                            }
                        }
                        popItems = (JMenuItem[]) popVector.toArray(popItems);
                        popVector.removeAllElements();
                    }
                }

                //
                // get array of items in current menu
                menuItems = new JMenuItem[menu.getMenuComponentCount()];
                for (int i = 0; i < menuItems.length; i++) {
                    menuItems[i] = (JMenuItem) menu.getMenuComponent(i);
                }
                for (int i = 0; i < menuItems.length; i++) {
                    menuItems[i].setEnabled(false);
                    for (int j = 0; j < popItems.length; j++) {
                        if (menuItems[i].getText().equals(popItems[j].getText())) {
                            menu.remove(menuItems[i]);
                            menu.insert(popItems[j], i);
                            break;
                        }
                    }
                }
            }

        });
        menuBar.add(insertMenu, 2);
        itemVendor.setText("vendor");
        itemPack.setText("package");
        itemClass.setText("class");
        itemField.setText("field");
        itemJavaDoc.setText("javadoc");
        itemDefault.setText("default");
        insertMenu.add(itemVendor);
        insertMenu.add(itemPack);
        insertMenu.add(itemClass);
        insertMenu.add(itemField);
        insertMenu.add(itemJavaDoc);
        insertMenu.add(itemDefault);
    }

    private static void showGeneratorErrorDialog(String msg) {
        JOptionPane.showMessageDialog(XMLEditorFrame.getSharedInstance(),
                                      msg,
                                      GeneratorDialog.ERROR_DIALOG_TITLE,
                                      JOptionPane.ERROR_MESSAGE);
    }

    private static void patchFileMenu(JMenu fileMenu) {
        String      openLibText = MerlotResource.getString(MerlotConstants.UI,
                "library.open");
        String      newLibText = MerlotResource.getString(MerlotConstants.UI,
                "library.new");
        final JMenu menu = fileMenu;
        JMenuItem   current = null;
        Component[] menuComps = fileMenu.getMenuComponents();
        JMenuItem   buildMenu = new JMenuItem();

        buildMenu.setText("Generate Java Source...");
        buildMenu.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                XMLEditorDoc doc =
                    XMLEditorFrame.getSharedInstance().getCurrentDocument();

                if (doc != null) {
                    GeneratorDialog dialog =
                        new GeneratorDialog((Frame) XMLEditorFrame.getSharedInstance());    // was: menu.getParent()x4

                    dialog.addWindowListener(new WindowAdapter() {
                        public void windowClosing(WindowEvent e) {
                            System.exit(0);
                        }

                    });
                } else {
                    DOMLPatch.showGeneratorErrorDialog(GeneratorDialog.NO_DOCUMENT_ERROR_MESSAGE);
                }
            }

        });
        for (int i = 0; i < menuComps.length; i++) {
            if (menuComps[i] instanceof JMenuItem) {
                current = (JMenuItem) menuComps[i];
                if (current.getText().equalsIgnoreCase(openLibText)) {
                    fileMenu.remove(current);
                    fileMenu.insert(buildMenu,
                                    i);    // OMEAD - comment this and uncomment next to remove from menu

                    // fileMenu.remove(menuComps[i-1]); // OMEAD - if removing Generate option, uncomment this to remove separator
                } else if (current.getText().equalsIgnoreCase(newLibText)) {
                    fileMenu.remove(current);
                }
            }
        }
    }

    private static void patchHelpMenu(JMenu fileMenu) {
        String      plugText = MerlotResource.getString(MerlotConstants.UI,
                "plugin.about");
        JMenuItem   current = null;
        JMenuItem   helpMenuItem = new JMenuItem();
        Component[] menuComps = fileMenu.getMenuComponents();

        helpMenuItem.setText("Browse Help");
        helpMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                browseHelp(event);
            }

        });
        for (int i = 0; i < menuComps.length; i++) {
            if (menuComps[i] instanceof JMenuItem) {
                current = (JMenuItem) menuComps[i];
                if (current.getText().equalsIgnoreCase(plugText)) {
                    fileMenu.remove(current);
                }
            }
        }
        fileMenu.insert(helpMenuItem, 0);
    }

    /**
     * ***
     * private String getExecutable() {
     *
     * SecurityManager sm = null;
     *
     * sm = System.getSecurityManager();   // security bug
     * System.setSecurityManager(null);    // security bug
     *
     * }
     * **
     */
    public static File update(File in, Component owner) {
        File out = in;

        if (in != null) {
            try {
                DOMLUpdate update = new DOMLUpdate(in);

                out = update.convert(owner);
            } catch (IOException e) {
                out = null;
                e.printStackTrace();
            }
        }
        return out;
    }

    public static FileFilter getDOMLFilter() {
        FileFilter filter = new FileFilter() {
            public String getDescription() {
                return "Enhydra DODS (*.doml)";
            }

            public boolean accept(File file) {
                boolean ok = false;

                if (file.exists()) {
                    if (file.isDirectory()) {
                        ok = true;
                    } else {
                        ok =
                            file.getAbsolutePath().toLowerCase().endsWith(".doml");
                    }
                }
                return ok;
            }

        };

        return filter;
    }

    private static void browseHelp(ActionEvent event) {
/*    
        StringBuffer buf = new StringBuffer();
        PathHandle   ph = null;
        Browser      browser = null;

        buf.append(ToolBoxInfo.getEnhydraRoot());
        buf.append("/dods/doc/DODS.html");
        ph = PathHandle.createPathHandle(buf.toString());
        if (ph.isFile()) {
            browser = new Browser();
            if (event.getSource() instanceof Component) {
                browser.setOwner((Component) event.getSource());
            }
            browser.open(ph.getPath());
        } else {
            System.err.println("Help not found: " + ph.getPath());
        }
*/
		//try and launch the browser    
        String url = "http://dods.enhydra.org/cvs_source/Dods";
		Process p = null;
		try {
			p = Runtime.getRuntime().exec("start "+url);
		} catch (Exception e) {}
		if (p==null)
		try {
			p = Runtime.getRuntime().exec("cmd /C \"start "+url+"\"");
		} catch (Exception e) {
			System.out.println("Error launching browser...invalid URL:"+url);
			System.out.println("Error details: "+e);
			e.printStackTrace();
		}
    }

}
